@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,14 +16,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "applicationdialog.h"
-#include <QVBoxLayout>
-#include <QPushButton>
-#include <QHBoxLayout>
-#include <QLabel>
+#include <QString>
+#include <QWidget>
+#include <QDialog>
 #include <QFileDialog>
 #include <QDebug>
 #include <QMessageBox>
+#include "applicationdialog.h"
 
 
 ApplicationDialog::ApplicationDialog(const QString &name,
