@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,9 +16,15 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "applicationlist.h"
 #include <QStringList>
+#include <QFileInfo>
+#include <QObject>
+#include <QSettings>
+#include <QStringList>
+#include <stdlib.h>
 #include "common.h"
+#include "applicationlist.h"
+
 
 ApplicationList::ApplicationList(QObject *parent) :
     QObject(parent)
@@ -37,6 +43,29 @@ void ApplicationList::LoadSettings(QSettings *programSettings)
     QStringList names = programSettings->value(SETTINGS_APPLICATION_NAMES, QStringList()).toStringList();
     QStringList paths = programSettings->value(SETTINGS_APPLICATION_PATHS, QStringList()).toStringList();
 
+    if (names.empty() && paths.empty())
+    {
+        do
+        {
+            // use as default for gnome environments
+            if (QFileInfo("/usr/bin/gedit").isExecutable())
+            {
+                AddApplicationType("gedit", "/usr/bin/gedit +(line) (file)");
+                break;
+            }
+            // use as default for kde environments
+            if (QFileInfo("/usr/bin/kate").isExecutable())
+            {
+                AddApplicationType("kate", "/usr/bin/kate -l(line) (file)");
+                break;
+            }
+            // use as default for windows environments
+            if (FindDefaultWindowsEditor())
+                break;
+        }
+        while (0);
+    }
+
     if (names.size() == paths.size())
     {
         for (int i = 0; i < names.size(); i++)
@@ -88,7 +117,6 @@ QString ApplicationList::GetApplicationPath(const int index) const
 
 }
 
-
 void ApplicationList::SetApplicationType(const int index,
         const QString &name,
         const QString &path)
@@ -118,7 +146,6 @@ void ApplicationList::RemoveApplication(const int index)
     mApplications.removeAt(index);
 }
 
-
 void ApplicationList::MoveFirst(const int index)
 {
     if (index < mApplications.size() && index > 0)
@@ -127,7 +154,6 @@ void ApplicationList::MoveFirst(const int index)
     }
 }
 
-
 void ApplicationList::Copy(ApplicationList *list)
 {
     if (!list)
@@ -147,3 +173,22 @@ void ApplicationList::Clear()
     mApplications.clear();
 }
 
+bool ApplicationList::FindDefaultWindowsEditor()
+{
+    const QString appPath(getenv("ProgramFiles"));
+    const QString notepadppPath = appPath + "\\Notepad++\\notepad++.exe";
+    if (QFileInfo(notepadppPath).isExecutable())
+    {
+        AddApplicationType("Notepad++", "\"" + notepadppPath + "\" -n(line) (file)");
+        return true;
+    }
+
+    const QString windowsPath(getenv("windir"));
+    const QString notepadPath = windowsPath + "\\system32\\notepad.exe";
+    if (QFileInfo(notepadPath).isExecutable())
+    {
+        AddApplicationType("Notepad", notepadPath + " (file)");
+        return true;
+    }
+    return false;
+}
