@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,9 +16,11 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-#include "checkthread.h"
+#include <QString>
 #include <QDebug>
+#include "checkthread.h"
+#include "threadresult.h"
+#include "cppcheck.h"
 
 CheckThread::CheckThread(ThreadResult &result) :
     mState(Ready),
@@ -33,7 +35,7 @@ CheckThread::~CheckThread()
     //dtor
 }
 
-void CheckThread::Check(Settings settings)
+void CheckThread::Check(const Settings &settings)
 {
     mCppcheck.settings(settings);
     start();
