@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,11 @@
 typedef enum
 {
     SHOW_STYLE = 0,
-    SHOW_ERRORS,
+    SHOW_WARNINGS,
+    SHOW_PERFORMANCE,
+    SHOW_PORTABILITY,
+    SHOW_INFORMATION,
+    SHOW_ERRORS, // Keep this as last real item
     SHOW_NONE
 }
 ShowTypes;
@@ -38,26 +42,44 @@ ShowTypes;
 /**
 * QSetting value names
 */
+
+// Window/dialog sizes
 #define SETTINGS_WINDOW_MAXIMIZED       "Window maximized"
 #define SETTINGS_WINDOW_WIDTH           "Window width"
 #define SETTINGS_WINDOW_HEIGHT          "Window height"
+#define SETTINGS_LOG_VIEW_WIDTH         "Log/View width"
+#define SETTINGS_LOG_VIEW_HEIGHT        "Log/View height"
+#define SETTINGS_MAINWND_SPLITTER_STATE "Mainwindow/Vertical splitter state"
+#define SETTINGS_CHECK_DIALOG_WIDTH     "Check dialog width"
+#define SETTINGS_CHECK_DIALOG_HEIGHT    "Check dialog height"
+
+// Main window settings
+#define SETTINGS_RESULT_COLUMN_WIDTH    "Result column %1 width"
+#define SETTINGS_TOOLBARS_MAIN_SHOW     "Toolbars/ShowStandard"
+#define SETTINGS_TOOLBARS_VIEW_SHOW     "Toolbars/ShowView"
+
+// Show * states
 #define SETTINGS_SHOW_STYLE             "Show style"
 #define SETTINGS_SHOW_ERRORS            "Show errors"
+#define SETTINGS_SHOW_WARNINGS          "Show warnings"
+#define SETTINGS_SHOW_PERFORMANCE       "Show performance"
+#define SETTINGS_SHOW_INFORMATION       "Show information"
+#define SETTINGS_SHOW_PORTABILITY       "Show portability"
+
+// Other settings
 #define SETTINGS_CHECK_PATH             "Check path"
 #define SETTINGS_CHECK_FORCE            "Check force"
 #define SETTINGS_CHECK_THREADS          "Check threads"
 #define SETTINGS_SHOW_FULL_PATH         "Show full path"
 #define SETTINGS_SHOW_NO_ERRORS         "Show no errors message"
+#define SETTINGS_SHOW_DEBUG_WARNINGS    "Show debug warnings"
 #define SETTINGS_SAVE_ALL_ERRORS        "Save all errors"
 #define SETTINGS_SAVE_FULL_PATH         "Save full path"
-#define SETTINGS_CHECK_DIALOG_WIDTH     "Check dialog width"
-#define SETTINGS_CHECK_DIALOG_HEIGHT    "Check dialog height"
 #define SETTINGS_APPLICATION_NAMES      "Application names"
 #define SETTINGS_APPLICATION_PATHS      "Application paths"
-#define SETTINGS_RESULT_COLUMN_WIDTH    "Result column %1 width"
 #define SETTINGS_LANGUAGE               "Application language"
-#define SETTINGS_TOOLBARS_MAIN_SHOW     "Toolbars/ShowStandard"
-#define SETTINGS_TOOLBARS_VIEW_SHOW     "Toolbars/ShowView"
+#define SETTINGS_GLOBAL_INCLUDE_PATHS   "Global include paths"
+#define SETTINGS_INLINE_SUPPRESSIONS    "Inline suppressions"
 
 /// @}
 #endif
