@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,18 @@
 #include <QApplication>
 #include <QTextCodec>
 #include <QTranslator>
+#include <QMetaType>
 #include "mainwindow.h"
+#include "erroritem.h"
 
 int main(int argc, char *argv[])
 {
     QApplication app(argc, argv);
     app.setWindowIcon(QIcon(":icon.png"));
 
+    // Register this metatype that is used to transfer error info
+    qRegisterMetaType<ErrorItem>("ErrorItem");
+
     // Set codecs so that UTF-8 strings in sources are handled correctly.
     QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));
     QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
