@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,38 +31,49 @@
 #include "threadhandler.h"
 #include "fileviewdialog.h"
 #include "projectfile.h"
+#include "project.h"
 #include "report.h"
-#include "../lib/filelister.h"
-
-// HTMLHelp is only available in Windows
-#ifdef WIN32
-#include <windows.h>
-#include <htmlhelp.h>
-#endif
+#include "statsdialog.h"
+#include "logview.h"
+#include "filelist.h"
+#include "helpwindow.h"
 
 MainWindow::MainWindow() :
     mSettings(new QSettings("Cppcheck", "Cppcheck-GUI", this)),
     mApplications(new ApplicationList(this)),
     mTranslation(new TranslationHandler(this)),
-    mLanguages(new QActionGroup(this))
+    mLanguages(new QActionGroup(this)),
+    mLogView(NULL),
+    mHelpWindow(NULL),
+    mProject(NULL),
+    mExiting(false)
 {
     mUI.setupUi(this);
     mUI.mResults->Initialize(mSettings, mApplications);
 
     mThread = new ThreadHandler(this);
+    mLogView = new LogView(mSettings);
 
     connect(mUI.mActionQuit, SIGNAL(triggered()), this, SLOT(close()));
     connect(mUI.mActionCheckFiles, SIGNAL(triggered()), this, SLOT(CheckFiles()));
     connect(mUI.mActionCheckDirectory, SIGNAL(triggered()), this, SLOT(CheckDirectory()));
     connect(mUI.mActionSettings, SIGNAL(triggered()), this, SLOT(ProgramSettings()));
     connect(mUI.mActionClearResults, SIGNAL(triggered()), this, SLOT(ClearResults()));
+    connect(mUI.mActionOpenXML, SIGNAL(triggered()), this, SLOT(OpenXML()));
 
     connect(mUI.mActionShowStyle, SIGNAL(toggled(bool)), this, SLOT(ShowStyle(bool)));
     connect(mUI.mActionShowErrors, SIGNAL(toggled(bool)), this, SLOT(ShowErrors(bool)));
+    connect(mUI.mActionShowWarnings, SIGNAL(toggled(bool)), this, SLOT(ShowWarnings(bool)));
+    connect(mUI.mActionShowPortability, SIGNAL(toggled(bool)), this, SLOT(ShowPortability(bool)));
+    connect(mUI.mActionShowPerformance, SIGNAL(toggled(bool)), this, SLOT(ShowPerformance(bool)));
+    connect(mUI.mActionShowInformation, SIGNAL(toggled(bool)), this, SLOT(ShowInformation(bool)));
     connect(mUI.mActionCheckAll, SIGNAL(triggered()), this, SLOT(CheckAll()));
     connect(mUI.mActionUncheckAll, SIGNAL(triggered()), this, SLOT(UncheckAll()));
     connect(mUI.mActionCollapseAll, SIGNAL(triggered()), mUI.mResults, SLOT(CollapseAllResults()));
     connect(mUI.mActionExpandAll, SIGNAL(triggered()), mUI.mResults, SLOT(ExpandAllResults()));
+    connect(mUI.mActionShowHidden, SIGNAL(triggered()), mUI.mResults, SLOT(ShowHiddenResults()));
+    connect(mUI.mActionViewLog, SIGNAL(triggered()), this, SLOT(ShowLogView()));
+    connect(mUI.mActionViewStats, SIGNAL(triggered()), this, SLOT(ShowStatistics()));
 
     connect(mUI.mActionRecheck, SIGNAL(triggered()), this, SLOT(ReCheck()));
 
@@ -77,16 +88,16 @@ MainWindow::MainWindow() :
     connect(mUI.mActionAuthors, SIGNAL(triggered()), this, SLOT(ShowAuthors()));
     connect(mThread, SIGNAL(Done()), this, SLOT(CheckDone()));
     connect(mUI.mResults, SIGNAL(GotResults()), this, SLOT(ResultsAdded()));
+    connect(mUI.mResults, SIGNAL(ResultsHidden(bool)), mUI.mActionShowHidden, SLOT(setEnabled(bool)));
     connect(mUI.mMenuView, SIGNAL(aboutToShow()), this, SLOT(AboutToShowViewMenu()));
 
-#ifdef WIN32
+    connect(mUI.mActionNewProjectFile, SIGNAL(triggered()), this, SLOT(NewProjectFile()));
+    connect(mUI.mActionOpenProjectFile, SIGNAL(triggered()), this, SLOT(OpenProjectFile()));
+    connect(mUI.mActionCloseProjectFile, SIGNAL(triggered()), this, SLOT(CloseProjectFile()));
+    connect(mUI.mActionEditProjectFile, SIGNAL(triggered()), this, SLOT(EditProjectFile()));
+
     connect(mUI.mActionHelpContents, SIGNAL(triggered()), this, SLOT(OpenHelpContents()));
-#else
-    // Hide if not Windows
-    mUI.mActionHelpContents->setVisible(false);
-#endif
 
-    CreateLanguageMenuItems();
     LoadSettings();
 
     mThread->Initialize(mUI.mResults);
@@ -97,6 +108,8 @@ MainWindow::MainWindow() :
     mUI.mActionClearResults->setEnabled(false);
     mUI.mActionSave->setEnabled(false);
     mUI.mActionRecheck->setEnabled(false);
+    EnableProjectOpenActions(true);
+    EnableProjectActions(false);
 
     QStringList args = QCoreApplication::arguments();
     //Remove the application itself
@@ -109,39 +122,9 @@ MainWindow::MainWindow() :
 
 MainWindow::~MainWindow()
 {
-}
-
-void MainWindow::CreateLanguageMenuItems()
-{
-    QStringList languages = mTranslation->GetNames();
-
-    for (int i = 0; i < languages.size(); i++)
-    {
-        //Create an action for each language
-        //Language name is pre translated
-        QAction *temp = new QAction(languages[i], this);
-
-        temp->setCheckable(true);
-
-        //Add the action to menu
-        mUI.mMenuLanguage->addAction(temp);
-
-        //Add action to the group
-        mLanguages->addAction(temp);
-
-        //Check it if it's the value stored to settings
-        if (i == mSettings->value(SETTINGS_LANGUAGE, 0).toInt())
-        {
-            temp->setChecked(true);
-        }
-        else
-        {
-            temp->setChecked(false);
-        }
-    }
-
-    connect(mLanguages, SIGNAL(triggered(QAction *)),
-            this, SLOT(MapLanguage(QAction *)));
+    delete mLogView;
+    delete mHelpWindow;
+    delete mProject;
 }
 
 void MainWindow::LoadSettings()
@@ -158,6 +141,10 @@ void MainWindow::LoadSettings()
 
     mUI.mActionShowStyle->setChecked(mSettings->value(SETTINGS_SHOW_STYLE, true).toBool());
     mUI.mActionShowErrors->setChecked(mSettings->value(SETTINGS_SHOW_ERRORS, true).toBool());
+    mUI.mActionShowWarnings->setChecked(mSettings->value(SETTINGS_SHOW_WARNINGS, true).toBool());
+    mUI.mActionShowPortability->setChecked(mSettings->value(SETTINGS_SHOW_PORTABILITY, true).toBool());
+    mUI.mActionShowPerformance->setChecked(mSettings->value(SETTINGS_SHOW_PERFORMANCE, true).toBool());
+    mUI.mActionShowInformation->setChecked(mSettings->value(SETTINGS_SHOW_INFORMATION, true).toBool());
 
     mUI.mResults->ShowResults(SHOW_ERRORS, mUI.mActionShowErrors->isChecked());
     mUI.mResults->ShowResults(SHOW_STYLE, mUI.mActionShowStyle->isChecked());
@@ -182,13 +169,17 @@ void MainWindow::SaveSettings()
 
     mSettings->setValue(SETTINGS_SHOW_STYLE, mUI.mActionShowStyle->isChecked());
     mSettings->setValue(SETTINGS_SHOW_ERRORS, mUI.mActionShowErrors->isChecked());
+    mSettings->setValue(SETTINGS_SHOW_WARNINGS, mUI.mActionShowWarnings->isChecked());
+    mSettings->setValue(SETTINGS_SHOW_PORTABILITY, mUI.mActionShowPortability->isChecked());
+    mSettings->setValue(SETTINGS_SHOW_PERFORMANCE, mUI.mActionShowPerformance->isChecked());
+    mSettings->setValue(SETTINGS_SHOW_INFORMATION, mUI.mActionShowInformation->isChecked());
     mSettings->setValue(SETTINGS_TOOLBARS_MAIN_SHOW, mUI.mToolBarMain->isVisible());
     mSettings->setValue(SETTINGS_TOOLBARS_VIEW_SHOW, mUI.mToolBarView->isVisible());
 
     mApplications->SaveSettings(mSettings);
 
     mSettings->setValue(SETTINGS_LANGUAGE, mTranslation->GetCurrentLanguage());
-    mUI.mResults->SaveSettings();
+    mUI.mResults->SaveSettings(mSettings);
 }
 
 void MainWindow::DoCheckFiles(const QStringList &files)
@@ -199,13 +190,9 @@ void MainWindow::DoCheckFiles(const QStringList &files)
     }
     ClearResults();
 
-    QStringList fileNames;
-    QString selection;
-
-    foreach(selection, files)
-    {
-        fileNames << RemoveUnacceptedFiles(GetFilesRecursively(selection));
-    }
+    FileList pathList;
+    pathList.AddPathList(files);
+    QStringList fileNames = pathList.GetFileList();
 
     mUI.mResults->Clear();
     mThread->ClearFiles();
@@ -223,21 +210,34 @@ void MainWindow::DoCheckFiles(const QStringList &files)
 
     mUI.mResults->CheckingStarted(fileNames.count());
 
-    mThread->SetFiles(RemoveUnacceptedFiles(fileNames));
-    QFileInfo inf(fileNames[0]);
-    QString absDirectory = inf.absoluteDir().path();
-    mSettings->setValue(SETTINGS_CHECK_PATH, absDirectory);
+    mThread->SetFiles(fileNames);
+    QDir inf(mCurrentDirectory);
+    const QString checkPath = inf.canonicalPath();
+    mSettings->setValue(SETTINGS_CHECK_PATH, checkPath);
     EnableCheckButtons(false);
     mUI.mActionSettings->setEnabled(false);
-
-    mUI.mResults->SetCheckDirectory(absDirectory);
+    mUI.mActionOpenXML->setEnabled(false);
+    mUI.mResults->SetCheckDirectory(checkPath);
 
     Settings checkSettings = GetCppcheckSettings();
+    EnableProjectActions(false);
+    EnableProjectOpenActions(false);
     mThread->Check(checkSettings, false);
 }
 
 QStringList MainWindow::SelectFilesToCheck(QFileDialog::FileMode mode)
 {
+    if (mProject)
+    {
+        QMessageBox msgBox(this);
+        msgBox.setWindowTitle(tr("Cppcheck"));
+        const QString msg(tr("You must close the project file before selecting new files or directories!"));
+        msgBox.setText(msg);
+        msgBox.setIcon(QMessageBox::Critical);
+        msgBox.exec();
+        return QStringList();
+    }
+
     QStringList selected;
 
     // NOTE: we use QFileDialog::getOpenFileNames() and
@@ -264,6 +264,7 @@ QStringList MainWindow::SelectFilesToCheck(QFileDialog::FileMode mode)
                       mSettings->value(SETTINGS_CHECK_PATH, "").toString());
         if (!dir.isEmpty())
         {
+            qDebug() << "Setting current directory to: " << dir;
             mCurrentDirectory = dir;
             selected.append(dir);
             dir = QDir::toNativeSeparators(dir);
@@ -284,60 +285,88 @@ void MainWindow::CheckDirectory()
     DoCheckFiles(SelectFilesToCheck(QFileDialog::DirectoryOnly));
 }
 
-Settings MainWindow::GetCppcheckSettings()
+bool MainWindow::GetCheckProject()
 {
-    ProjectFile pfile;
-    Settings result;
+    // We have successfully loaded project earlier and use that project
+    if (mProject)
+        return true;
 
     if (!mCurrentDirectory.isEmpty())
     {
         // Format project filename (directory name + .cppcheck) and load
         // the project file if it is found.
         QStringList parts = mCurrentDirectory.split("/");
-        QString projfile = mCurrentDirectory + "/" + parts[parts.count() - 1] + ".cppcheck";
-        bool projectRead = false;
+        const QString filename = parts[parts.count() - 1] + ".cppcheck";;
+        const QString projfile = mCurrentDirectory + "/" + filename;
         if (QFile::exists(projfile))
         {
-            qDebug() << "Reading project file " << projfile;
-            projectRead = pfile.Read(projfile);
+            qDebug() << "Opening project file: " << projfile;
+            mProject = new Project();
+            mProject->SetFilename(projfile);
+            FormatAndSetTitle(tr("Project: ") + QString(" ") + filename);
+            return mProject->Open();
         }
+    }
+    return false;
+}
 
-        if (projectRead)
+void MainWindow::AddIncludeDirs(const QStringList &includeDirs, Settings &result)
+{
+    QString dir;
+    foreach(dir, includeDirs)
+    {
+        QString incdir;
+        if (!QDir::isAbsolutePath(dir))
+            incdir = mCurrentDirectory + "/";
+        incdir += dir;
+        incdir = QDir::cleanPath(incdir);
+
+        // include paths must end with '/'
+        if (!incdir.endsWith("/"))
+            incdir += "/";
+        result._includePaths.push_back(incdir.toStdString());
+    }
+}
+
+Settings MainWindow::GetCppcheckSettings()
+{
+    Settings result;
+
+    QString globalIncludes = mSettings->value(SETTINGS_GLOBAL_INCLUDE_PATHS).toString();
+    if (!globalIncludes.isEmpty())
+    {
+        QStringList includes = globalIncludes.split(";");
+        AddIncludeDirs(includes, result);
+    }
+
+    bool projectRead = GetCheckProject();
+    if (projectRead)
+    {
+        ProjectFile *pfile = mProject->GetProjectFile();
+        QStringList dirs = pfile->GetIncludeDirs();
+        AddIncludeDirs(dirs, result);
+
+        QStringList defines = pfile->GetDefines();
+        QString define;
+        foreach(define, defines)
         {
-            QStringList dirs = pfile.GetIncludeDirs();
-            QString dir;
-            foreach(dir, dirs)
-            {
-                QString incdir;
-                if (!QDir::isAbsolutePath(dir))
-                    incdir = mCurrentDirectory + "/";
-                incdir += dir;
-                incdir = QDir::cleanPath(incdir);
-
-                // include paths must end with '/'
-                if (!incdir.endsWith("/"))
-                    incdir += "/";
-                result._includePaths.push_back(incdir.toStdString());
-            }
-
-            QStringList defines = pfile.GetDefines();
-            QString define;
-            foreach(define, defines)
-            {
-                if (!result.userDefines.empty())
-                    result.userDefines += ";";
-                result.userDefines += define.toStdString();
-            }
+            if (!result.userDefines.empty())
+                result.userDefines += ";";
+            result.userDefines += define.toStdString();
         }
     }
 
-    result._debug = false;
-    result._checkCodingStyle = true;
+    result.addEnabled("style");
+    result.addEnabled("information");
+    result.addEnabled("missingInclude");
+    result.debug = false;
+    result.debugwarnings = mSettings->value(SETTINGS_SHOW_DEBUG_WARNINGS, false).toBool();
     result._errorsOnly = false;
     result._verbose = true;
     result._force = mSettings->value(SETTINGS_CHECK_FORCE, 1).toBool();
     result._xml = false;
     result._jobs = mSettings->value(SETTINGS_CHECK_THREADS, 1).toInt();
+    result._inlineSuppressions = mSettings->value(SETTINGS_INLINE_SUPPRESSIONS, false).toBool();
 
     if (result._jobs <= 0)
     {
@@ -347,48 +376,21 @@ Settings MainWindow::GetCppcheckSettings()
     return result;
 }
 
-QStringList MainWindow::GetFilesRecursively(const QString &path)
-{
-    QFileInfo info(path);
-    QStringList list;
-
-    if (info.isDir())
-    {
-        QDirIterator it(path, QDirIterator::Subdirectories);
-
-        while (it.hasNext())
-        {
-            list << it.next();
-        }
-    }
-    else
-    {
-        list << path;
-    }
-
-    return list;
-}
-
-QStringList MainWindow::RemoveUnacceptedFiles(const QStringList &list)
-{
-    QStringList result;
-    QString str;
-    foreach(str, list)
-    {
-        if (getFileLister()->acceptFile(str.toStdString()))
-        {
-            result << str;
-        }
-    }
-
-    return result;
-}
-
 void MainWindow::CheckDone()
 {
+    if (mExiting)
+    {
+        close();
+        return;
+    }
+
     mUI.mResults->CheckingFinished();
     EnableCheckButtons(true);
     mUI.mActionSettings->setEnabled(true);
+    mUI.mActionOpenXML->setEnabled(true);
+    EnableProjectActions(true);
+    EnableProjectOpenActions(true);
+
     if (mUI.mResults->HasResults())
     {
         mUI.mActionClearResults->setEnabled(true);
@@ -401,14 +403,18 @@ void MainWindow::CheckDone()
 
 void MainWindow::ProgramSettings()
 {
-    SettingsDialog dialog(mSettings, mApplications, this);
+    SettingsDialog dialog(mSettings, mApplications, mTranslation, this);
     if (dialog.exec() == QDialog::Accepted)
     {
-        dialog.SaveCheckboxValues();
+        dialog.SaveSettingValues();
         mUI.mResults->UpdateSettings(dialog.ShowFullPath(),
                                      dialog.SaveFullPath(),
                                      dialog.SaveAllErrors(),
                                      dialog.ShowNoErrorsMessage());
+        const int currentLang = mTranslation->GetCurrentLanguage();
+        const int newLang = mSettings->value(SETTINGS_LANGUAGE, 0).toInt();
+        if (currentLang != newLang)
+            SetLanguage(newLang);
     }
 }
 
@@ -431,6 +437,22 @@ void MainWindow::ClearResults()
     mUI.mActionSave->setEnabled(false);
 }
 
+void MainWindow::OpenXML()
+{
+    QString selectedFilter;
+    QString filter(tr("XML files (*.xml)"));
+    QString selectedFile = QFileDialog::getOpenFileName(this,
+                           tr("Open the report file"),
+                           QString(),
+                           filter,
+                           &selectedFilter);
+
+    if (!selectedFile.isEmpty())
+    {
+        mUI.mResults->ReadErrorsXml(selectedFile);
+    }
+}
+
 void MainWindow::EnableCheckButtons(bool enable)
 {
     mUI.mActionStop->setEnabled(!enable);
@@ -452,6 +474,26 @@ void MainWindow::ShowErrors(bool checked)
     mUI.mResults->ShowResults(SHOW_ERRORS, checked);
 }
 
+void MainWindow::ShowWarnings(bool checked)
+{
+    mUI.mResults->ShowResults(SHOW_WARNINGS, checked);
+}
+
+void MainWindow::ShowPortability(bool checked)
+{
+    mUI.mResults->ShowResults(SHOW_PORTABILITY, checked);
+}
+
+void MainWindow::ShowPerformance(bool checked)
+{
+    mUI.mResults->ShowResults(SHOW_PERFORMANCE, checked);
+}
+
+void MainWindow::ShowInformation(bool checked)
+{
+    mUI.mResults->ShowResults(SHOW_INFORMATION, checked);
+}
+
 void MainWindow::CheckAll()
 {
     ToggleAllChecked(true);
@@ -467,21 +509,33 @@ void MainWindow::closeEvent(QCloseEvent *event)
     // Check that we aren't checking files
     if (!mThread->IsChecking())
     {
+        delete mHelpWindow;
+        mHelpWindow = NULL;
+
         SaveSettings();
         event->accept();
     }
     else
     {
-        QString text(tr("Cannot exit while checking.\n\n" \
-                        "Stop the checking before exiting."));
+        QString text(tr("Checking is running.\n\n" \
+                        "Do you want to stop the checking and exit Cppcheck?."));
 
         QMessageBox msg(QMessageBox::Warning,
                         tr("Cppcheck"),
                         text,
-                        QMessageBox::Ok,
+                        QMessageBox::Yes | QMessageBox::No,
                         this);
 
-        msg.exec();
+        msg.setDefaultButton(QMessageBox::No);
+        int rv = msg.exec();
+        if (rv == QMessageBox::Yes)
+        {
+            // This isn't really very clean way to close threads but since the app is
+            // exiting it doesn't matter.
+            mThread->Stop();
+            SaveSettings();
+            mExiting = true;
+        }
         event->ignore();
     }
 }
@@ -490,9 +544,16 @@ void MainWindow::ToggleAllChecked(bool checked)
 {
     mUI.mActionShowStyle->setChecked(checked);
     ShowStyle(checked);
-
     mUI.mActionShowErrors->setChecked(checked);
     ShowErrors(checked);
+    mUI.mActionShowWarnings->setChecked(checked);
+    ShowWarnings(checked);
+    mUI.mActionShowPortability->setChecked(checked);
+    ShowPortability(checked);
+    mUI.mActionShowPerformance->setChecked(checked);
+    ShowPerformance(checked);
+    mUI.mActionShowInformation->setChecked(checked);
+    ShowInformation(checked);
 }
 
 void MainWindow::About()
@@ -518,7 +579,7 @@ void MainWindow::ShowAuthors()
 void MainWindow::Save()
 {
     QString selectedFilter;
-    QString filter(tr("XML files (*.xml);;Text files (*.txt);;CSV files (*.csv)"));
+    QString filter(tr("XML files version 2 (*.xml);;XML files version 1 (*.xml);;Text files (*.txt);;CSV files (*.csv)"));
     QString selectedFile = QFileDialog::getSaveFileName(this,
                            tr("Save the report file"),
                            QString(),
@@ -528,12 +589,18 @@ void MainWindow::Save()
     if (!selectedFile.isEmpty())
     {
         Report::Type type = Report::TXT;
-        if (selectedFilter == tr("XML files (*.xml)"))
+        if (selectedFilter == tr("XML files version 1 (*.xml)"))
         {
             type = Report::XML;
             if (!selectedFile.endsWith(".xml", Qt::CaseInsensitive))
                 selectedFile += ".xml";
         }
+        else if (selectedFilter == tr("XML files version 2 (*.xml)"))
+        {
+            type = Report::XMLV2;
+            if (!selectedFile.endsWith(".xml", Qt::CaseInsensitive))
+                selectedFile += ".xml";
+        }
         else if (selectedFilter == tr("Text files (*.txt)"))
         {
             type = Report::TXT;
@@ -621,19 +688,6 @@ void MainWindow::SetLanguage(int index)
     }
 }
 
-void MainWindow::MapLanguage(QAction *action)
-{
-    //Find the action that has the language that user clicked
-    QList<QAction *> actions = mLanguages->actions();
-    for (int i = 0; i < actions.size(); i++)
-    {
-        if (actions[i] == action)
-        {
-            SetLanguage(i);
-        }
-    }
-}
-
 void MainWindow::AboutToShowViewMenu()
 {
     mUI.mActionToolBarMain->setChecked(mUI.mToolBarMain->isVisible());
@@ -653,11 +707,174 @@ void MainWindow::OpenHelpContents()
 
 void MainWindow::OpenHtmlHelpContents()
 {
-#ifdef WIN32
-    QString file("/cppcheck.chm");
-    QString exeFolder = QDir::currentPath();
-    exeFolder += file;
-    exeFolder = QDir::toNativeSeparators(exeFolder);
-    HtmlHelp(NULL, exeFolder.utf16(), HH_DISPLAY_TOPIC, NULL);
-#endif // WIN32
+    if (mHelpWindow == NULL)
+    {
+        const QString fname = qApp->applicationDirPath() + "/online-help.qhc";
+        if (!QFileInfo(fname).exists())
+        {
+            QMessageBox::warning(this, tr("Cppcheck Help"), tr("Failed to load help file (not found)"));
+            return;
+        }
+
+        mHelpWindow = new HelpWindow;
+        if (!mHelpWindow->load(fname))
+        {
+            delete mHelpWindow;
+            mHelpWindow = NULL;
+            QMessageBox::warning(this, tr("Cppcheck Help"), tr("Failed to load help file"));
+            return;
+        }
+    }
+
+    mHelpWindow->show();
+    if (!mHelpWindow->isActiveWindow())
+        mHelpWindow->activateWindow();
+}
+
+void MainWindow::OpenProjectFile()
+{
+    delete mProject;
+
+    const QString filter = tr("Project files (*.cppcheck);;All files(*.*)");
+    QString filepath = QFileDialog::getOpenFileName(this,
+                       tr("Select Project File"),
+                       QString(),
+                       filter);
+
+    if (!filepath.isEmpty())
+    {
+        QFileInfo inf(filepath);
+        const QString filename = inf.fileName();
+        FormatAndSetTitle(tr("Project: ") + QString(" ") + filename);
+
+        mUI.mActionCloseProjectFile->setEnabled(true);
+        mUI.mActionEditProjectFile->setEnabled(true);
+        mProject = new Project(filepath, this);
+        mProject->Open();
+        QString rootpath = mProject->GetProjectFile()->GetRootPath();
+
+        // If root path not give or "current dir" then use project file's directory
+        // as check path
+        if (rootpath.isEmpty() || rootpath == ".")
+            mCurrentDirectory = inf.canonicalPath();
+        else
+            mCurrentDirectory = rootpath;
+
+        QStringList paths = mProject->GetProjectFile()->GetCheckPaths();
+        if (!paths.isEmpty())
+        {
+            for (int i = 0; i < paths.size(); i++)
+            {
+                if (!QDir::isAbsolutePath(paths[i]))
+                {
+                    QString path = mCurrentDirectory + "/";
+                    path += paths[i];
+                    paths[i] = QDir::cleanPath(path);
+                }
+            }
+
+            DoCheckFiles(paths);
+        }
+    }
+}
+
+void MainWindow::NewProjectFile()
+{
+    const QString filter = tr("Project files (*.cppcheck);;All files(*.*)");
+    QString filepath = QFileDialog::getSaveFileName(this,
+                       tr("Select Project Filename"),
+                       QString(),
+                       filter);
+
+    if (!filepath.isEmpty())
+    {
+        EnableProjectActions(true);
+        QFileInfo inf(filepath);
+        const QString filename = inf.fileName();
+        FormatAndSetTitle(tr("Project: ") + QString(" ") + filename);
+
+        delete mProject;
+        mProject = new Project(filepath, this);
+        mProject->Create();
+        mProject->Edit();
+    }
+}
+
+void MainWindow::CloseProjectFile()
+{
+    delete mProject;
+    mProject = NULL;
+    EnableProjectActions(false);
+    EnableProjectOpenActions(true);
+    FormatAndSetTitle();
+}
+
+void MainWindow::EditProjectFile()
+{
+    if (!mProject)
+    {
+        QMessageBox msg(QMessageBox::Critical,
+                        tr("Cppcheck"),
+                        QString(tr("No project file loaded")),
+                        QMessageBox::Ok,
+                        this);
+        msg.exec();
+        return;
+    }
+    mProject->Edit();
+}
+
+void MainWindow::ShowLogView()
+{
+    if (mLogView == NULL)
+        mLogView = new LogView(mSettings);
+
+    mLogView->show();
+    if (!mLogView->isActiveWindow())
+        mLogView->activateWindow();
+}
+
+void MainWindow::ShowStatistics()
+{
+    StatsDialog statsDialog(this);
+
+    // Show a dialog with the previous scan statistics and project information
+    if (mProject)
+    {
+        statsDialog.setProject(*mProject);
+    }
+    statsDialog.setPathSelected(mCurrentDirectory);
+    statsDialog.setNumberOfFilesScanned(mThread->GetPreviousFilesCount());
+    statsDialog.setScanDuration(mThread->GetPreviousScanDuration() / 1000.0);
+    statsDialog.setStatistics(mUI.mResults->GetStatistics());
+
+    statsDialog.exec();
+}
+
+void MainWindow::Log(const QString &logline)
+{
+    if (mLogView)
+    {
+        mLogView->AppendLine(logline);
+    }
+}
+
+void MainWindow::DebugError(const ErrorItem &item)
+{
+    if (mLogView)
+    {
+        mLogView->AppendLine(item.ToString());
+    }
+}
+
+void MainWindow::EnableProjectActions(bool enable)
+{
+    mUI.mActionCloseProjectFile->setEnabled(enable);
+    mUI.mActionEditProjectFile->setEnabled(enable);
+}
+
+void MainWindow::EnableProjectOpenActions(bool enable)
+{
+    mUI.mActionNewProjectFile->setEnabled(enable);
+    mUI.mActionOpenProjectFile->setEnabled(enable);
 }
