@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,20 @@
 #include <QSignalMapper>
 #include <QActionGroup>
 #include <QToolBar>
+#include <QStringList>
 
 #include "resultsview.h"
 #include "settingsdialog.h"
 #include "translationhandler.h"
-
+#include "settings.h"
 #include "ui_main.h"
+
 class ThreadHandler;
+class LogView;
+class HelpWindow;
+class Project;
+class ErrorItem;
+class StatsDialog;
 
 /// @addtogroup GUI
 /// @{
@@ -67,6 +74,12 @@ public slots:
     */
     void ClearResults();
 
+    /**
+    * @brief Slot to open XML report file
+    *
+    */
+    void OpenXML();
+
     /**
     * @brief Show errors with type "style"
     * @param checked Should errors be shown (true) or hidden (false)
@@ -79,6 +92,30 @@ public slots:
     */
     void ShowErrors(bool checked);
 
+    /**
+    * @brief Show errors with type "warning"
+    * @param checked Should errors be shown (true) or hidden (false)
+    */
+    void ShowWarnings(bool checked);
+
+    /**
+    * @brief Show errors with type "portability"
+    * @param checked Should errors be shown (true) or hidden (false)
+    */
+    void ShowPortability(bool checked);
+
+    /**
+    * @brief Show errors with type "performance"
+    * @param checked Should errors be shown (true) or hidden (false)
+    */
+    void ShowPerformance(bool checked);
+
+    /**
+    * @brief Show errors with type "information"
+    * @param checked Should errors be shown (true) or hidden (false)
+    */
+    void ShowInformation(bool checked);
+
     /**
     * @brief Slot to check all "Show errors" menu items
     */
@@ -125,6 +162,42 @@ public slots:
     */
     void Save();
 
+    /**
+    * @brief Slot to create new project file..
+    *
+    */
+    void NewProjectFile();
+
+    /**
+    * @brief Slot to open project file and start checking contained paths.
+    *
+    */
+    void OpenProjectFile();
+
+    /**
+    * @brief Slot to close open project file.
+    *
+    */
+    void CloseProjectFile();
+
+    /**
+    * @brief Slot to edit project file.
+    *
+    */
+    void EditProjectFile();
+
+    /**
+    * @brief Slot for showing the log view.
+    *
+    */
+    void ShowLogView();
+
+    /**
+    * @brief Slot for showing the scan and project statistics.
+    *
+    */
+    void ShowStatistics();
+
 protected slots:
 
     /**
@@ -139,12 +212,6 @@ protected slots:
     */
     void ResultsAdded();
 
-    /**
-    * @brief Slot for changing the program's language
-    *
-    */
-    void MapLanguage(QAction *);
-
     /**
     * @brief Slot for showing/hiding standard toolbar
     */
@@ -172,13 +239,19 @@ protected slots:
     */
     void OpenHelpContents();
 
-protected:
-
     /**
-    * @brief Create menu items to change language
+    * @brief Add new line to log.
     *
     */
-    void CreateLanguageMenuItems();
+    void Log(const QString &logline);
+
+    /**
+    * @brief Handle new debug error.
+    *
+    */
+    void DebugError(const ErrorItem &item);
+
+protected:
 
     /**
     * @brief Set current language
@@ -221,12 +294,13 @@ protected:
     void DoCheckFiles(const QStringList &files);
 
     /**
-    * @brief Get all files recursively from given path
-    *
-    * @param path Path to get files from
-    * @return List of file paths
+    * @brief Check if we have a project for the checked directory.
+    * This method checks if there is open project for the directory. If no open
+    * project then we check if there is project file in the directory and load
+    * it.
+    * @return true if we have project, false if no project.
     */
-    QStringList GetFilesRecursively(const QString &path);
+    bool GetCheckProject();
 
     /**
     * @brief Get our default cppcheck settings and read project file.
@@ -235,14 +309,6 @@ protected:
     */
     Settings GetCppcheckSettings();
 
-    /**
-    * @brief Removes all unaccepted (by cppcheck core) files from the list
-    *
-    * @param list List to remove unaccepted files from
-    * @return List of files that are all accepted by cppcheck core
-    */
-    QStringList RemoveUnacceptedFiles(const QStringList &list);
-
     /**
     * @brief Load program settings
     *
@@ -266,6 +332,28 @@ protected:
     */
     void OpenHtmlHelpContents();
 
+    /**
+    * @brief Enable or disable project file actions.
+    * Project editing and closing actions should be only enabled when project is
+    * open and we are not checking files.
+    * @param enable If true then actions are enabled.
+    */
+    void EnableProjectActions(bool enable);
+
+    /**
+    * @brief Enable or disable project file actions.
+    * Project opening and creating actions should be disabled when checking.
+    * @param enable If true then actions are enabled.
+    */
+    void EnableProjectOpenActions(bool enable);
+
+    /**
+    * @brief Add include directories.
+    * @param includeDirs List of include directories to add.
+    * @param result Settings class where include directories are added.
+    */
+    void AddIncludeDirs(const QStringList &includeDirs, Settings &result);
+
     /**
     * @brief Program settings
     *
@@ -307,6 +395,30 @@ protected:
     */
     QString mCurrentDirectory;
 
+    /**
+    * @brief Log view.
+    */
+    LogView *mLogView;
+
+    /**
+     * @brief Help window..
+     */
+    HelpWindow *mHelpWindow;
+
+    /**
+    * @brief Project (file).
+    */
+    Project *mProject;
+
+private:
+
+    /**
+    * @brief Are we exiting the cppcheck?
+    * If this is true then the cppcheck is waiting for check threads to exit
+    * so that the application can be closed.
+    */
+    bool mExiting;
+
 };
 /// @}
 #endif // MAINWINDOW_H
