@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,29 +17,46 @@
  */
 
 #include <QApplication>
+#include <QWidget>
+#include <QStandardItem>
+#include <QModelIndex>
 #include <QDebug>
+#include <QString>
+#include <QStringList>
+#include <QList>
+#include <QMap>
+#include <QVariant>
 #include <QMenu>
 #include <QSignalMapper>
 #include <QProcess>
 #include <QDir>
 #include <QMessageBox>
+#include <QAction>
 #include <QFileInfo>
+#include <QFileDialog>
 #include <QClipboard>
+#include <QContextMenuEvent>
+#include <QModelIndex>
+#include <QItemSelectionModel>
+#include "erroritem.h"
+#include "settings.h"
+#include "applicationlist.h"
 #include "resultstree.h"
+#include "report.h"
 #include "xmlreport.h"
 
 ResultsTree::ResultsTree(QWidget * parent) :
     QTreeView(parent),
     mContextItem(0),
-    mCheckPath(""),
-    mVisibleErrors(false)
+    mVisibleErrors(false),
+    mSelectionModel(0)
 {
     for (int i = 0; i < SHOW_NONE; i++)
         mShowTypes[i] = false;
 
     setModel(&mModel);
     QStringList labels;
-    labels << tr("File") << tr("Severity") << tr("Line") << tr("Message");
+    labels << tr("File") << tr("Severity") << tr("Line") << tr("Summary");
     mModel.setHorizontalHeaderLabels(labels);
     setExpandsOnDoubleClick(false);
     setSortingEnabled(true);
@@ -60,35 +77,38 @@ void ResultsTree::Initialize(QSettings *settings, ApplicationList *list)
 }
 
 
-QStandardItem *ResultsTree::CreateItem(const QString &name)
+QStandardItem *ResultsTree::CreateNormalItem(const QString &name)
 {
     QStandardItem *item = new QStandardItem(name);
+    item->setData(name, Qt::ToolTipRole);
     item->setEditable(false);
     return item;
 }
 
-void ResultsTree::AddErrorItem(const QString &file,
-                               const QString &severity,
-                               const QString &message,
-                               const QStringList &files,
-                               const QVariantList &lines,
-                               const QString &id)
+QStandardItem *ResultsTree::CreateLineNumberItem(const QString &linenumber)
 {
-    Q_UNUSED(file);
+    QStandardItem *item = new QStandardItem(linenumber);
+    item->setData(linenumber, Qt::ToolTipRole);
+    item->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
+    item->setEditable(false);
+    return item;
+}
 
-    if (files.isEmpty())
+void ResultsTree::AddErrorItem(const ErrorItem &item)
+{
+    if (item.files.isEmpty())
     {
         return;
     }
 
-    QString realfile = StripPath(files[0], false);
+    QString realfile = StripPath(item.files[0], false);
 
     if (realfile.isEmpty())
     {
         realfile = tr("Undefined file");
     }
 
-    bool hide = !mShowTypes[SeverityToShowType(severity)];
+    bool hide = !mShowTypes[SeverityToShowType(item.severity)];
 
     //if there is at least one error that is not hidden, we have a visible error
     if (!hide)
@@ -96,48 +116,53 @@ void ResultsTree::AddErrorItem(const QString &file,
         mVisibleErrors = true;
     }
 
+    ErrorLine line;
+    line.file = realfile;
+    line.id = item.id;
+    line.line = item.lines[0];
+    line.summary = item.summary;
+    line.message = item.message;
+    line.severity = item.severity;
     //Create the base item for the error and ensure it has a proper
     //file item as a parent
-    QStandardItem *item = AddBacktraceFiles(EnsureFileItem(files[0], hide),
-                                            realfile,
-                                            lines[0].toInt(),
-                                            severity,
-                                            message,
-                                            hide,
-                                            SeverityToIcon(severity));
+    QStandardItem *stditem = AddBacktraceFiles(EnsureFileItem(line.file, hide),
+                             line,
+                             hide,
+                             SeverityToIcon(line.severity));
 
-    if (!item)
+    if (!stditem)
         return;
 
     //Add user data to that item
     QMap<QString, QVariant> data;
-    data["severity"]  = SeverityToShowType(severity);
-    data["message"]  = message;
-    data["file"]  = files[0];
-    data["line"]  = lines[0];
-    data["id"]  = id;
-    item->setData(QVariant(data));
+    data["hide"] = false;
+    data["severity"]  = SeverityToShowType(item.severity);
+    data["summary"] = item.summary;
+    data["message"]  = item.message;
+    data["file"]  = item.files[0];
+    data["line"]  = item.lines[0];
+    data["id"]  = item.id;
+    stditem->setData(QVariant(data));
 
     //Add backtrace files as children
-    for (int i = 1; i < files.size() && i < lines.size(); i++)
+    for (int i = 1; i < item.files.size() && i < item.lines.size(); i++)
     {
+        line.file = StripPath(item.files[i], false);
+        line.line = item.lines[i];
         QStandardItem *child_item;
-
-        child_item = AddBacktraceFiles(item,
-                                       StripPath(files[i], false),
-                                       lines[i].toInt(),
-                                       severity,
-                                       message,
+        child_item = AddBacktraceFiles(stditem,
+                                       line,
                                        hide,
                                        ":images/go-down.png");
 
         //Add user data to that item
         QMap<QString, QVariant> child_data;
-        child_data["severity"]  = SeverityToShowType(severity);
-        child_data["message"]  = message;
-        child_data["file"]  = files[i];
-        child_data["line"]  = lines[i];
-        child_data["id"]  = id;
+        child_data["severity"]  = SeverityToShowType(line.severity);
+        child_data["summary"] = line.summary;
+        child_data["message"]  = line.message;
+        child_data["file"]  = item.files[i];
+        child_data["line"]  = line.line;
+        child_data["id"]  = line.id;
         child_item->setData(QVariant(child_data));
     }
 
@@ -150,10 +175,7 @@ void ResultsTree::AddErrorItem(const QString &file,
 }
 
 QStandardItem *ResultsTree::AddBacktraceFiles(QStandardItem *parent,
-        const QString &file,
-        const int line,
-        const QString &severity,
-        const QString &message,
+        const ErrorLine &item,
         const bool hide,
         const QString &icon)
 
@@ -164,17 +186,22 @@ QStandardItem *ResultsTree::AddBacktraceFiles(QStandardItem *parent,
     }
 
     QList<QStandardItem*> list;
-    list << CreateItem(file);
-    list << CreateItem(tr(severity.toLatin1()));
-    list << CreateItem(QString("%1").arg(line));
+    // Ensure shown path is with native separators
+    const QString file = QDir::toNativeSeparators(item.file);
+    list << CreateNormalItem(file);
+    list << CreateNormalItem(tr(item.severity.toLatin1()));
+    list << CreateLineNumberItem(QString("%1").arg(item.line));
     //TODO message has parameter names so we'll need changes to the core
     //cppcheck so we can get proper translations
-    list << CreateItem(tr(message.toLatin1()));
+    list << CreateNormalItem(tr(item.summary.toLatin1()));
 
     // Check for duplicate rows and don't add them if found
     for (int i = 0; i < parent->rowCount(); i++)
     {
-        // the first column is the file name and is always the same so skip it
+        // The first column is the file name and is always the same so
+        // we skip it in other platforms than Windows, where filename case
+        // is ignored. So in Windows we can get filenames "header.h" and
+        // "Header.h" and must compare them as identical filenames.
 
         // the third column is the line number so check it first
         if (parent->child(i, 2)->text() == list[2]->text())
@@ -182,11 +209,18 @@ QStandardItem *ResultsTree::AddBacktraceFiles(QStandardItem *parent,
             // the second column is the severity so check it next
             if (parent->child(i, 1)->text() == list[1]->text())
             {
-                // the forth column is the message so check it last
+                // the fourth column is the summary so check it last
                 if (parent->child(i, 3)->text() == list[3]->text())
                 {
+#if defined(_WIN32)
+                    const QString first = parent->child(i, 0)->text().toLower();
+                    const QString second = list[0]->text().toLower();
+                    if (first == second)
+                        return 0;
+#else
                     // this row matches so don't add it
                     return 0;
+#endif // _WIN32
                 }
             }
         }
@@ -222,6 +256,14 @@ ShowTypes ResultsTree::SeverityToShowType(const QString & severity)
         return SHOW_ERRORS;
     if (severity == "style")
         return SHOW_STYLE;
+    if (severity == "warning")
+        return SHOW_WARNINGS;
+    if (severity == "performance")
+        return SHOW_PERFORMANCE;
+    if (severity == "portability")
+        return SHOW_PORTABILITY;
+    if (severity == "information")
+        return SHOW_INFORMATION;
 
     return SHOW_NONE;
 }
@@ -273,6 +315,36 @@ void ResultsTree::ShowResults(ShowTypes type, bool show)
     }
 }
 
+void ResultsTree::ShowHiddenResults()
+{
+    //Clear the "hide" flag for each item
+    int filecount = mModel.rowCount();
+    for (int i = 0; i < filecount; i++)
+    {
+        QStandardItem *file = mModel.item(i, 0);
+        if (!file)
+            continue;
+
+        QVariantMap data = file->data().toMap();
+        data["hide"] = false;
+        file->setData(QVariant(data));
+
+        int errorcount = file->rowCount();
+        for (int j = 0; j < errorcount; j++)
+        {
+            QStandardItem *child = file->child(j, 0);
+            if (child)
+            {
+                data = child->data().toMap();
+                data["hide"] = false;
+                child->setData(QVariant(data));
+            }
+        }
+    }
+    RefreshTree();
+    emit ResultsHidden(false);
+}
+
 
 void ResultsTree::RefreshTree()
 {
@@ -310,7 +382,7 @@ void ResultsTree::RefreshTree()
             QVariantMap data = userdata.toMap();
 
             //Check if this error should be hidden
-            bool hide = !mShowTypes[VariantToShowType(data["severity"])];
+            bool hide = (data["hide"].toBool() || !mShowTypes[VariantToShowType(data["severity"])]);
 
             if (!hide)
             {
@@ -327,6 +399,12 @@ void ResultsTree::RefreshTree()
             }
         }
 
+        //Hide the file if its "hide" attribute is set
+        if (file->data().toMap()["hide"].toBool())
+        {
+            show = false;
+        }
+
         //Show the file if any of it's errors are visible
         setRowHidden(i, QModelIndex(), !show);
     }
@@ -335,14 +413,18 @@ void ResultsTree::RefreshTree()
 QStandardItem *ResultsTree::EnsureFileItem(const QString &fullpath, bool hide)
 {
     QString name = StripPath(fullpath, false);
-    QStandardItem *item = FindFileItem(name);
+    // Since item has path with native separators we must use path with
+    // native separators to find it.
+    QStandardItem *item = FindFileItem(QDir::toNativeSeparators(name));
 
     if (item)
     {
         return item;
     }
 
-    item = CreateItem(name);
+    // Ensure shown path is with native separators
+    name = QDir::toNativeSeparators(name);
+    item = CreateNormalItem(name);
     item->setIcon(QIcon(":images/text-x-generic.png"));
 
     //Add user data to that item
@@ -370,6 +452,11 @@ void ResultsTree::contextMenuEvent(QContextMenuEvent * e)
     QModelIndex index = indexAt(e->pos());
     if (index.isValid())
     {
+        bool multipleSelection = false;
+        mSelectionModel = selectionModel();
+        if (mSelectionModel->selectedRows().count() > 1)
+            multipleSelection = true;
+
         mContextItem = mModel.itemFromIndex(index);
 
         //Create a new context menu
@@ -389,6 +476,8 @@ void ResultsTree::contextMenuEvent(QContextMenuEvent * e)
             {
                 //Create an action for the application
                 QAction *start = new QAction(mApplications->GetApplicationName(i), &menu);
+                if (multipleSelection)
+                    start->setDisabled(true);
 
                 //Add it to our list so we can disconnect later on
                 actions << start;
@@ -419,14 +508,24 @@ void ResultsTree::contextMenuEvent(QContextMenuEvent * e)
             QAction *copyfilename 	= new QAction(tr("Copy filename"), &menu);
             QAction *copypath 		= new QAction(tr("Copy full path"), &menu);
             QAction *copymessage 	= new QAction(tr("Copy message"), &menu);
+            QAction *hide        	= new QAction(tr("Hide"), &menu);
+
+            if (multipleSelection)
+            {
+                copyfilename->setDisabled(true);
+                copypath->setDisabled(true);
+                copymessage->setDisabled(true);
+            }
 
             menu.addAction(copyfilename);
             menu.addAction(copypath);
             menu.addAction(copymessage);
+            menu.addAction(hide);
 
             connect(copyfilename, SIGNAL(triggered()), this, SLOT(CopyFilename()));
             connect(copypath, SIGNAL(triggered()), this, SLOT(CopyFullPath()));
             connect(copymessage, SIGNAL(triggered()), this, SLOT(CopyMessage()));
+            connect(hide, SIGNAL(triggered()), this, SLOT(HideResult()));
         }
 
         //Start the menu
@@ -452,7 +551,7 @@ void ResultsTree::contextMenuEvent(QContextMenuEvent * e)
 
 void ResultsTree::StartApplication(QStandardItem *target, int application)
 {
-    //If there are now application's specified, tell the user about it
+    //If there are no applications specified, tell the user about it
     if (mApplications->GetApplicationCount() == 0)
     {
         QMessageBox msg(QMessageBox::Information,
@@ -476,6 +575,34 @@ void ResultsTree::StartApplication(QStandardItem *target, int application)
 
         //Replace (file) with filename
         QString file = data["file"].toString();
+
+        QFileInfo info(file);
+        if (!info.exists())
+        {
+            if (info.isAbsolute())
+            {
+                QMessageBox msgbox(this);
+                msgbox.setWindowTitle("Cppcheck");
+                msgbox.setText(tr("Could not find the file!"));
+                msgbox.setIcon(QMessageBox::Critical);
+                msgbox.exec();
+            }
+            else
+            {
+                QDir checkdir(mCheckPath);
+                if (checkdir.isAbsolute() && checkdir.exists())
+                {
+                    file = mCheckPath + "/" + file;
+                }
+                else
+                {
+                    QString dir = AskFileDir(file);
+                    dir += '/';
+                    file = dir + file;
+                }
+            }
+        }
+
         if (file.indexOf(" ") > -1)
         {
             file.insert(0, "\"");
@@ -506,6 +633,22 @@ void ResultsTree::StartApplication(QStandardItem *target, int application)
     }
 }
 
+QString ResultsTree::AskFileDir(const QString &file)
+{
+    QString text = tr("Could not find file:\n%1\nPlease select the directory where file is located.").arg(file);
+    QMessageBox msgbox(this);
+    msgbox.setWindowTitle("Cppcheck");
+    msgbox.setText(text);
+    msgbox.setIcon(QMessageBox::Warning);
+    msgbox.exec();
+
+    QString dir = QFileDialog::getExistingDirectory(this, tr("Select Directory"),
+                  "",
+                  QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
+    mCheckPath = dir;
+    return dir;
+}
+
 void ResultsTree::CopyFilename()
 {
     CopyPath(mContextItem, false);
@@ -533,6 +676,26 @@ void ResultsTree::CopyMessage()
     }
 }
 
+void ResultsTree::HideResult()
+{
+    if (!mSelectionModel)
+        return;
+
+    QModelIndexList selectedRows = mSelectionModel->selectedRows();
+    QModelIndex index;
+    foreach(index, selectedRows)
+    {
+        QStandardItem *item = mModel.itemFromIndex(index);
+        //Set the "hide" flag for this item
+        QVariantMap data = item->data().toMap();
+        data["hide"] = true;
+        item->setData(QVariant(data));
+
+        RefreshTree();
+        emit ResultsHidden(true);
+    }
+}
+
 void ResultsTree::Context(int application)
 {
     StartApplication(mContextItem, application);
@@ -556,7 +719,7 @@ void ResultsTree::CopyPath(QStandardItem *target, bool fullPath)
 
         //Replace (file) with filename
         QString file = data["file"].toString();
-        pathStr = file;
+        pathStr = QDir::toNativeSeparators(file);
         if (!fullPath)
         {
             QFileInfo fi(pathStr);
@@ -568,13 +731,19 @@ void ResultsTree::CopyPath(QStandardItem *target, bool fullPath)
     }
 }
 
-QString ResultsTree::SeverityToIcon(const QString &severity)
+QString ResultsTree::SeverityToIcon(const QString &severity) const
 {
-    if (severity == "possible error")
-        return ":images/dialog-warning.png";
     if (severity == "error")
         return ":images/dialog-error.png";
-    if (severity == "style" || severity == "possible style")
+    if (severity == "style")
+        return ":images/applications-development.png";
+    if (severity == "warning")
+        return ":images/dialog-warning.png";
+    if (severity == "portability")
+        return ":images/applications-system.png";
+    if (severity == "performance")
+        return ":images/utilities-system-monitor.png";
+    if (severity == "information")
         return ":images/dialog-information.png";
 
     return "";
@@ -601,8 +770,6 @@ void ResultsTree::SaveErrors(Report *report, QStandardItem *item)
         return;
     }
 
-    //qDebug() << item->text() << "has" << item->rowCount() << "errors";
-
     for (int i = 0; i < item->rowCount(); i++)
     {
         QStandardItem *error = item->child(i, 0);
@@ -622,17 +789,16 @@ void ResultsTree::SaveErrors(Report *report, QStandardItem *item)
         //Convert it to QVariantMap
         QVariantMap data = userdata.toMap();
 
-        QString severity = ShowTypeToString(VariantToShowType(data["severity"]));
-        QString message = data["message"].toString();
-        QString id = data["id"].toString();
+        ErrorItem item;
+        item.severity = ShowTypeToString(VariantToShowType(data["severity"]));
+        item.summary = data["summary"].toString();
+        item.message = data["message"].toString();
+        item.id = data["id"].toString();
         QString file = StripPath(data["file"].toString(), true);
-        QString line = data["line"].toString();
+        unsigned int line = data["line"].toUInt();
 
-        QStringList files;
-        QStringList lines;
-
-        files << file;
-        lines << line;
+        item.files << file;
+        item.lines << line;
 
         for (int j = 0; j < error->rowCount(); j++)
         {
@@ -643,13 +809,13 @@ void ResultsTree::SaveErrors(Report *report, QStandardItem *item)
             QVariantMap child_data = child_userdata.toMap();
 
             file = StripPath(child_data["file"].toString(), true);
-            line = child_data["line"].toString();
+            line = child_data["line"].toUInt();
 
-            files << file;
-            lines << line;
+            item.files << file;
+            item.lines << line;
         }
 
-        report->WriteError(files, lines, id, severity, message);
+        report->WriteError(item);
     }
 }
 
@@ -665,6 +831,22 @@ QString ResultsTree::ShowTypeToString(ShowTypes type)
         return tr("error");
         break;
 
+    case SHOW_WARNINGS:
+        return tr("warning");
+        break;
+
+    case SHOW_PERFORMANCE:
+        return tr("performance");
+        break;
+
+    case SHOW_PORTABILITY:
+        return tr("portability");
+        break;
+
+    case SHOW_INFORMATION:
+        return tr("information");
+        break;
+
     case SHOW_NONE:
         return "";
         break;
@@ -793,8 +975,13 @@ bool ResultsTree::HasResults() const
 void ResultsTree::Translate()
 {
     QStringList labels;
-    labels << tr("File") << tr("Severity") << tr("Line") << tr("Message");
+    labels << tr("File") << tr("Severity") << tr("Line") << tr("Summary");
     mModel.setHorizontalHeaderLabels(labels);
     //TODO go through all the errors in the tree and translate severity and message
 }
 
+void ResultsTree::currentChanged(const QModelIndex &current, const QModelIndex &previous)
+{
+    QTreeView::currentChanged(current, previous);
+    emit SelectionChanged(current);
+}
