@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,44 +17,61 @@
  */
 
 #include <QDebug>
-#include <QVBoxLayout>
 #include <QFile>
 #include <QMessageBox>
+#include <QStandardItemModel>
+#include <QStandardItem>
+#include <QVariant>
+#include <QString>
+#include <QModelIndex>
+#include <QSettings>
+#include "erroritem.h"
 #include "resultsview.h"
+#include "resultstree.h"
 #include "report.h"
 #include "txtreport.h"
 #include "xmlreport.h"
+#include "xmlreportv1.h"
+#include "xmlreportv2.h"
 #include "csvreport.h"
+#include "applicationlist.h"
+#include "checkstatistics.h"
 
 ResultsView::ResultsView(QWidget * parent) :
     QWidget(parent),
     mErrorsFound(false),
-    mShowNoErrorsMessage(true)
+    mShowNoErrorsMessage(true),
+    mStatistics(new CheckStatistics(this))
 {
     mUI.setupUi(this);
+
+    connect(mUI.mTree, SIGNAL(ResultsHidden(bool)), this, SIGNAL(ResultsHidden(bool)));
+    connect(mUI.mTree, SIGNAL(SelectionChanged(const QModelIndex &)), this, SLOT(UpdateDetails(const QModelIndex &)));
 }
 
 void ResultsView::Initialize(QSettings *settings, ApplicationList *list)
 {
-
     mUI.mProgress->setMinimum(0);
     mUI.mProgress->setVisible(false);
+
+    QByteArray state = settings->value(SETTINGS_MAINWND_SPLITTER_STATE).toByteArray();
+    mUI.mVerticalSplitter->restoreState(state);
     mShowNoErrorsMessage = settings->value(SETTINGS_SHOW_NO_ERRORS, true).toBool();
 
     mUI.mTree->Initialize(settings, list);
 }
 
-
 ResultsView::~ResultsView()
 {
     //dtor
 }
 
-
 void ResultsView::Clear()
 {
     mUI.mTree->Clear();
+    mUI.mDetails->setText("");
     mErrorsFound = false;
+    mStatistics->Clear();
 
     //Clear the progressbar
     mUI.mProgress->setMaximum(100);
@@ -66,16 +83,12 @@ void ResultsView::Progress(int value)
     mUI.mProgress->setValue(value);
 }
 
-void ResultsView::Error(const QString &file,
-                        const QString &severity,
-                        const QString &message,
-                        const QStringList &files,
-                        const QVariantList &lines,
-                        const QString &id)
+void ResultsView::Error(const ErrorItem &item)
 {
     mErrorsFound = true;
-    mUI.mTree->AddErrorItem(file, severity, message, files, lines, id);
+    mUI.mTree->AddErrorItem(item);
     emit GotResults();
+    mStatistics->AddItem(ResultsTree::SeverityToShowType(item.severity));
 }
 
 void ResultsView::ShowResults(ShowTypes type, bool show)
@@ -93,6 +106,11 @@ void ResultsView::ExpandAllResults()
     mUI.mTree->expandAll();
 }
 
+void ResultsView::ShowHiddenResults()
+{
+    mUI.mTree->ShowHiddenResults();
+}
+
 void ResultsView::Save(const QString &filename, Report::Type type)
 {
     if (!mErrorsFound)
@@ -114,7 +132,10 @@ void ResultsView::Save(const QString &filename, Report::Type type)
         report = new TxtReport(filename, this);
         break;
     case Report::XML:
-        report = new XmlReport(filename, this);
+        report = new XmlReportV1(filename, this);
+        break;
+    case Report::XMLV2:
+        report = new XmlReportV2(filename, this);
         break;
     }
 
@@ -203,9 +224,12 @@ bool ResultsView::HasResults() const
     return mUI.mTree->HasResults();
 }
 
-void ResultsView::SaveSettings()
+void ResultsView::SaveSettings(QSettings *settings)
 {
     mUI.mTree->SaveSettings();
+    QByteArray state = mUI.mVerticalSplitter->saveState();
+    settings->setValue(SETTINGS_MAINWND_SPLITTER_STATE, state);
+    mUI.mVerticalSplitter->restoreState(state);
 }
 
 void ResultsView::Translate()
@@ -217,3 +241,84 @@ void ResultsView::DisableProgressbar()
 {
     mUI.mProgress->setEnabled(false);
 }
+
+void ResultsView::ReadErrorsXml(const QString &filename)
+{
+    const int version = XmlReport::determineVersion(filename);
+    if (version == 0)
+    {
+        QMessageBox msgBox;
+        msgBox.setText(tr("Failed to read the report."));
+        msgBox.setIcon(QMessageBox::Critical);
+        msgBox.exec();
+        return;
+    }
+
+    XmlReport *report = NULL;
+    if (version == 1)
+        report = new XmlReportV1(filename, this);
+    else if (version == 2)
+        report = new XmlReportV2(filename, this);
+
+    QList<ErrorItem> errors;
+    if (report)
+    {
+        if (report->Open())
+            errors = report->Read();
+        else
+        {
+            QMessageBox msgBox;
+            msgBox.setText(tr("Failed to read the report."));
+            msgBox.setIcon(QMessageBox::Critical);
+            msgBox.exec();
+        }
+        delete report;
+        report = NULL;
+    }
+    else
+    {
+        QMessageBox msgBox;
+        msgBox.setText(tr("Failed to read the report."));
+        msgBox.setIcon(QMessageBox::Critical);
+        msgBox.exec();
+    }
+
+    ErrorItem item;
+    foreach(item, errors)
+    {
+        mUI.mTree->AddErrorItem(item);
+    }
+    mUI.mTree->SetCheckDirectory("");
+}
+
+void ResultsView::UpdateDetails(const QModelIndex &index)
+{
+    QStandardItemModel *model = qobject_cast<QStandardItemModel*>(mUI.mTree->model());
+    QStandardItem *item = model->itemFromIndex(index);
+
+    if (!item)
+    {
+        mUI.mDetails->setText("");
+        return;
+    }
+
+    // Make sure we are working with the first column
+    if (item->parent() && item->column() != 0)
+        item = item->parent()->child(item->row(), 0);
+
+    QVariantMap data = item->data().toMap();
+
+    // If there is no severity data then it is a parent item without summary and message
+    if (!data.contains("severity"))
+    {
+        mUI.mDetails->setText("");
+        return;
+    }
+
+    const QString summary = data["summary"].toString();
+    const QString message = data["message"].toString();
+    const QString formattedMsg = QString("%1: %2\n%3: %4")
+                                 .arg(tr("Summary")).arg(summary)
+                                 .arg(tr("Message")).arg(message);
+    mUI.mDetails->setText(formattedMsg);
+}
