@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,57 +17,71 @@
  */
 
 
-#include "settingsdialog.h"
 #include <QDialog>
 #include <QWidget>
 #include <QLabel>
+#include <QList>
 #include <QDebug>
+#include <QListWidgetItem>
 #include <QTabWidget>
+#include <QSettings>
+#include <QFileDialog>
+#include <QThread>
+#include "settingsdialog.h"
 #include "applicationdialog.h"
+#include "applicationlist.h"
+#include "translationhandler.h"
 #include "common.h"
 
 SettingsDialog::SettingsDialog(QSettings *programSettings,
                                ApplicationList *list,
+                               TranslationHandler *translator,
                                QWidget *parent) :
     QDialog(parent),
     mSettings(programSettings),
     mApplications(list),
-    mTempApplications(new ApplicationList(this))
+    mTempApplications(new ApplicationList(this)),
+    mTranslator(translator)
 {
     mUI.setupUi(this);
     mTempApplications->Copy(list);
 
-    connect(mUI.mButtons, SIGNAL(accepted()), this, SLOT(Ok()));
-    connect(mUI.mButtons, SIGNAL(rejected()), this, SLOT(reject()));
-
+    mUI.mEditIncludePaths->setText(programSettings->value(SETTINGS_GLOBAL_INCLUDE_PATHS).toString());
     mUI.mJobs->setText(programSettings->value(SETTINGS_CHECK_THREADS, 1).toString());
     mUI.mForce->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_CHECK_FORCE, false).toBool()));
     mUI.mShowFullPath->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_SHOW_FULL_PATH, false).toBool()));
     mUI.mShowNoErrorsMessage->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_SHOW_NO_ERRORS, false).toBool()));
+    mUI.mShowDebugWarnings->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_SHOW_DEBUG_WARNINGS, false).toBool()));
+    mUI.mSaveAllErrors->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_SAVE_ALL_ERRORS, false).toBool()));
+    mUI.mSaveFullPath->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_SAVE_FULL_PATH, false).toBool()));
+    mUI.mInlineSuppressions->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_INLINE_SUPPRESSIONS, false).toBool()));
 
-
+    connect(mUI.mButtons, SIGNAL(accepted()), this, SLOT(Ok()));
+    connect(mUI.mButtons, SIGNAL(rejected()), this, SLOT(reject()));
     connect(mUI.mButtonAdd, SIGNAL(clicked()),
             this, SLOT(AddApplication()));
-
     connect(mUI.mButtonDelete, SIGNAL(clicked()),
             this, SLOT(DeleteApplication()));
-
     connect(mUI.mButtonModify, SIGNAL(clicked()),
             this, SLOT(ModifyApplication()));
-
     connect(mUI.mButtonDefault, SIGNAL(clicked()),
             this, SLOT(DefaultApplication()));
-
     connect(mUI.mListWidget, SIGNAL(itemDoubleClicked(QListWidgetItem *)),
             this, SLOT(ModifyApplication()));
+    connect(mUI.mBtnAddIncludePath, SIGNAL(clicked()),
+            this, SLOT(AddIncludePath()));
 
     mUI.mListWidget->setSortingEnabled(false);
     PopulateListWidget();
 
-    mUI.mSaveAllErrors->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_SAVE_ALL_ERRORS, false).toBool()));
-    mUI.mSaveFullPath->setCheckState(BoolToCheckState(programSettings->value(SETTINGS_SAVE_FULL_PATH, false).toBool()));
+    const int count = QThread::idealThreadCount();
+    if (count != -1)
+        mUI.mLblIdealThreads->setText(QString::number(count));
+    else
+        mUI.mLblIdealThreads->setText(tr("N/A"));
 
     LoadSettings();
+    InitTranslationsList();
 }
 
 SettingsDialog::~SettingsDialog()
@@ -75,7 +89,18 @@ SettingsDialog::~SettingsDialog()
     SaveSettings();
 }
 
-Qt::CheckState SettingsDialog::BoolToCheckState(bool yes)
+void SettingsDialog::InitTranslationsList()
+{
+    QStringList languages = mTranslator->GetNames();
+    foreach(const QString lang, languages)
+    {
+        mUI.mListLanguages->addItem(lang);
+    }
+    const int current = mTranslator->GetCurrentLanguage();
+    mUI.mListLanguages->setCurrentRow(current);
+}
+
+Qt::CheckState SettingsDialog::BoolToCheckState(bool yes) const
 {
     if (yes)
     {
@@ -84,7 +109,7 @@ Qt::CheckState SettingsDialog::BoolToCheckState(bool yes)
     return Qt::Unchecked;
 }
 
-bool SettingsDialog::CheckStateToBool(Qt::CheckState state)
+bool SettingsDialog::CheckStateToBool(Qt::CheckState state) const
 {
     if (state == Qt::Checked)
     {
@@ -106,7 +131,7 @@ void SettingsDialog::SaveSettings()
     mSettings->setValue(SETTINGS_CHECK_DIALOG_HEIGHT, size().height());
 }
 
-void SettingsDialog::SaveCheckboxValues()
+void SettingsDialog::SaveSettingValues()
 {
     int jobs = mUI.mJobs->text().toInt();
     if (jobs <= 0)
@@ -120,6 +145,10 @@ void SettingsDialog::SaveCheckboxValues()
     SaveCheckboxValue(mUI.mSaveFullPath, SETTINGS_SAVE_FULL_PATH);
     SaveCheckboxValue(mUI.mShowFullPath, SETTINGS_SHOW_FULL_PATH);
     SaveCheckboxValue(mUI.mShowNoErrorsMessage, SETTINGS_SHOW_NO_ERRORS);
+    SaveCheckboxValue(mUI.mShowDebugWarnings, SETTINGS_SHOW_DEBUG_WARNINGS);
+    SaveCheckboxValue(mUI.mInlineSuppressions, SETTINGS_INLINE_SUPPRESSIONS);
+    mSettings->setValue(SETTINGS_GLOBAL_INCLUDE_PATHS, mUI.mEditIncludePaths->text());
+    mSettings->setValue(SETTINGS_LANGUAGE, mUI.mListLanguages->currentRow());
 }
 
 void SettingsDialog::SaveCheckboxValue(QCheckBox *box, const QString &name)
@@ -162,7 +191,7 @@ void SettingsDialog::ModifyApplication()
 
         ApplicationDialog dialog(mTempApplications->GetApplicationName(row),
                                  mTempApplications->GetApplicationPath(row),
-                                 tr("Modify an application"));
+                                 tr("Modify an application"), this);
 
         if (dialog.exec() == QDialog::Accepted)
         {
@@ -224,5 +253,18 @@ bool SettingsDialog::ShowNoErrorsMessage()
     return CheckStateToBool(mUI.mShowNoErrorsMessage->checkState());
 }
 
+void SettingsDialog::AddIncludePath()
+{
+    QString selectedDir = QFileDialog::getExistingDirectory(this,
+                          tr("Select include directory"),
+                          QString());
 
-
+    if (!selectedDir.isEmpty())
+    {
+        QString text = mUI.mEditIncludePaths->text();
+        if (!text.isEmpty())
+            text += ';';
+        text += selectedDir;
+        mUI.mEditIncludePaths->setText(text);
+    }
+}
