@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,18 +21,14 @@
 #define SETTINGSDIALOG_H
 
 #include <QDialog>
-#include <QLineEdit>
-#include <QComboBox>
-#include <QSettings>
-#include <QCheckBox>
-#include <QVBoxLayout>
-#include <QPushButton>
-#include "applicationlist.h"
-
-#include <QListWidget>
 #include <QKeyEvent>
 #include "ui_settings.h"
 
+class QSettings;
+class QWidget;
+class ApplicationList;
+class TranslationHandler;
+
 /// @addtogroup GUI
 /// @{
 
@@ -46,6 +42,7 @@ class SettingsDialog : public QDialog
 public:
     SettingsDialog(QSettings *programSettings,
                    ApplicationList *list,
+                   TranslationHandler *translator,
                    QWidget *parent = 0);
     virtual ~SettingsDialog();
 
@@ -53,7 +50,7 @@ public:
     * @brief Save all values to QSettings
     *
     */
-    void SaveCheckboxValues();
+    void SaveSettingValues();
 
     /**
     * @brief Get checkbox value for mShowFullPath
@@ -114,6 +111,13 @@ protected slots:
     *
     */
     void DefaultApplication();
+
+    /**
+    * @brief Slot for adding new include path
+    *
+    */
+    void AddIncludePath();
+
 protected:
 
     /**
@@ -151,7 +155,7 @@ protected:
     * @param yes value to convert
     * @return value converted to Qt::CheckState
     */
-    Qt::CheckState BoolToCheckState(bool yes);
+    Qt::CheckState BoolToCheckState(bool yes) const;
 
     /**
     * @brief Converts Qt::CheckState to bool
@@ -159,8 +163,12 @@ protected:
     * @param state Qt::CheckState to convert
     * @return converted value
     */
-    bool CheckStateToBool(Qt::CheckState state);
+    bool CheckStateToBool(Qt::CheckState state) const;
 
+    /**
+    * @brief Populate the translations list.
+    */
+    void InitTranslationsList();
 
     /**
     * @brief Settings
@@ -181,6 +189,12 @@ protected:
     */
     ApplicationList *mTempApplications;
 
+    /**
+    * @brief List of translations.
+    *
+    */
+    TranslationHandler *mTranslator;
+
     /**
     * @brief Dialog from UI designer
     *
