@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,12 +16,11 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "translationhandler.h"
-
 #include <QApplication>
 #include <QFile>
 #include <QDebug>
 #include <QLocale>
+#include "translationhandler.h"
 
 TranslationHandler::TranslationHandler(QObject *parent) :
     QObject(parent),
@@ -35,7 +34,9 @@ TranslationHandler::TranslationHandler(QObject *parent) :
             << QT_TRANSLATE_NOOP("MainWindow", "Swedish")
             << QT_TRANSLATE_NOOP("MainWindow", "German")
             << QT_TRANSLATE_NOOP("MainWindow", "Russian")
-            << QT_TRANSLATE_NOOP("MainWindow", "Polish");
+            << QT_TRANSLATE_NOOP("MainWindow", "Polish")
+            << QT_TRANSLATE_NOOP("MainWindow", "Japanese")
+            << QT_TRANSLATE_NOOP("MainWindow", "Serbian");
 
     mFiles  << "cppcheck_en"
             << "cppcheck_nl"
@@ -43,9 +44,11 @@ TranslationHandler::TranslationHandler(QObject *parent) :
             << "cppcheck_se"
             << "cppcheck_de"
             << "cppcheck_ru"
-            << "cppcheck_pl";
+            << "cppcheck_pl"
+            << "cppcheck_ja"
+            << "cppcheck_sr";
 
-    //Load english as a fallback language
+    //Load English as a fallback language
     QTranslator *english = new QTranslator();
     if (english->load("cppcheck_en"))
     {
@@ -62,19 +65,19 @@ TranslationHandler::~TranslationHandler()
 {
 }
 
-const QStringList TranslationHandler::GetNames()
+const QStringList TranslationHandler::GetNames() const
 {
     return mNames;
 }
 
-const QStringList TranslationHandler::GetFiles()
+const QStringList TranslationHandler::GetFiles() const
 {
     return mFiles;
 }
 
 bool TranslationHandler::SetLanguage(const int index, QString &error)
 {
-    //If english is the language
+    //If English is the language
     if (index == 0)
     {
         //Just remove all extra translators
@@ -143,7 +146,7 @@ int TranslationHandler::SuggestLanguage() const
     //And see if we can find it from our list of language files
     int index = mFiles.indexOf(file);
 
-    //If nothing found, return english
+    //If nothing found, return English
     if (index < 0)
     {
         return 0;
@@ -151,3 +154,4 @@ int TranslationHandler::SuggestLanguage() const
 
     return index;
 }
+
