@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,8 +33,8 @@ class TranslationHandler : QObject
 public:
     TranslationHandler(QObject *parent);
     virtual ~TranslationHandler();
-    const QStringList GetNames();
-    const QStringList GetFiles();
+    const QStringList GetNames() const;
+    const QStringList GetFiles() const;
     bool SetLanguage(const int index, QString &error);
     int GetCurrentLanguage() const;
     int SuggestLanguage() const;
