@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 #include <list>
 #include <iostream>
+#include <set>
 
 /// @addtogroup Core
 /// @{
@@ -38,16 +39,11 @@ class Check
 {
 public:
     /** This constructor is used when registering the CheckClass */
-    Check()
-        : _tokenizer(0), _settings(0), _errorLogger(0)
-    {
-        instances().push_back(this);
-        instances().sort();
-    }
+    Check(const std::string &aname);
 
     /** This constructor is used when running checks. */
-    Check(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
-        : _tokenizer(tokenizer), _settings(settings), _errorLogger(errorLogger)
+    Check(const std::string &aname, const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
+        : _name(aname), _tokenizer(tokenizer), _settings(settings), _errorLogger(errorLogger)
     { }
 
     virtual ~Check()
@@ -94,10 +90,13 @@ public:
     virtual void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) = 0;
 
     /** get error messages */
-    virtual void getErrorMessages() = 0;
+    virtual void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) = 0;
 
     /** class name, used to generate documentation */
-    virtual std::string name() const = 0;
+    std::string name() const
+    {
+        return _name;
+    }
 
     /** get information about this class, used to generate documentation */
     virtual std::string classInfo() const = 0;
@@ -109,16 +108,17 @@ public:
      */
     static void reportError(const ErrorLogger::ErrorMessage &errmsg)
     {
-        std::cout << errmsg.toXML() << std::endl;
+        std::cout << errmsg.toXML(true, 1) << std::endl;
     }
 
 protected:
+    const std::string _name;
     const Tokenizer * const _tokenizer;
     const Settings * const _settings;
     ErrorLogger * const _errorLogger;
 
     /** report an error */
-    void reportError(const Token *tok, const Severity::e severity, const std::string &id, const std::string &msg)
+    void reportError(const Token *tok, const Severity::SeverityType severity, const std::string &id, const std::string &msg)
     {
         std::list<const Token *> callstack;
         if (tok)
@@ -127,16 +127,8 @@ protected:
     }
 
     /** report an error */
-    void reportError(const std::list<const Token *> &callstack, const Severity::e severity, const std::string &id, std::string msg)
+    void reportError(const std::list<const Token *> &callstack, Severity::SeverityType severity, const std::string &id, std::string msg)
     {
-        // If the verbose flag hasn't been given, don't show verbose information
-        if (!_settings || !_settings->_verbose)
-        {
-            std::string::size_type pos = msg.find("\n");
-            if (pos != std::string::npos)
-                msg.erase(pos);
-        }
-
         std::list<ErrorLogger::ErrorMessage::FileLocation> locationList;
         for (std::list<const Token *>::const_iterator it = callstack.begin(); it != callstack.end(); ++it)
         {
@@ -146,11 +138,13 @@ protected:
 
             ErrorLogger::ErrorMessage::FileLocation loc;
             loc.line = (*it)->linenr();
-            loc.file = _tokenizer->file(*it);
+            loc.setfile(_tokenizer->file(*it));
             locationList.push_back(loc);
         }
 
-        const ErrorLogger::ErrorMessage errmsg(locationList, Severity::stringify(severity), msg, id);
+        ErrorLogger::ErrorMessage errmsg(locationList, severity, msg, id);
+        if (_tokenizer && _tokenizer->getFiles() && !_tokenizer->getFiles()->empty())
+            errmsg.file0 = _tokenizer->getFiles()->at(0);
         if (_errorLogger)
             _errorLogger->reportErr(errmsg);
         else
@@ -160,17 +154,30 @@ protected:
 
 
 private:
-    /** compare the names of Check classes, used when sorting the Check descendants */
-    bool operator<(const Check *other) const
-    {
-        return (name() < other->name());
-    }
-
     /** disabled assignment operator */
     void operator=(const Check &);
 
 };
 
+namespace std
+{
+/** compare the names of Check classes, used when sorting the Check descendants */
+template <> struct less<Check *>
+{
+    bool operator()(const Check *p1, const Check *p2) const
+    {
+        return (p1->name() < p2->name());
+    }
+};
+}
+
+inline Check::Check(const std::string &aname)
+    : _name(aname), _tokenizer(0), _settings(0), _errorLogger(0)
+{
+    instances().push_back(this);
+    instances().sort(std::less<Check *>());
+}
+
 /// @}
 
 #endif
