@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 //---------------------------------------------------------------------------
 
 #include "checkautovariables.h"
+#include "symboldatabase.h"
 
 #include <sstream>
 #include <iostream>
@@ -137,6 +138,11 @@ void CheckAutoVariables::autoVariables()
     bool begin_function_decl = false;
     int bindent = 0;
 
+    // Which variables have an unknown type?
+    std::set<unsigned int> unknown_type;
+
+    const SymbolDatabase * const symbolDatabase = _tokenizer->getSymbolDatabase();
+
     for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next())
     {
 
@@ -188,17 +194,24 @@ void CheckAutoVariables::autoVariables()
         else if (Token::Match(tok, "%var% %var% ;") && !isExternOrStatic(tok) && isTypeName(tok))
         {
             addVD(tok->next()->varId());
+            if (!tok->isStandardType() &&
+                !symbolDatabase->isClassOrStruct(tok->str()))
+            {
+                unknown_type.insert(tok->next()->varId());
+            }
         }
         else if (Token::Match(tok, "const %var% %var% ;") && !isExternOrStatic(tok) && isTypeName(tok->next()))
         {
             addVD(tok->tokAt(2)->varId());
         }
+
         //Critical assignment
         else if (Token::Match(tok, "[;{}] %var% = & %var%") && errorAv(tok->tokAt(1), tok->tokAt(4)))
         {
             errorAutoVariableAssignment(tok);
         }
-        else if (Token::Match(tok, "[;{}] * %var% = & %var%") && errorAv(tok->tokAt(2), tok->tokAt(5)))
+        else if (Token::Match(tok, "[;{}] * %var% = & %var%") && errorAv(tok->tokAt(2), tok->tokAt(5)) &&
+                 unknown_type.find(tok->tokAt(5)->varId()) == unknown_type.end())
         {
             errorAutoVariableAssignment(tok);
         }
@@ -303,11 +316,14 @@ void CheckAutoVariables::errorReturnPointerToLocalArray(const Token *tok)
 
 void CheckAutoVariables::errorAutoVariableAssignment(const Token *tok)
 {
-    reportError(tok, Severity::error, "autoVariables", "Wrong assignment of an auto-variable to an effective parameter of a function");
+    reportError(tok, Severity::error, "autoVariables",
+                "Assigning address of local auto-variable to a function parameter.\n"
+                "Dangerous assignment - function parameter takes the address of a local "
+                "auto-variable. Local auto-variables are reserved from the stack. And the "
+                "stack is freed when the function ends. So the pointer to a local variable "
+                "is invalid after the function ends.");
 }
 
-
-
 // return temporary?
 bool CheckAutoVariables::returnTemporary(const Token *tok) const
 {
@@ -432,13 +448,17 @@ void CheckAutoVariables::returncstr()
         }
 
         // have we reached a function that returns a reference?
-        if (Token::Match(tok, "const char * %var% ("))
+        if (Token::simpleMatch(tok, "const char *"))
         {
             // go to the '('
-            const Token *tok2 = tok->tokAt(4);
+            const Token *tok2 = tok->tokAt(3);
+            while (Token::Match(tok2, "%var% ::"))
+                tok2 = tok2->tokAt(2);
+            if (!Token::Match(tok2, "%var% ("))
+                continue;
 
             // go to the ')'
-            tok2 = tok2->link();
+            tok2 = tok2->next()->link();
 
             // is this a function implementation?
             if (Token::Match(tok2, ") const| {"))
@@ -472,7 +492,7 @@ void CheckAutoVariables::returncstr()
                             tok2 = tok2->tokAt(2);
 
                         // is it a variable declaration?
-                        if (Token::Match(tok2, "%type% %var% ;"))
+                        if (Token::Match(tok2, "%type% %var% [;=]"))
                             localvar.insert(tok2->next()->varId());
                     }
 
