@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,12 +34,12 @@ class CheckAutoVariables : public Check
 {
 public:
     /** This constructor is used when registering the CheckClass */
-    CheckAutoVariables() : Check()
+    CheckAutoVariables() : Check(myName())
     { }
 
     /** This constructor is used when running checks. */
     CheckAutoVariables(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
-        : Check(tokenizer, settings, errorLogger)
+        : Check(myName(), tokenizer, settings, errorLogger)
     { }
 
     void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
@@ -48,6 +48,7 @@ public:
         checkAutoVariables.autoVariables();
         checkAutoVariables.returnPointerToLocalArray();
         checkAutoVariables.returnReference();
+        checkAutoVariables.returncstr();
     }
 
     /** Check auto variables */
@@ -86,17 +87,18 @@ private:
     void errorReturnAutocstr(const Token *tok);
     void errorReturnTempPointer(const Token *tok);
 
-    void getErrorMessages()
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings)
     {
-        errorAutoVariableAssignment(0);
-        errorReturnPointerToLocalArray(0);
-        errorReturnReference(0);
-        errorReturnTempReference(0);
-        errorReturnAutocstr(0);
-        errorReturnTempPointer(0);
+        CheckAutoVariables c(0,settings,errorLogger);
+        c.errorAutoVariableAssignment(0);
+        c.errorReturnPointerToLocalArray(0);
+        c.errorReturnReference(0);
+        c.errorReturnTempReference(0);
+        c.errorReturnAutocstr(0);
+        c.errorReturnTempPointer(0);
     }
 
-    std::string name() const
+    std::string myName() const
     {
         return "Auto Variables";
     }
