@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include "check.h"
 #include "settings.h"
+#include "symboldatabase.h"
 
 class Token;
 
@@ -35,13 +36,11 @@ class CheckClass : public Check
 {
 public:
     /** @brief This constructor is used when registering the CheckClass */
-    CheckClass() : Check()
+    CheckClass() : Check(myName()), symbolDatabase(NULL)
     { }
 
     /** @brief This constructor is used when running checks. */
-    CheckClass(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
-        : Check(tokenizer, settings, errorLogger)
-    { }
+    CheckClass(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger);
 
     /** @brief Run checks on the normal token list */
     void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
@@ -86,7 +85,7 @@ public:
      */
     void noMemset();
 
-    /** @brief 'operator=' should return something. */
+    /** @brief 'operator=' should return something and it should not be const. */
     void operatorEq();
 
     /** @brief 'operator=' should return reference to *this */
@@ -103,80 +102,15 @@ public:
 
     /** @brief can member function be const? */
     void checkConst();
+
 private:
-
-    /** @brief Information about a member variable. Used when checking for uninitialized variables */
-    class Var
-    {
-    public:
-        Var(const std::string &name_, bool init_ = false, bool priv_ = false, bool mutable_ = false, bool static_ = false, Var *next_ = 0)
-            : name(name_),
-              init(init_),
-              priv(priv_),
-              isMutable(mutable_),
-              isStatic(static_),
-              next(next_)
-        {
-        }
-
-        /** @brief name of variable */
-        const std::string name;
-
-        /** @brief has this variable been initialized? */
-        bool        init;
-
-        /** @brief is this variable declared in the private section? */
-        bool        priv;
-
-        /** @brief is this variable mutable? */
-        bool        isMutable;
-
-        /** @brief is this variable static? */
-        bool        isStatic;
-
-        /** @brief next Var item */
-        Var *next;
-
-    private:
-        Var& operator=(const Var&); // disallow assignments
-    };
-
     /**
-     * @brief parse a scope for a constructor or member function and set the "init" flags in the provided varlist
-     * @param tok1 pointer to class declaration
-     * @param ftok pointer to the function that should be checked
-     * @param varlist variable list (the "init" flag will be set in these variables)
-     * @param classname name of class
-     * @param callstack the function doesn't look into recursive function calls.
-     * @param isStruct if this is a struct instead of a class
+     * @brief Create symbol database. For performance reasons, only call
+     * it if it's needed.
      */
-    void initializeVarList(const Token *tok1, const Token *ftok, Var *varlist, const std::string &classname, std::list<std::string> &callstack, bool isStruct);
+    void createSymbolDatabase();
 
-    /** @brief initialize a variable in the varlist */
-    void initVar(Var *varlist, const std::string &varname);
-
-    /**
-     * @brief get varlist from a class definition
-     * @param tok1 pointer to class definition
-     * @param withClasses if class variables should be extracted too.
-     * @param isStruct is this a struct?
-     */
-    Var *getVarList(const Token *tok1, bool withClasses, bool isStruct);
-
-    // Check constructors for a specified class
-    void checkConstructors(const Token *tok1, const std::string &funcname, bool hasPrivateConstructor, bool isStruct);
-
-    bool sameFunc(int nest, const Token *firstEnd, const Token *secondEnd);
-    bool isMemberFunc(const Token *tok);
-    bool isMemberVar(const std::string &classname, const Var *varlist, const Token *tok);
-    bool checkConstFunc(const std::string &classname, const Var *varlist, const Token *tok);
-
-    /**
-     * @brief Helper function for operatorEqRetRefThis that checks if there are errors
-     * @param tok The "operator" token in a operator=(.. function
-     * @param classname Name of class
-     */
-    void operatorEqRetRefThis_finderr(const Token *tok, const std::string &classname);
+    const SymbolDatabase *symbolDatabase;
 
     // Reporting errors..
     void noConstructorError(const Token *tok, const std::string &classname, bool isStruct);
@@ -190,27 +124,27 @@ private:
     void thisSubtractionError(const Token *tok);
     void operatorEqRetRefThisError(const Token *tok);
     void operatorEqToSelfError(const Token *tok);
-
     void checkConstError(const Token *tok, const std::string &classname, const std::string &funcname);
     void checkConstError2(const Token *tok1, const Token *tok2, const std::string &classname, const std::string &funcname);
 
-    void getErrorMessages()
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings)
     {
-        noConstructorError(0, "classname", false);
-        uninitVarError(0, "classname", "varname");
-        operatorEqVarError(0, "classname", "");
-        unusedPrivateFunctionError(0, "classname", "funcname");
-        memsetClassError(0, "memfunc");
-        memsetStructError(0, "memfunc", "classname");
-        operatorEqReturnError(0);
-        virtualDestructorError(0, "Base", "Derived");
-        thisSubtractionError(0);
-        operatorEqRetRefThisError(0);
-        operatorEqToSelfError(0);
-        checkConstError(0, "class", "function");
+        CheckClass c(0, settings, errorLogger);
+        c.noConstructorError(0, "classname", false);
+        c.uninitVarError(0, "classname", "varname");
+        c.operatorEqVarError(0, "classname", "");
+        c.unusedPrivateFunctionError(0, "classname", "funcname");
+        c.memsetClassError(0, "memfunc");
+        c.memsetStructError(0, "memfunc", "classname");
+        c.operatorEqReturnError(0);
+        //c.virtualDestructorError(0, "Base", "Derived");
+        c.thisSubtractionError(0);
+        c.operatorEqRetRefThisError(0);
+        c.operatorEqToSelfError(0);
+        c.checkConstError(0, "class", "function");
     }
 
-    std::string name() const
+    std::string myName() const
     {
         return "Class";
     }
@@ -221,12 +155,80 @@ private:
                "* Missing constructors\n"
                "* Are all variables initialized by the constructors?\n"
                "* [[CheckMemset|Warn if memset, memcpy etc are used on a class]]\n"
-               "* If it's a base class, check that the destructor is virtual\n"
+               //"* If it's a base class, check that the destructor is virtual\n"
                "* Are there unused private functions\n"
                "* 'operator=' should return reference to self\n"
                "* 'operator=' should check for assignment to self\n"
                "* Constness for member functions\n";
     }
+
+    // operatorEqRetRefThis helper function
+    void checkReturnPtrThis(const Scope *scope, const Function *func, const Token *tok, const Token *last);
+
+    // operatorEqToSelf helper functions
+    bool hasDeallocation(const Token *first, const Token *last);
+    bool hasAssignSelf(const Token *first, const Token *last, const Token *rhs);
+
+    // checkConst helper functions
+    bool isMemberVar(const Scope *scope, const Token *tok);
+    bool isConstMemberFunc(const Scope *scope, const Token *tok);
+    bool checkConstFunc(const Scope *scope, const Token *tok);
+    /** @brief check if this function is virtual in the base classes */
+    bool isVirtualFunc(const Scope *scope, const Token *functionToken) const;
+
+    // constructors helper function
+    /** @brief Information about a member variable. Used when checking for uninitialized variables */
+    struct Usage
+    {
+        Usage() : assign(false), init(false) { }
+
+        /** @brief has this variable been assigned? */
+        bool        assign;
+
+        /** @brief has this variable been initialized? */
+        bool        init;
+    };
+
+    bool isBaseClassFunc(const Token *tok, const Scope *scope);
+
+    /**
+     * @brief assign a variable in the varlist
+     * @param varname name of variable to mark assigned
+     * @param scope pointer to variable Scope
+     * @param usage reference to usage vector
+     */
+    void assignVar(const std::string &varname, const Scope *scope, std::vector<Usage> &usage);
+
+    /**
+     * @brief initialize a variable in the varlist
+     * @param varname name of variable to mark initialized
+     * @param scope pointer to variable Scope
+     * @param usage reference to usage vector
+     */
+    void initVar(const std::string &varname, const Scope *scope, std::vector<Usage> &usage);
+
+    /**
+     * @brief set all variables in list assigned
+     * @param usage reference to usage vector
+     */
+    void assignAllVar(std::vector<Usage> &usage);
+
+    /**
+     * @brief set all variables in list not assigned and not initialized
+     * @param usage reference to usage vector
+     */
+    void clearAllVar(std::vector<Usage> &usage);
+
+    /**
+     * @brief parse a scope for a constructor or member function and set the "init" flags in the provided varlist
+     * @param func reference to the function that should be checked
+     * @param callstack the function doesn't look into recursive function calls.
+     * @param scope pointer to variable Scope
+     * @param usage reference to usage vector
+     */
+    void initializeVarList(const Function &func, std::list<std::string> &callstack, const Scope *scope, std::vector<Usage> &usage);
+
+    bool canNotCopy(const Scope *scope) const;
 };
 /// @}
 //---------------------------------------------------------------------------
