@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,10 +17,9 @@
  */
 
 #include "checkstl.h"
-#include "tokenize.h"
 #include "token.h"
-
-
+#include "executionpath.h"
+#include <sstream>
 
 // Register this check class (by creating a static instance of it)
 namespace
@@ -46,42 +45,56 @@ void CheckStl::dereferenceErasedError(const Token *tok, const std::string &itern
     reportError(tok, Severity::error, "eraseDereference", "Dereferenced iterator '" + itername + "' has been erased");
 }
 
-void CheckStl::eraseByValueError(const Token *tok, const std::string &containername, const std::string &itername)
-{
-    reportError(tok, Severity::error, "eraseByValue", "Iterator '" +  itername + "' becomes invalid when deleted by value from '" + containername +  "'");
-}
-
 void CheckStl::iterators()
 {
+    // Using same iterator against different containers.
+    // for (it = foo.begin(); it != bar.end(); ++it)
     for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next())
     {
+        // Locate an iterator..
         if (!Token::Match(tok, "%var% = %var% . begin ( ) ;|+"))
             continue;
 
+        // Get variable ids for both the iterator and container
         const unsigned int iteratorId(tok->varId());
         const unsigned int containerId(tok->tokAt(2)->varId());
         if (iteratorId == 0 || containerId == 0)
             continue;
 
+        // the validIterator flag says if the iterator has a valid value or not
         bool validIterator = true;
+
+        // counter for { and }
         unsigned int indent = 0;
+
+        // Scan through the rest of the code and see if the iterator is
+        // used against other containers.
         for (const Token *tok2 = tok->tokAt(7); tok2; tok2 = tok2->next())
         {
+            // If a { is found then count it and continue
             if (tok2->str() == "{" && ++indent)
                 continue;
+
+            // If a } is found then count it. break if indentlevel becomes 0.
             if (tok2->str() == "}" && --indent == 0)
                 break;
 
+            // Is iterator compared against different container?
             if (Token::Match(tok2, "%varid% != %var% . end ( )", iteratorId) && tok2->tokAt(2)->varId() != containerId)
             {
                 iteratorsError(tok2, tok->strAt(2), tok2->strAt(2));
                 tok2 = tok2->tokAt(6);
             }
+
+            // Is the iterator used in a insert/erase operation?
             else if (Token::Match(tok2, "%var% . insert|erase ( %varid% )|,", iteratorId))
             {
+                // It is bad to insert/erase an invalid iterator
                 if (!validIterator)
                     invalidIteratorError(tok2, tok2->strAt(4));
 
+                // If insert/erase is used on different container then
+                // report an error
                 if (tok2->varId() != containerId && tok2->tokAt(5)->str() != ".")
                 {
                     // skip error message if container is a set..
@@ -97,18 +110,40 @@ void CheckStl::iterators()
                     // Show error message, mismatching iterator is used.
                     iteratorsError(tok2, tok->strAt(2), tok2->str());
                 }
+
+                // invalidate the iterator if it is erased
                 else if (tok2->strAt(2) == std::string("erase"))
                     validIterator = false;
 
+                // skip the operation
                 tok2 = tok2->tokAt(4);
             }
+
+            // it = foo.erase(..
+            // taking the result of an erase is ok
             else if (Token::Match(tok2, "%varid% = %var% . erase (", iteratorId))
             {
+                // the returned iterator is valid
                 validIterator = true;
+
+                // skip the operation
                 tok2 = tok2->tokAt(5)->link();
                 if (!tok2)
                     break;
             }
+
+            // Reassign the iterator
+            else if (Token::Match(tok2, "%varid% = %var% ;", iteratorId))
+            {
+                // Assume that the iterator becomes valid.
+                // TODO: add checking that checks if the iterator becomes valid or not
+                validIterator = true;
+
+                // skip ahead
+                tok2 = tok2->tokAt(2);
+            }
+
+            // Dereferencing invalid iterator?
             else if (!validIterator && Token::Match(tok2, "* %varid%", iteratorId))
             {
                 dereferenceErasedError(tok2, tok2->strAt(1));
@@ -121,12 +156,22 @@ void CheckStl::iterators()
             }
             else if (Token::Match(tok2, "%var% . erase ( * %varid%", iteratorId) && tok2->varId() == containerId)
             {
-                eraseByValueError(tok2, tok2->strAt(0), tok2->strAt(5));
+//                eraseByValueError(tok2, tok2->strAt(0), tok2->strAt(5));
             }
+
+            // bailout handling. Assume that the iterator becomes valid if we see return/break.
+            // TODO: better handling
             else if (Token::Match(tok2, "return|break ;"))
             {
                 validIterator = true;
             }
+
+            // bailout handling. Assume that the iterator becomes valid if we see else.
+            // TODO: better handling
+            else if (tok2->str() == "else")
+            {
+                validIterator = true;
+            }
         }
     }
 }
@@ -140,11 +185,13 @@ void CheckStl::mismatchingContainersError(const Token *tok)
 
 void CheckStl::mismatchingContainers()
 {
+    // Check if different containers are used in various calls of standard functions
     for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next())
     {
         if (tok->str() != "std")
             continue;
 
+        // TODO: If iterator variables are used instead then there are false negatives.
         if (Token::Match(tok, "std :: find|find_if|count|transform|replace|replace_if|sort ( %var% . begin|rbegin ( ) , %var% . end|rend ( ) ,"))
         {
             if (tok->tokAt(4)->str() != tok->tokAt(10)->str())
@@ -158,11 +205,14 @@ void CheckStl::mismatchingContainers()
 
 void CheckStl::stlOutOfBounds()
 {
+    // Scan through all tokens..
     for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next())
     {
+        // only interested in "for" loops
         if (!Token::simpleMatch(tok, "for ("))
             continue;
 
+        // check if the for loop condition is wrong
         unsigned int indent = 0;
         for (const Token *tok2 = tok->tokAt(2); tok2; tok2 = tok2->next())
         {
@@ -179,18 +229,24 @@ void CheckStl::stlOutOfBounds()
 
             if (Token::Match(tok2, "; %var% <= %var% . size ( ) ;"))
             {
-                unsigned int indent2 = 0;
+                // Count { and } for tok3
+                unsigned int indent3 = 0;
+
+                // variable id for loop variable.
                 unsigned int numId = tok2->tokAt(1)->varId();
+
+                // variable id for the container variable
                 unsigned int varId = tok2->tokAt(3)->varId();
+
                 for (const Token *tok3 = tok2->tokAt(8); tok3; tok3 = tok3->next())
                 {
                     if (tok3->str() == "{")
-                        ++indent2;
+                        ++indent3;
                     else if (tok3->str() == "}")
                     {
-                        if (indent2 <= 1)
+                        if (indent3 <= 1)
                             break;
-                        --indent2;
+                        --indent3;
                     }
                     else if (tok3->varId() == varId)
                     {
@@ -213,6 +269,162 @@ void CheckStl::stlOutOfBoundsError(const Token *tok, const std::string &num, con
 }
 
 
+/**
+ * @brief %Check for invalid iterator usage after erase/insert/etc
+ */
+class EraseCheckLoop : public ExecutionPath
+{
+public:
+    static void checkScope(CheckStl *checkStl, const Token *it)
+    {
+        const Token *tok = it;
+
+        // Search for the start of the loop body..
+        int indentlevel = 1;
+        while (indentlevel > 0 && 0 != (tok = tok->next()))
+        {
+            if (tok->str() == "(")
+                tok = tok->link();
+            else if (tok->str() == ")")
+                break;
+
+            // reassigning iterator in loop head
+            else if (Token::Match(tok, "%var% =") && tok->str() == it->str())
+                break;
+        }
+
+        if (! Token::simpleMatch(tok, ") {"))
+            return;
+
+        EraseCheckLoop c(checkStl, it->varId());
+        std::list<ExecutionPath *> checks;
+        checks.push_back(c.copy());
+        ExecutionPath::checkScope(tok->tokAt(2), checks);
+
+        c.end(checks, tok->link());
+
+        while (!checks.empty())
+        {
+            delete checks.back();
+            checks.pop_back();
+        }
+    }
+
+private:
+    /** Startup constructor */
+    EraseCheckLoop(Check *o, unsigned int varid)
+        : ExecutionPath(o, varid), eraseToken(0)
+    {
+    }
+
+    /** @brief token where iterator is erased (non-zero => the iterator is invalid) */
+    const Token *eraseToken;
+
+    /** @brief Copy this check. Called from the ExecutionPath baseclass. */
+    ExecutionPath *copy()
+    {
+        return new EraseCheckLoop(*this);
+    }
+
+    /** @brief is another execution path equal? */
+    bool is_equal(const ExecutionPath *e) const
+    {
+        const EraseCheckLoop *c = static_cast<const EraseCheckLoop *>(e);
+        return (eraseToken == c->eraseToken);
+    }
+
+    /** @brief no implementation => compiler error if used by accident */
+    void operator=(const EraseCheckLoop &);
+
+    /** @brief parse tokens */
+    const Token *parse(const Token &tok, std::list<ExecutionPath *> &checks) const
+    {
+        // bail out if there are assignments. We don't check the assignments properly.
+        if (Token::Match(&tok, "[;{}] %var% =") || Token::Match(&tok, "= %var% ;"))
+        {
+            ExecutionPath::bailOutVar(checks, tok.next()->varId());
+        }
+
+        // the loop stops here. Bail out all execution checks that reach
+        // this statement
+        if (Token::Match(&tok, "[;{}] break ;"))
+        {
+            ExecutionPath::bailOut(checks);
+        }
+
+        // erasing iterator => it is invalidated
+        if (Token::Match(&tok, "erase ( ++|--| %var% )"))
+        {
+            // check if there is a "it = ints.erase(it);" pattern. if so
+            // the it is not invalidated.
+            const Token *token = &tok;
+            while (NULL != (token = token ? token->previous() : 0))
+            {
+                if (Token::Match(token, "[;{}]"))
+                    break;
+                else if (token->str() == "=")
+                    token = 0;
+            }
+
+            // the it is invalidated by the erase..
+            if (token)
+            {
+                // get variable id for the iterator
+                unsigned int iteratorId = 0;
+                if (tok.tokAt(2)->isName())
+                    iteratorId = tok.tokAt(2)->varId();
+                else
+                    iteratorId = tok.tokAt(3)->varId();
+
+                // invalidate this iterator in the corresponding checks
+                for (std::list<ExecutionPath *>::const_iterator it = checks.begin(); it != checks.end(); ++it)
+                {
+                    EraseCheckLoop *c = dynamic_cast<EraseCheckLoop *>(*it);
+                    if (c && c->varId == iteratorId)
+                    {
+                        c->eraseToken = &tok;
+                    }
+                }
+            }
+        }
+
+        // don't skip any tokens. return the token that we received.
+        return &tok;
+    }
+
+    /**
+     * Parse condition. @sa ExecutionPath::parseCondition
+     * @param tok first token in condition.
+     * @param checks The execution paths. All execution paths in the list are executed in the current scope
+     * @return true => bail out all checking
+     **/
+    bool parseCondition(const Token &tok, std::list<ExecutionPath *> &checks)
+    {
+        // no checking of conditions.
+        (void)tok;
+        (void)checks;
+        return false;
+    }
+
+    /** @brief going out of scope - all execution paths end */
+    void end(const std::list<ExecutionPath *> &checks, const Token * /*tok*/) const
+    {
+        // check if there are any invalid iterators. If so there is an error.
+        for (std::list<ExecutionPath *>::const_iterator it = checks.begin(); it != checks.end(); ++it)
+        {
+            EraseCheckLoop *c = dynamic_cast<EraseCheckLoop *>(*it);
+            if (c && c->eraseToken)
+            {
+                CheckStl *checkStl = dynamic_cast<CheckStl *>(c->owner);
+                if (checkStl)
+                {
+                    checkStl->eraseError(c->eraseToken);
+                }
+            }
+        }
+    }
+};
+
 
 void CheckStl::erase()
 {
@@ -228,16 +440,16 @@ void CheckStl::erase()
                     {
                         const unsigned int varid = tok2->next()->varId();
                         if (varid > 0 && Token::findmatch(_tokenizer->tokens(), "> :: iterator %varid%", varid))
-                            eraseCheckLoop(tok2->next());
+                            EraseCheckLoop::checkScope(this, tok2->next());
                     }
                     break;
                 }
 
-                if (Token::Match(tok2, "%var% = %var% . begin ( ) ; %var% != %var% . end ( ) ") &&
+                if (Token::Match(tok2, "%var% = %var% . begin ( ) ; %var% != %var% . end ( )") &&
                     tok2->str() == tok2->tokAt(8)->str() &&
                     tok2->tokAt(2)->str() == tok2->tokAt(10)->str())
                 {
-                    eraseCheckLoop(tok2);
+                    EraseCheckLoop::checkScope(this, tok2);
                     break;
                 }
             }
@@ -247,67 +459,20 @@ void CheckStl::erase()
         {
             const unsigned int varid = tok->tokAt(2)->varId();
             if (varid > 0 && Token::findmatch(_tokenizer->tokens(), "> :: iterator %varid%", varid))
-                eraseCheckLoop(tok->tokAt(2));
+                EraseCheckLoop::checkScope(this, tok->tokAt(2));
         }
     }
 }
 
-void CheckStl::eraseCheckLoop(const Token *it)
-{
-    const Token *tok = it;
-
-    // Search for the start of the loop body..
-    int indentlevel = 1;
-    while (indentlevel > 0 && 0 != (tok = tok->next()))
-    {
-        if (tok->str() == "(")
-            ++indentlevel;
-        else if (tok->str() == ")")
-            --indentlevel;
-    }
-
-    if (! Token::simpleMatch(tok, ") {"))
-        return;
-
-    // Parse loop..
-    // Error if it contains "erase(it)" but neither "break;", "=it" nor "it="
-    indentlevel = 0;
-    const Token *tok2 = 0;
-    while (0 != (tok = tok->next()))
-    {
-        if (tok->str() == "{")
-            ++indentlevel;
-        else if (tok->str() == "}")
-        {
-            --indentlevel;
-            if (indentlevel <= 0)
-                break;
-        }
-        else if (Token::Match(tok, "break|return|goto") ||
-                 Token::simpleMatch(tok, (it->str() + " =").c_str()) ||
-                 Token::simpleMatch(tok, ("= " + it->str()).c_str()))
-        {
-            tok2 = 0;
-            break;
-        }
-        else if (Token::simpleMatch(tok, ("erase ( " + it->str() + " )").c_str()))
-            tok2 = tok;
-    }
-
-    // Write error message..
-    if (tok2)
-        eraseError(tok2);
-}
-
-
 // Error message for bad iterator usage..
 void CheckStl::eraseError(const Token *tok)
 {
-    reportError(tok, Severity::error, "erase", "Dangerous iterator usage. After erase the iterator is invalid so dereferencing it or comparing it with another iterator is invalid.");
+    reportError(tok, Severity::error, "erase",
+                "Dangerous iterator usage after erase()-method.\n"
+                "The iterator is invalid after it has been used in erase() function. "
+                "Dereferencing or comparing it with another iterator is invalid operation.");
 }
 
-
-
 void CheckStl::pushback()
 {
     // Pointer can become invalid after push_back or push_front..
@@ -315,11 +480,16 @@ void CheckStl::pushback()
     {
         if (Token::Match(tok, "%var% = & %var% ["))
         {
+            // Variable id for pointer
             const unsigned int pointerId(tok->varId());
+
+            // Variable id for the container variable
             const unsigned int containerId(tok->tokAt(3)->varId());
+
             if (pointerId == 0 || containerId == 0)
                 continue;
 
+            // Count { , } and parantheses for tok2
             int indent = 0;
             bool invalidPointer = false;
             for (const Token *tok2 = tok; indent >= 0 && tok2; tok2 = tok2->next())
@@ -382,8 +552,12 @@ void CheckStl::pushback()
             tok = tok->tokAt(3);
         }
 
+        // the variable id for the vector
         unsigned int vectorid = 0;
+
+        // count { , } and parantheses for tok2
         int indent = 0;
+
         std::string invalidIterator;
         for (const Token *tok2 = tok; indent >= 0 && tok2; tok2 = tok2->next())
         {
@@ -398,18 +572,21 @@ void CheckStl::pushback()
             }
 
             // Using push_back or push_front inside a loop..
-            if (Token::Match(tok2, "for ("))
+            if (Token::simpleMatch(tok2, "for ("))
             {
                 tok2 = tok2->tokAt(2);
             }
 
             if (Token::Match(tok2, "%varid% = %var% . begin ( ) ; %varid% != %var% . end ( ) ; ++| %varid% ++| ) {", iteratorid))
             {
+                // variable id for the loop iterator
                 const unsigned int varId(tok2->tokAt(2)->varId());
                 if (varId == 0)
                     continue;
 
                 const Token *pushbackTok = 0;
+
+                // Count { and } for tok3
                 unsigned int indent3 = 0;
                 for (const Token *tok3 = tok2->tokAt(20); tok3; tok3 = tok3->next())
                 {
@@ -421,7 +598,7 @@ void CheckStl::pushback()
                             break;
                         --indent3;
                     }
-                    else if (tok3->str() == "break")
+                    else if (tok3->str() == "break" || tok3->str() == "return")
                     {
                         pushbackTok = 0;
                         break;
@@ -461,15 +638,10 @@ void CheckStl::pushback()
                 }
 
                 invalidIterator = tok2->strAt(2);
-                if (!iteratorDeclaredInsideLoop)
-                {
-                    tok2 = tok2->tokAt(3)->link();
-                    if (!tok2)
-                        break;
-                }
+                tok2 = tok2->tokAt(3)->link();
             }
 
-            else if (tok2->str() == "return")
+            else if (tok2->str() == "return" || tok2->str() == "break")
             {
                 invalidIterator.clear();
             }
@@ -511,10 +683,9 @@ void CheckStl::stlBoundries()
     for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next())
     {
         // Declaring iterator..
-        const std::string checkStr = (std::string(STL_CONTAINER_LIST) + " <");
-        if (Token::Match(tok, checkStr.c_str()))
+        if (tok->str() == "<" && Token::Match(tok->previous(), STL_CONTAINER_LIST))
         {
-            const std::string container_name(tok->strAt(0));
+            const std::string container_name(tok->strAt(-1));
             while (tok && tok->str() != ">")
                 tok = tok->next();
             if (!tok)
@@ -551,11 +722,14 @@ void CheckStl::stlBoundries()
 // Error message for bad boundry usage..
 void CheckStl::stlBoundriesError(const Token *tok, const std::string &container_name)
 {
-    reportError(tok, Severity::error, "stlBoundries", container_name + " range check should use != and not < since the order of the pointers isn't guaranteed");
+    reportError(tok, Severity::error, "stlBoundries",
+                "Dangerous container iterator compare using < operator for " + container_name + "\n"
+                "Container '" + container_name + "' iterator compared with < operator. "
+                "Using < operator with container type iterators is dangerous since the order of "
+                "the items is not guaranteed. One should use != operator instead when comparing "
+                "iterators in the container.");
 }
 
-
-
 void CheckStl::if_find()
 {
     if (!_settings->_checkCodingStyle)
@@ -607,9 +781,14 @@ void CheckStl::if_find()
 void CheckStl::if_findError(const Token *tok, bool str)
 {
     if (str)
-        reportError(tok, Severity::style, "stlIfStrFind", "Suspicious condition. string::find will return 0 if the string is found at position 0. If this is what you want to check then string::compare is a faster alternative because it doesn't scan through the string.");
+        reportError(tok, Severity::warning, "stlIfStrFind",
+                    "Suspicious checking of string::find() return value.\n"
+                    "Checking of string::find() return value looks Suspicious. "
+                    "string::find will return 0 if the string is found at position 0. "
+                    "If that is wanted to check then string::compare is a faster alternative "
+                    "because it doesn't scan through the string.");
     else
-        reportError(tok, Severity::style, "stlIfFind", "Suspicious condition. The result of find is an iterator, but it is not properly checked.");
+        reportError(tok, Severity::warning, "stlIfFind", "Suspicious condition. The result of find is an iterator, but it is not properly checked.");
 }
 
 
@@ -691,6 +870,193 @@ void CheckStl::size()
 void CheckStl::sizeError(const Token *tok)
 {
     const std::string varname(tok ? tok->str().c_str() : "list");
-    const bool verbose(_settings ? _settings->_verbose : true);
-    reportError(tok, Severity::style, "stlSize", "Use " + varname + ".empty() instead of " + varname + ".size() to guarantee fast code." + (verbose ? " size() can take linear time but empty() is guaranteed to take constant time." : ""));
+    reportError(tok, Severity::performance, "stlSize",
+                "Possible inefficient checking for '" + varname + "' emptiness.\n"
+                "Checking for '" + varname + "' emptiness might be inefficient. "
+                "Using " + varname + ".empty() instead of " + varname + ".size() can be faster. " +
+                varname + ".size() can take linear time but " + varname + ".empty() is "
+                "guaranteed to take constant time.");
 }
+
+void CheckStl::redundantCondition()
+{
+    const char pattern[] = "if ( %var% . find ( %any% ) != %var% . end ( ) ) "
+                           "{|{|"
+                           "    %var% . remove ( %any% ) ; "
+                           "}|}|";
+    const Token *tok = Token::findmatch(_tokenizer->tokens(), pattern);
+    while (tok)
+    {
+        bool b(tok->tokAt(15)->str() == "{");
+
+        // Get tokens for the fields %var% and %any%
+        const Token *var1 = tok->tokAt(2);
+        const Token *any1 = tok->tokAt(6);
+        const Token *var2 = tok->tokAt(9);
+        const Token *var3 = tok->tokAt(b ? 16 : 15);
+        const Token *any2 = tok->tokAt(b ? 20 : 19);
+
+        // Check if all the "%var%" fields are the same and if all the "%any%" are the same..
+        if (var1->str() == var2->str() &&
+            var2->str() == var3->str() &&
+            any1->str() == any2->str())
+        {
+            redundantIfRemoveError(tok);
+        }
+
+        tok = Token::findmatch(tok->next(), pattern);
+    }
+}
+
+void CheckStl::redundantIfRemoveError(const Token *tok)
+{
+    reportError(tok, Severity::style, "redundantIfRemove",
+                "Redundant checking of STL container element.\n"
+                "Redundant checking of STL container element existence before removing it. "
+                "The remove method in the STL will not do anything if element doesn't exist");
+}
+
+void CheckStl::missingComparison()
+{
+    for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next())
+    {
+        if (Token::simpleMatch(tok, "for ("))
+        {
+            for (const Token *tok2 = tok->tokAt(2); tok2; tok2 = tok2->next())
+            {
+                if (tok2->str() == ";")
+                    break;
+
+                if (!Token::Match(tok2, "%var% = %var% . begin ( ) ; %var% != %var% . end ( ) ; ++| %var% ++| ) {"))
+                    continue;
+
+                // same iterator name
+                if (tok2->str() != tok2->strAt(8))
+                    continue;
+
+                // same container
+                if (tok2->strAt(2) != tok2->strAt(10))
+                    continue;
+
+                // increment iterator
+                if (!Token::simpleMatch(tok2->tokAt(16), ("++ " + tok2->str() + " )").c_str()) &&
+                    !Token::simpleMatch(tok2->tokAt(16), (tok2->str() + " ++ )").c_str()))
+                {
+                    continue;
+                }
+
+                const unsigned int &iteratorId(tok2->varId());
+                if (iteratorId == 0)
+                    continue;
+
+                const Token *incrementToken = 0;
+
+                // Count { and } for tok3
+                unsigned int indentlevel = 0;
+
+                // Parse loop..
+                for (const Token *tok3 = tok2->tokAt(20); tok3; tok3 = tok3->next())
+                {
+                    if (tok3->str() == "{")
+                        ++indentlevel;
+                    else if (tok3->str() == "}")
+                    {
+                        if (indentlevel == 0)
+                            break;
+                        --indentlevel;
+                    }
+                    else if (Token::Match(tok3, "%varid% ++", iteratorId))
+                        incrementToken = tok3;
+                    else if (Token::Match(tok3->previous(), "++ %varid% !!.", iteratorId))
+                        incrementToken = tok3;
+                    else if (Token::Match(tok3, "%varid% !=|==", iteratorId))
+                        incrementToken = 0;
+                    else if (tok3->str() == "break" || tok3->str() == "return")
+                        incrementToken = 0;
+                }
+                if (incrementToken)
+                    missingComparisonError(incrementToken, tok2->tokAt(16));
+            }
+        }
+    }
+}
+
+void CheckStl::missingComparisonError(const Token *incrementToken1, const Token *incrementToken2)
+{
+    std::ostringstream errmsg;
+    errmsg << "Missing bounds check for extra iterator increment in loop.\n"
+           << "The iterator incrementing is suspicious - it is incremented at line "
+           << incrementToken1->linenr() << " and then at line " << incrementToken2->linenr()
+           << " The loop might unintentionally skip an element in the container. "
+           << "There is no comparison between these increments to prevent that the iterator is "
+           << "incremented beyond the end.";
+
+    reportError(incrementToken1, Severity::warning, "StlMissingComparison", errmsg.str());
+}
+
+
+void CheckStl::string_c_str()
+{
+    // Try to detect common problems when using string::c_str()
+    for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next())
+    {
+        // Locate executable scopes:
+        if (Token::Match(tok, ") const| {"))
+        {
+            std::set<unsigned int> localvar;
+            std::set<unsigned int> pointers;
+
+            // scan through this executable scope:
+            unsigned int indentlevel = 0;
+            while (NULL != (tok = tok->next()))
+            {
+                if (tok->str() == "{")
+                    ++indentlevel;
+                else if (tok->str() == "}")
+                {
+                    if (indentlevel <= 1)
+                        break;
+                    --indentlevel;
+                }
+
+                // Variable declarations..
+                else if (Token::Match(tok->previous(), "[;{}] std :: %type% %var% ;"))
+                    localvar.insert(tok->tokAt(3)->varId());
+                else if (Token::Match(tok->previous(), "[;{}] %type% %var% ;"))
+                    localvar.insert(tok->next()->varId());
+                else if (Token::Match(tok->previous(), "[;{}] %type% * %var% ;"))
+                    pointers.insert(tok->tokAt(2)->varId());
+                else if (Token::Match(tok->previous(), "[;{}] %type% %type% * %var% ;"))
+                    pointers.insert(tok->tokAt(3)->varId());
+
+                // Invalid usage..
+                else if (Token::Match(tok, "throw %var% . c_str ( ) ;") &&
+                         tok->next()->varId() > 0 &&
+                         localvar.find(tok->next()->varId()) != localvar.end())
+                {
+                    string_c_strError(tok);
+                }
+                else if (Token::Match(tok, "[;{}] %var% = %var% . str ( ) . c_str ( ) ;") &&
+                         tok->next()->varId() > 0 &&
+                         pointers.find(tok->next()->varId()) != pointers.end())
+                {
+                    string_c_strError(tok);
+                }
+                else if (Token::Match(tok, "[;{}] %var% = %var% (") &&
+                         Token::simpleMatch(tok->tokAt(4)->link(), ") . c_str ( ) ;") &&
+                         tok->next()->varId() > 0 &&
+                         pointers.find(tok->next()->varId()) != pointers.end() &&
+                         Token::findmatch(_tokenizer->tokens(), ("std :: string " + tok->strAt(3) + " (").c_str()))
+                {
+                    string_c_strError(tok);
+                }
+            }
+        }
+    }
+}
+
+void CheckStl::string_c_strError(const Token *tok)
+{
+    reportError(tok, Severity::error, "stlcstr", "Dangerous usage of c_str()");
+}
+
