@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,14 +35,15 @@ class CheckStl : public Check
 {
 public:
     /** This constructor is used when registering the CheckClass */
-    CheckStl() : Check()
+    CheckStl() : Check(myName())
     { }
 
     /** This constructor is used when running checks. */
     CheckStl(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
-        : Check(tokenizer, settings, errorLogger)
+        : Check(myName(), tokenizer, settings, errorLogger)
     { }
 
+    /** Simplified checks. The token list is simplified. */
     void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
     {
         CheckStl checkStl(tokenizer, settings, errorLogger);
@@ -54,9 +55,12 @@ public:
         checkStl.pushback();
         checkStl.stlBoundries();
         checkStl.if_find();
+        checkStl.string_c_str();
 
         // Style check
         checkStl.size();
+        checkStl.redundantCondition();
+        checkStl.missingComparison();
     }
 
 
@@ -78,13 +82,20 @@ public:
      */
     void mismatchingContainers();
 
-    /** Dereferencing an erased iterator */
+    /**
+     * Dereferencing an erased iterator
+     * @param tok token where error occurs
+     * @param itername iterator name
+     */
     void dereferenceErasedError(const Token *tok, const std::string &itername);
 
     /**
-     * Dangerous usage of erase
+     * Dangerous usage of erase. The iterator is invalidated by erase so
+     * it is bad to dereference it after the erase.
      */
     void erase();
+    void eraseError(const Token *tok);
+
 
     /**
      * Dangerous usage of push_back and insert
@@ -105,6 +116,24 @@ public:
      */
     void size();
 
+    /**
+     * Check for redundant condition 'if (ints.find(1) != ints.end()) ints.remove(123);'
+     * */
+    void redundantCondition();
+
+    /**
+     * @brief Missing inner comparison, when incrementing iterator inside loop
+     * Dangers:
+     *  - may increment iterator beyond end
+     *  - may unintentionally skip elements in list/set etc
+     */
+    void missingComparison();
+    void missingComparisonError(const Token *incrementToken1, const Token *incrementToken2);
+
+    /** Check for common mistakes when using the function string::c_str() */
+    void string_c_str();
+    void string_c_strError(const Token *tok);
+
 private:
 
     /**
@@ -118,32 +147,33 @@ private:
     void invalidIteratorError(const Token *tok, const std::string &iteratorName);
     void iteratorsError(const Token *tok, const std::string &container1, const std::string &container2);
     void mismatchingContainersError(const Token *tok);
-    void eraseError(const Token *tok);
     void invalidIteratorError(const Token *tok, const std::string &func, const std::string &iterator_name);
     void invalidPointerError(const Token *tok, const std::string &pointer_name);
     void stlBoundriesError(const Token *tok, const std::string &container_name);
     void if_findError(const Token *tok, bool str);
     void sizeError(const Token *tok);
-    void eraseByValueError(const Token *tok, const std::string &containername, const std::string &itername);
+    void redundantIfRemoveError(const Token *tok);
 
-    void getErrorMessages()
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings)
     {
-        invalidIteratorError(0, "iterator");
-        iteratorsError(0, "container1", "container2");
-        mismatchingContainersError(0);
-        dereferenceErasedError(0, "iter");
-        stlOutOfBoundsError(0, "i", "foo");
-        eraseError(0);
-        invalidIteratorError(0, "push_back|push_front|insert", "iterator");
-        invalidPointerError(0, "pointer");
-        stlBoundriesError(0, "container");
-        if_findError(0, false);
-        if_findError(0, true);
-        sizeError(0);
-        eraseByValueError(0, "container", "iterator");
+        CheckStl c(0, settings, errorLogger);
+        c.invalidIteratorError(0, "iterator");
+        c.iteratorsError(0, "container1", "container2");
+        c.mismatchingContainersError(0);
+        c.dereferenceErasedError(0, "iter");
+        c.stlOutOfBoundsError(0, "i", "foo");
+        c.eraseError(0);
+        c.invalidIteratorError(0, "push_back|push_front|insert", "iterator");
+        c.invalidPointerError(0, "pointer");
+        c.stlBoundriesError(0, "container");
+        c.if_findError(0, false);
+        c.if_findError(0, true);
+        c.string_c_strError(0);
+        c.sizeError(0);
+        c.redundantIfRemoveError(0);
     }
 
-    std::string name() const
+    std::string myName() const
     {
         return "STL usage";
     }
@@ -157,7 +187,9 @@ private:
                "* dereferencing an erased iterator\n"
                "* for vectors: using iterator/pointer after push_back has been used\n"
                "* optimisation: use empty() instead of size() to guarantee fast code\n"
-               "* suspicious condition when using find\n";
+               "* suspicious condition when using find\n"
+               "* redundant condition\n"
+               "* common mistakes when using string::c_str()";
     }
 
     bool isStlContainer(const Token *tok);
