@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -90,7 +90,7 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer)
             // Multiple files => filename = "+"
             else if (func.filename != tokenizer.getFiles()->at(0))
             {
-                func.filename = "+";
+                //func.filename = "+";
                 func.usedOtherFile |= func.usedSameFile;
             }
         }
@@ -115,7 +115,7 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer)
         else
             continue;
 
-        // funcname ( => Assert that the end paranthesis isn't followed by {
+        // funcname ( => Assert that the end parenthesis isn't followed by {
         if (Token::Match(funcname, "%var% ("))
         {
             int parlevel = 0;
@@ -187,12 +187,12 @@ void CheckUnusedFunctions::unusedFunctionError(ErrorLogger * const errorLogger,
     if (!filename.empty())
     {
         ErrorLogger::ErrorMessage::FileLocation fileLoc;
-        fileLoc.file = filename;
+        fileLoc.setfile(filename);
         fileLoc.line = 1;
         locationList.push_back(fileLoc);
     }
 
-    const ErrorLogger::ErrorMessage errmsg(locationList, Severity::stringify(Severity::style), "The function '" + funcname + "' is never used", "unusedFunction");
+    const ErrorLogger::ErrorMessage errmsg(locationList, Severity::style, "The function '" + funcname + "' is never used", "unusedFunction");
     if (errorLogger)
         errorLogger->reportErr(errmsg);
     else
