@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,14 +19,14 @@
 #ifndef CPPCHECK_H
 #define CPPCHECK_H
 
+#include "settings.h"
+#include "errorlogger.h"
+#include "checkunusedfunctions.h"
+
 #include <string>
 #include <list>
 #include <sstream>
 #include <vector>
-#include <map>
-#include "settings.h"
-#include "errorlogger.h"
-#include "checkunusedfunctions.h"
 
 /// @addtogroup Core
 /// @{
@@ -74,13 +74,11 @@ public:
     /**
      * @brief Add new file to be checked.
      *
-     * @param path Relative or absolute path to the file to be checked,
+     * @param filepath Relative or absolute path to the file to be checked,
      * e.g. "cppcheck.cpp". Note that only source files (.c, .cc or .cpp)
      * should be added to the list. Include files are gathered automatically.
-     * You can also give path, e.g. "src/" which will be scanned for source
-     * files recursively.
      */
-    void addFile(const std::string &path);
+    void addFile(const std::string &filepath);
 
     /**
      * @brief Add new unreal file to be checked.
@@ -96,16 +94,6 @@ public:
      */
     void clearFiles();
 
-    /**
-     * @brief Parse command line args and get settings and file lists
-     * from there.
-     *
-     * @param argc argc from main()
-     * @param argv argv from main()
-     * @return false when errors are found in the input
-     */
-    bool parseFromArgs(int argc, const char* const argv[]);
-
     /**
      * @brief Returns current version number as a string.
      * @return version, e.g. "1.38"
@@ -126,7 +114,7 @@ public:
 
     /**
      * @brief Call all "getErrorMessages" in all registered Check classes.
-     * Also print out xml header and footer.
+     * Also print out XML header and footer.
      */
     void getErrorMessages();
 
@@ -161,6 +149,8 @@ private:
     Settings _settings;
     std::vector<std::string> _filenames;
 
+    void reportProgress(const std::string &filename, const char stage[], const unsigned int value);
+
     /** @brief Key is file name, and value is the content of the file */
     std::map<std::string, std::string> _fileContents;
 
@@ -169,8 +159,6 @@ private:
 
     /** @brief Current preprocessor configuration */
     std::string     cfg;
-
-    std::list<std::string> _xmllist;
 };
 
 /// @}
