@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,30 +17,59 @@
  */
 
 #include "errorlogger.h"
-#include "tokenize.h"
-#include "token.h"
+#include "path.h"
+#include "cppcheck.h"
 
 #include <sstream>
+#include <vector>
 
 ErrorLogger::ErrorMessage::ErrorMessage()
+    :_severity(Severity::none)
 {
 
 }
-#include <iostream>
-ErrorLogger::ErrorMessage::ErrorMessage(const std::list<FileLocation> &callStack, const std::string &severity, const std::string &msg, const std::string &id)
+
+ErrorLogger::ErrorMessage::ErrorMessage(const std::list<FileLocation> &callStack, Severity::SeverityType severity, const std::string &msg, const std::string &id)
 {
+    // locations for this error message
     _callStack = callStack;
+
+    // severity for this error message
     _severity = severity;
-    _msg = msg;
+
+    // set the summary and verbose messages
+    setmsg(msg);
+
+    // set the message id
     _id = id;
 }
 
+void ErrorLogger::ErrorMessage::setmsg(const std::string &msg)
+{
+    // The summary and verbose message are separated by a newline
+    // If there is no newline then both the summary and verbose messages
+    // are the given message
+    const std::string::size_type pos = msg.find("\n");
+    if (pos == std::string::npos)
+    {
+        _shortMessage = msg;
+        _verboseMessage = msg;
+    }
+    else
+    {
+        _shortMessage = msg.substr(0, pos);
+        _verboseMessage = msg.substr(pos + 1);
+    }
+}
+
 std::string ErrorLogger::ErrorMessage::serialize() const
 {
+    // Serialize this message into a simple string
     std::ostringstream oss;
     oss << _id.length() << " " << _id;
-    oss << _severity.length() << " " << _severity;
-    oss << _msg.length() << " " << _msg;
+    oss << Severity::toString(_severity).length() << " " << Severity::toString(_severity);
+    oss << _shortMessage.length() << " " << _shortMessage;
+    oss << _verboseMessage.length() << " " << _verboseMessage;
     oss << _callStack.size() << " ";
 
     for (std::list<ErrorLogger::ErrorMessage::FileLocation>::const_iterator tok = _callStack.begin(); tok != _callStack.end(); ++tok)
@@ -49,6 +78,7 @@ std::string ErrorLogger::ErrorMessage::serialize() const
         smallStream << (*tok).line << ":" << (*tok).getfile();
         oss << smallStream.str().length() << " " << smallStream.str();
     }
+
     return oss.str();
 }
 
@@ -72,13 +102,14 @@ bool ErrorLogger::ErrorMessage::deserialize(const std::string &data)
         }
 
         results.push_back(temp);
-        if (results.size() == 3)
+        if (results.size() == 4)
             break;
     }
 
     _id = results[0];
-    _severity = results[1];
-    _msg = results[2];
+    _severity = Severity::fromString(results[1]);
+    _shortMessage = results[2];
+    _verboseMessage = results[3];
 
     unsigned int stackSize = 0;
     if (!(iss >> stackSize))
@@ -99,7 +130,7 @@ bool ErrorLogger::ErrorMessage::deserialize(const std::string &data)
         }
 
         ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.file = temp.substr(temp.find(':') + 1);
+        loc.setfile(temp.substr(temp.find(':') + 1));
         temp = temp.substr(0, temp.find(':'));
         std::istringstream fiss(temp);
         fiss >> loc.line;
@@ -113,21 +144,41 @@ bool ErrorLogger::ErrorMessage::deserialize(const std::string &data)
     return true;
 }
 
-std::string ErrorLogger::ErrorMessage::getXMLHeader()
+std::string ErrorLogger::ErrorMessage::getXMLHeader(int xml_version)
 {
-    return  "<?xml version=\"1.0\"?>\n"
-            "<results>";
+    // xml_version 1 is the default xml format
+
+    // standard xml header
+    std::ostringstream ostr;
+    ostr << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
+
+    // version 1 header
+    if (xml_version <= 1)
+    {
+        ostr << "<results>";
+    }
+
+    // version 2 header
+    else
+    {
+        ostr << "<results version=\"" << xml_version << "\">\n";
+        ostr << "  <cppcheck version=\"" << CppCheck::version() << "\"/>\n";
+        ostr << "  <errors>";
+    }
+
+    return ostr.str();
 }
 
-std::string ErrorLogger::ErrorMessage::getXMLFooter()
+std::string ErrorLogger::ErrorMessage::getXMLFooter(int xml_version)
 {
-    return "</results>";
+    return (xml_version<=1) ? "</results>" : "  </errors>\n</results>";
 }
 
 static std::string stringToXml(std::string s)
 {
+    // convert a string so it can be save as xml attribute data
     std::string::size_type pos = 0;
-    while ((pos = s.find_first_of("<>&\"", pos)) != std::string::npos)
+    while ((pos = s.find_first_of("<>&\"\n", pos)) != std::string::npos)
     {
         if (s[pos] == '<')
             s.insert(pos + 1, "&lt;");
@@ -137,25 +188,55 @@ static std::string stringToXml(std::string s)
             s.insert(pos + 1, "&amp;");
         else if (s[pos] == '"')
             s.insert(pos + 1, "&quot;");
+        else if (s[pos] == '\n')
+            s.insert(pos + 1, "&#xa;");
         s.erase(pos, 1);
         ++pos;
     }
     return s;
 }
 
-std::string ErrorLogger::ErrorMessage::toXML() const
+std::string ErrorLogger::ErrorMessage::toXML(bool verbose, int version) const
 {
+    // Save this ErrorMessage as an XML element
     std::ostringstream xml;
-    xml << "<error";
-    if (!_callStack.empty())
+
+    // The default xml format
+    if (version == 1)
     {
-        xml << " file=\"" << stringToXml(_callStack.back().getfile()) << "\"";
-        xml << " line=\"" << _callStack.back().line << "\"";
+        xml << "<error";
+        if (!_callStack.empty())
+        {
+            xml << " file=\"" << stringToXml(_callStack.back().getfile()) << "\"";
+            xml << " line=\"" << _callStack.back().line << "\"";
+        }
+        xml << " id=\"" << _id << "\"";
+        xml << " severity=\"" << (_severity == Severity::error ? "error" : "style") << "\"";
+        xml << " msg=\"" << stringToXml(verbose ? _verboseMessage : _shortMessage) << "\"";
+        xml << "/>";
     }
-    xml << " id=\"" << _id << "\"";
-    xml << " severity=\"" << _severity << "\"";
-    xml << " msg=\"" << stringToXml(_msg) << "\"";
-    xml << "/>";
+
+    // The xml format you get when you use --xml-version=2
+    else if (version == 2)
+    {
+        xml << "  <error";
+        xml << " id=\"" << _id << "\"";
+        xml << " severity=\"" << Severity::toString(_severity) << "\"";
+        xml << " msg=\"" << stringToXml(_shortMessage) << "\"";
+        xml << " verbose=\"" << stringToXml(_verboseMessage) << "\"";
+        xml << ">" << std::endl;
+
+        for (std::list<FileLocation>::const_reverse_iterator it = _callStack.rbegin(); it != _callStack.rend(); ++it)
+        {
+            xml << "    <location";
+            xml << " file=\"" << stringToXml((*it).getfile()) << "\"";
+            xml << " line=\"" << (*it).line << "\"";
+            xml << "/>" << std::endl;
+        }
+
+        xml << "  </error>";
+    }
+
     return xml.str();
 }
 
@@ -169,24 +250,29 @@ void ErrorLogger::ErrorMessage::findAndReplace(std::string &source, const std::s
     }
 }
 
-std::string ErrorLogger::ErrorMessage::toText(const std::string &outputFormat) const
+std::string ErrorLogger::ErrorMessage::toString(bool verbose, const std::string &outputFormat) const
 {
+    // Save this ErrorMessage in plain text.
+
+    // No template is given
     if (outputFormat.length() == 0)
     {
         std::ostringstream text;
         if (!_callStack.empty())
             text << callStackToString(_callStack) << ": ";
-        if (!_severity.empty())
-            text << "(" << _severity << ") ";
-        text << _msg;
+        if (_severity != Severity::none)
+            text << "(" << Severity::toString(_severity) << ") ";
+        text << (verbose ? _verboseMessage : _shortMessage);
         return text.str();
     }
+
+    // template is given. Reformat the output according to it
     else
     {
         std::string result = outputFormat;
         findAndReplace(result, "{id}", _id);
-        findAndReplace(result, "{severity}", _severity);
-        findAndReplace(result, "{message}", _msg);
+        findAndReplace(result, "{severity}", Severity::toString(_severity));
+        findAndReplace(result, "{message}", verbose ? _verboseMessage : _shortMessage);
 
         if (!_callStack.empty())
         {
@@ -205,30 +291,6 @@ std::string ErrorLogger::ErrorMessage::toText(const std::string &outputFormat) c
     }
 }
 
-void ErrorLogger::_writemsg(const Tokenizer *tokenizer, const Token *tok, const char severity[], const std::string &msg, const std::string &id)
-{
-    std::list<const Token *> callstack;
-    callstack.push_back(tok);
-    _writemsg(tokenizer, callstack, severity, msg, id);
-}
-
-void ErrorLogger::_writemsg(const Tokenizer *tokenizer, const std::list<const Token *> &callstack, const char severity[], const std::string &msg, const std::string &id)
-{
-    std::list<ErrorLogger::ErrorMessage::FileLocation> locationList;
-    for (std::list<const Token *>::const_iterator tok = callstack.begin(); tok != callstack.end(); ++tok)
-    {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.file = tokenizer->file(*tok);
-        loc.line = (*tok)->linenr();
-        locationList.push_back(loc);
-    }
-
-    reportErr(ErrorLogger::ErrorMessage(locationList, severity, msg, id));
-}
-
-
-
-
 std::string ErrorLogger::callStackToString(const std::list<ErrorLogger::ErrorMessage::FileLocation> &callStack)
 {
     std::ostringstream ostr;
@@ -238,36 +300,17 @@ std::string ErrorLogger::callStackToString(const std::list<ErrorLogger::ErrorMes
 }
 
 
-std::string ErrorLogger::ErrorMessage::FileLocation::getfile() const
+std::string ErrorLogger::ErrorMessage::FileLocation::getfile(bool convert) const
 {
-    std::string f(file);
-
-    // replace "/ab/.." with "/"..
-    std::string::size_type pos = 0;
-    while ((pos = f.find("..", pos + 1)) != std::string::npos)
-    {
-        // position must be at least 4..
-        if (pos < 4)
-            continue;
-
-        // Previous char must be a separator..
-        if (f[pos-1] != '/' && f[pos-2] != '\\')
-            continue;
-
-        // Next char must be a separator..
-        if (f[pos+2] != '/' && f[pos+2] != '\\')
-            continue;
-
-        // Locate previous separator..
-        std::string::size_type sep = f.find_last_of("/\\", pos - 2);
-        if (sep == std::string::npos)
-            continue;
-
-        // Delete substring..
-        f.erase(sep, pos + 2 - sep);
-        pos = sep;
-    }
+    std::string f = Path::simplifyPath(_file.c_str());
 
+    if (convert)
+        f = Path::toNativeSeparators(f);
     return f;
 }
 
+void ErrorLogger::ErrorMessage::FileLocation::setfile(const std::string &file)
+{
+    _file = file;
+    _file = Path::fromNativeSeparators(_file);
+}
