@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,8 +25,6 @@
 #include <iostream>
 
 
-static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &checks);
-
 
 // default : bail out if the condition is has variable handling
 bool ExecutionPath::parseCondition(const Token &tok, std::list<ExecutionPath *> & checks)
@@ -76,13 +74,13 @@ void ExecutionPath::print() const
 }
 
 // I use this function when debugging ExecutionPaths with GDB
-#ifdef __GNUC__
+/*
 static void printchecks(const std::list<ExecutionPath *> &checks)
 {
     for (std::list<ExecutionPath *>::const_iterator it = checks.begin(); it != checks.end(); ++it)
         (*it)->print();
 }
-#endif
+*/
 
 
 
@@ -111,7 +109,7 @@ static void parseIfSwitchBody(const Token * const tok,
                 countif2.insert((*it)->varId);
         }
     }
-    checkExecutionPaths_(tok, c);
+    ExecutionPath::checkScope(tok, c);
     while (!c.empty())
     {
         if (c.back()->varId == 0)
@@ -124,7 +122,7 @@ static void parseIfSwitchBody(const Token * const tok,
         std::list<ExecutionPath *>::const_iterator it;
         for (it = checks.begin(); it != checks.end(); ++it)
         {
-            if (*(*it) == *c.back())
+            if (*(*it) == *c.back() && (*it)->numberOfIf == c.back()->numberOfIf)
             {
                 duplicate = true;
                 countif2.erase((*it)->varId);
@@ -141,7 +139,7 @@ static void parseIfSwitchBody(const Token * const tok,
 }
 
 
-static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &checks)
+void ExecutionPath::checkScope(const Token *tok, std::list<ExecutionPath *> &checks)
 {
     if (!tok || tok->str() == "}" || checks.empty())
         return;
@@ -150,6 +148,15 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
 
     for (; tok; tok = tok->next())
     {
+        // might be a noreturn function..
+        if (Token::simpleMatch(tok->tokAt(-2), ") ; }") &&
+            Token::Match(tok->tokAt(-2)->link()->tokAt(-2), "[;{}] %var% (") &&
+            tok->tokAt(-2)->link()->previous()->varId() == 0)
+        {
+            ExecutionPath::bailOut(checks);
+            return;
+        }
+
         if (tok->str() == "}" || tok->str() == "break")
             return;
 
@@ -176,8 +183,8 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
             }
         }
 
-        // goto => bailout
-        if (tok->str() == "goto")
+        // goto/setjmp/longjmp => bailout
+        if (Token::Match(tok, "goto|setjmp|longjmp"))
         {
             ExecutionPath::bailOut(checks);
             return;
@@ -186,12 +193,22 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
         // ?: => bailout
         if (tok->str() == "?")
         {
-            ExecutionPath::bailOut(checks);
-            return;
+            for (const Token *tok2 = tok; tok2 && tok2->str() != ";"; tok2 = tok2->next())
+            {
+                if (tok2->varId() > 0)
+                    ExecutionPath::bailOutVar(checks, tok2->varId());
+            }
         }
 
         if (tok->str() == "switch")
         {
+            // parse condition
+            if (checks.size() > 10 || check->parseCondition(*tok->next(), checks))
+            {
+                ExecutionPath::bailOut(checks);
+                return;
+            }
+
             const Token *tok2 = tok->next()->link();
             if (Token::simpleMatch(tok2, ") { case"))
             {
@@ -241,6 +258,27 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
                 return;
             }
 
+            if (tok->str() != "switch")
+            {
+                for (const Token *tok3 = tok; tok3 && tok3 != tok2; tok3 = tok3->next())
+                {
+                    if (tok3->varId())
+                        ExecutionPath::bailOutVar(checks, tok3->varId());
+                }
+
+                // it is not certain that a for/while will be executed:
+                for (std::list<ExecutionPath *>::iterator it = checks.begin(); it != checks.end();)
+                {
+                    if ((*it)->numberOfIf > 0)
+                        checks.erase(it++);
+                    else
+                        ++it;
+                }
+
+                // parse loop bodies
+                check->parseLoopBody(tok2->next(), checks);
+            }
+
             // skip { .. }
             tok2 = tok2->link();
 
@@ -303,9 +341,14 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
             while (tok && tok->str() != "{" && tok->str() != ";")
                 tok = tok->next();
             tok = tok ? tok->link() : 0;
+            if (!tok)
+            {
+                ExecutionPath::bailOut(checks);
+                return;
+            }
         }
 
-        if (Token::Match(tok, "= {"))
+        if (Token::simpleMatch(tok, "= {"))
         {
             // GCC struct initialization.. bail out
             if (Token::Match(tok->tokAt(2), ". %var% ="))
@@ -326,7 +369,7 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
         // ; { ... }
         if (Token::Match(tok->previous(), "[;{}] {"))
         {
-            checkExecutionPaths_(tok->next(), checks);
+            ExecutionPath::checkScope(tok->next(), checks);
             tok = tok->link();
             continue;
         }
@@ -379,7 +422,7 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
                     continue;
 
                 // there is no "if"..
-                checkExecutionPaths_(tok->next(), checks);
+                ExecutionPath::checkScope(tok->next(), checks);
                 tok = tok->link();
                 if (!tok)
                 {
@@ -398,6 +441,20 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
                 if (countif.find((*it)->varId) != countif.end())
                     (*it)->numberOfIf++;
             }
+
+            // Delete checks that have numberOfIf >= 2
+            for (it = checks.begin(); it != checks.end();)
+            {
+                if ((*it)->varId > 0 && (*it)->numberOfIf >= 2)
+                {
+                    delete *it;
+                    checks.erase(it++);
+                }
+                else
+                {
+                    ++it;
+                }
+            }
         }
 
 
@@ -408,7 +465,10 @@ static void checkExecutionPaths_(const Token *tok, std::list<ExecutionPath *> &c
         }
 
         // return/throw ends all execution paths
-        if (tok->str() == "return" || tok->str() == "throw")
+        if (tok->str() == "return" ||
+            tok->str() == "throw" ||
+            tok->str() == "continue" ||
+            tok->str() == "break")
         {
             ExecutionPath::bailOut(checks);
         }
@@ -432,7 +492,7 @@ void checkExecutionPaths(const Token *tok, ExecutionPath *c)
 
             std::list<ExecutionPath *> checks;
             checks.push_back(c->copy());
-            checkExecutionPaths_(tok, checks);
+            ExecutionPath::checkScope(tok, checks);
 
             c->end(checks, tok->link());
 
@@ -441,9 +501,6 @@ void checkExecutionPaths(const Token *tok, ExecutionPath *c)
                 delete checks.back();
                 checks.pop_back();
             }
-
-            // skip this scope - it has been checked
-            tok = tok->link();
         }
     }
 }
