@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,16 +24,18 @@
 #include <stdexcept>
 #include <iostream>
 #include <cctype>   // std::isdigit, std::isalnum, etc
+#include <set>
 
 Settings::Settings()
 {
-    _debug = false;
+    debug = debugwarnings = false;
     _checkCodingStyle = false;
     _errorsOnly = false;
     _inlineSuppressions = false;
     _verbose = false;
     _force = false;
     _xml = false;
+    _xml_version = 1;
     _jobs = 1;
     _exitCode = 0;
     _showtime = 0; // TODO: use enum
@@ -41,9 +43,11 @@ Settings::Settings()
     _terminate = false;
     inconclusive = false;
     test_2_pass = false;
+    reportProgress = false;
+    ifcfg = false;
 }
 
-bool Settings::Suppressions::parseFile(std::istream &istr)
+std::string Settings::Suppressions::parseFile(std::istream &istr)
 {
     // Change '\r' to '\n' in the istr
     std::string filedata;
@@ -53,8 +57,6 @@ bool Settings::Suppressions::parseFile(std::istream &istr)
     while (filedata.find("\r") != std::string::npos)
         filedata[filedata.find("\r")] = '\n';
 
-    bool ret = true;
-
     // Parse filedata..
     std::istringstream istr2(filedata);
     while (std::getline(istr2, line))
@@ -63,51 +65,80 @@ bool Settings::Suppressions::parseFile(std::istream &istr)
         if (line.empty())
             continue;
 
+        // Skip comments
+        if (line.length() >= 2 && line[0] == '/' && line[1] == '/')
+            continue;
+
         std::istringstream lineStream(line);
         std::string id;
         std::string file;
         unsigned int lineNumber = 0;
         if (std::getline(lineStream, id, ':'))
         {
-            if (std::getline(lineStream, file, ':'))
+            if (std::getline(lineStream, file))
             {
-                lineStream >> lineNumber;
+                // If there is not a dot after the last colon in "file" then
+                // the colon is a separator and the contents after the colon
+                // is a line number..
+
+                // Get position of last colon
+                const std::string::size_type pos = file.rfind(":");
+
+                // if a colon is found and there is no dot after it..
+                if (pos != std::string::npos &&
+                    file.find(".", pos) == std::string::npos)
+                {
+                    // Try to parse out the line number
+                    try
+                    {
+                        std::istringstream istr1(file.substr(pos+1));
+                        istr1 >> lineNumber;
+                    }
+                    catch (...)
+                    {
+                        lineNumber = 0;
+                    }
+
+                    if (lineNumber > 0)
+                    {
+                        file.erase(pos);
+                    }
+                }
             }
         }
 
         // We could perhaps check if the id is valid and return error if it is not
-        ret &= addSuppression(id, file, lineNumber);
+        const std::string errmsg(addSuppression(id, file, lineNumber));
+        if (!errmsg.empty())
+            return errmsg;
     }
 
-    return ret;
+    return "";
 }
 
-bool Settings::Suppressions::addSuppression(const std::string &errorId, const std::string &file, unsigned int line)
+std::string Settings::Suppressions::addSuppression(const std::string &errorId, const std::string &file, unsigned int line)
 {
     // Check that errorId is valid..
     if (errorId.empty())
     {
-        std::cerr << "Failed to add suppression. No id." << std::endl;
-        return false;
+        return "Failed to add suppression. No id.";
     }
     for (std::string::size_type pos = 0; pos < errorId.length(); ++pos)
     {
         if (errorId[pos] < 0 || !std::isalnum(errorId[pos]))
         {
-            std::cerr << "Failed to add suppression. Invalid id \"" << errorId << "\"" << std::endl;
-            return false;
+            return "Failed to add suppression. Invalid id \"" + errorId + "\"";
         }
         if (pos == 0 && std::isdigit(errorId[pos]))
         {
-            std::cerr << "Failed to add suppression. Invalid id \"" << errorId << "\"" << std::endl;
-            return false;
+            return "Failed to add suppression. Invalid id \"" + errorId + "\"";
         }
     }
 
     _suppressions[errorId][file].push_back(line);
     _suppressions[errorId][file].sort();
 
-    return true;
+    return "";
 }
 
 bool Settings::Suppressions::isSuppressed(const std::string &errorId, const std::string &file, unsigned int line)
@@ -132,7 +163,7 @@ bool Settings::Suppressions::isSuppressed(const std::string &errorId, const std:
     return true;
 }
 
-void Settings::addEnabled(const std::string &str)
+std::string Settings::addEnabled(const std::string &str)
 {
     // Enable parameters may be comma separated...
     if (str.find(",") != std::string::npos)
@@ -142,15 +173,16 @@ void Settings::addEnabled(const std::string &str)
         while ((pos = str.find(",", pos)) != std::string::npos)
         {
             if (pos == prevPos)
-                throw std::runtime_error("cppcheck: --enable parameter is empty");
-            addEnabled(str.substr(prevPos, pos - prevPos));
+                return std::string("cppcheck: --enable parameter is empty");
+            const std::string errmsg(addEnabled(str.substr(prevPos, pos - prevPos)));
+            if (!errmsg.empty())
+                return errmsg;
             ++pos;
             prevPos = pos;
         }
         if (prevPos >= str.length())
-            throw std::runtime_error("cppcheck: --enable parameter is empty");
-        addEnabled(str.substr(prevPos));
-        return;
+            return std::string("cppcheck: --enable parameter is empty");
+        return addEnabled(str.substr(prevPos));
     }
 
     bool handled = false;
@@ -161,9 +193,9 @@ void Settings::addEnabled(const std::string &str)
         handled = _checkCodingStyle = true;
 
     std::set<std::string> id;
-    id.insert("exceptNew");
-    id.insert("exceptRealloc");
-    id.insert("unusedFunctions");
+    id.insert("missingInclude");
+    id.insert("unusedFunction");
+    id.insert("information");
 
     if (str == "all")
     {
@@ -178,10 +210,12 @@ void Settings::addEnabled(const std::string &str)
     else if (!handled)
     {
         if (str.empty())
-            throw std::runtime_error("cppcheck: --enable parameter is empty");
+            return std::string("cppcheck: --enable parameter is empty");
         else
-            throw std::runtime_error("cppcheck: there is no --enable parameter with the name '" + str + "'");
+            return std::string("cppcheck: there is no --enable parameter with the name '" + str + "'");
     }
+
+    return std::string("");
 }
 
 bool Settings::isEnabled(const std::string &str) const
