@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #include <string>
 #include <istream>
 #include <map>
-#include <set>
 
 /// @addtogroup Core
 /// @{
@@ -50,7 +49,10 @@ public:
     Settings();
 
     /** @brief Is --debug given? */
-    bool _debug;
+    bool debug;
+
+    /** @brief Is --debug-warnings given? */
+    bool debugwarnings;
 
     /** @brief Inconclusive checks - for debugging of Cppcheck */
     bool inconclusive;
@@ -82,9 +84,12 @@ public:
     /** @brief Force checking the files with "too many" configurations (--force). */
     bool _force;
 
-    /** @brief write xml results (--xml) */
+    /** @brief write XML results (--xml) */
     bool _xml;
 
+    /** @brief XML version (--xmlver=..) */
+    int _xml_version;
+
     /** @brief How many processes/threads should do checking at the same
         time. Default is 1. (-j N) */
     unsigned int _jobs;
@@ -122,22 +127,23 @@ public:
      * @brief Enable extra checks by id. See isEnabled()
      * @param str single id or list of id values to be enabled
      * or empty string to enable all. e.g. "style,possibleError"
+     * @return error message. empty upon success
      */
-    void addEnabled(const std::string &str);
+    std::string addEnabled(const std::string &str);
 
     /** @brief class for handling suppressions */
     class Suppressions
     {
     private:
         /** @brief List of error which the user doesn't want to see. */
-        std::map<std::string, std::map<std::string, std::list<int> > > _suppressions;
+        std::map<std::string, std::map<std::string, std::list<unsigned int> > > _suppressions;
     public:
         /**
          * @brief Don't show errors listed in the file.
          * @param istr Open file stream where errors can be read.
-         * @return true on success, false in syntax error is noticed.
+         * @return error message. empty upon success
          */
-        bool parseFile(std::istream &istr);
+        std::string parseFile(std::istream &istr);
 
         /**
          * @brief Don't show this error. If file and/or line are optional. In which case
@@ -145,9 +151,9 @@ public:
          * @param errorId the id for the error, e.g. "arrayIndexOutOfBounds"
          * @param file File name with the path, e.g. "src/main.cpp"
          * @param line number, e.g. "123"
-         * @return true on success, false in syntax error is noticed.
+         * @return error message. empty upon success
          */
-        bool addSuppression(const std::string &errorId, const std::string &file = "", unsigned int line = 0);
+        std::string addSuppression(const std::string &errorId, const std::string &file = "", unsigned int line = 0);
 
         /**
          * @brief Returns true if this message should not be shown to the user.
@@ -168,8 +174,41 @@ public:
     /** @brief defines given by the user */
     std::string userDefines;
 
-    /** @brief Experimentat 2 pass checking of files */
+    /** @brief Experimental 2 pass checking of files */
     bool test_2_pass;
+
+    /** @brief --report-progress */
+    bool reportProgress;
+
+    /**
+     * @brief Is there any preprocessor configurations in the source code?
+     * As usual, include guards are not counted.
+     */
+    bool ifcfg;
+
+    /** Rule */
+    class Rule
+    {
+    public:
+        Rule()
+        {
+            // default id
+            id = "rule";
+
+            // default severity
+            severity = "style";
+        }
+
+        std::string pattern;
+        std::string id;
+        std::string severity;
+        std::string summary;
+    };
+
+    /**
+     * @brief Extra rules
+     */
+    std::list<Rule> rules;
 };
 
 /// @}
