@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
  */
 
 #include "token.h"
+#include "errorlogger.h"
+#include "check.h"
 #include <cassert>
 #include <cstdlib>
 #include <cstring>
@@ -41,7 +43,8 @@ Token::Token(Token **t) :
     _previous(0),
     _link(0),
     _fileIndex(0),
-    _linenr(0)
+    _linenr(0),
+    _progressValue(0)
 {
 }
 
@@ -197,20 +200,21 @@ std::string Token::strAt(int index) const
 int Token::multiCompare(const char *haystack, const char *needle)
 {
     bool emptyStringFound = false;
-    bool noMatch = false;
     const char *needlePointer = needle;
-    for (; *haystack && *haystack != ' '; ++haystack)
+    while (true)
     {
-        if (*haystack == '|')
+        if (*needlePointer == *haystack)
         {
-            if (noMatch)
+            if (*needlePointer == '\0')
+                return 1;
+            ++needlePointer;
+            ++haystack;
+        }
+        else if (*haystack == '|')
+        {
+            if (*needlePointer == 0)
             {
-                // We didn't have a match at this round
-                noMatch = false;
-            }
-            else if (*needlePointer == 0)
-            {
-                // If needle and haystack are both at the end, we have a match.
+                // If needle is at the end, we have a match.
                 return 1;
             }
             else if (needlePointer == needle)
@@ -221,39 +225,38 @@ int Token::multiCompare(const char *haystack, const char *needle)
             }
 
             needlePointer = needle;
-            continue;
+            ++haystack;
+        }
+        else if (*haystack == ' ' || *haystack == '\0')
+        {
+            if (needlePointer == needle)
+                return 0;
+            break;
         }
-
-        if (noMatch)
-            continue;
-
         // If haystack and needle don't share the same character,
         // find next '|' character.
-        if (*needlePointer != *haystack)
+        else
         {
-            noMatch = true;
-            continue;
-        }
+            needlePointer = needle;
 
-        // All characters in haystack and needle have matched this far
-        ++needlePointer;
-    }
+            do
+            {
+                ++haystack;
+            }
+            while (*haystack != ' ' && *haystack != '|' && *haystack);
 
+            if (*haystack == ' ' || *haystack == '\0')
+            {
+                return emptyStringFound ? 0 : -1;
+            }
 
-    if (!noMatch)
-    {
-        if (*needlePointer == 0)
-        {
-            // If both needle and haystack are at the end, then we have a match.
-            return 1;
-        }
-        else if (needlePointer == needle)
-        {
-            // Last string in haystack was empty string e.g. "one|two|"
-            return 0;
+            ++haystack;
         }
     }
 
+    if (*needlePointer == '\0')
+        return 1;
+
     // If empty string was found earlier from the haystack
     if (emptyStringFound)
         return 0;
@@ -294,10 +297,12 @@ int Token::firstWordEquals(const char *str, const char *word)
 {
     for (;;)
     {
-        if (*str == ' ' && *word == 0)
-            return 0;
-        else if (*str != *word)
+        if (*str != *word)
+        {
+            if (*str == ' ' && *word == 0)
+                return 0;
             return 1;
+        }
         else if (*str == 0)
             break;
 
@@ -341,17 +346,8 @@ bool Token::Match(const Token *tok, const char pattern[], unsigned int varid)
 {
     const char *p = pattern;
     bool firstpattern = true;
-    bool first = true;
     while (*p)
     {
-        if (!first)
-        {
-            while (*p && *p != ' ')
-                ++p;
-        }
-
-        first = false;
-
         // Skip spaces in pattern..
         while (*p == ' ')
             ++p;
@@ -364,20 +360,27 @@ bool Token::Match(const Token *tok, const char pattern[], unsigned int varid)
         {
             // If we have no tokens, pattern "!!else" should return true
             if (p[1] == '!' && p[0] == '!' && p[2] != '\0')
+            {
+                while (*p && *p != ' ')
+                    ++p;
                 continue;
+            }
             else
                 return false;
         }
 
         // If we are in the first token, we skip all initial !! patterns
         if (firstpattern && !tok->previous() && tok->next() && p[1] == '!' && p[0] == '!' && p[2] != '\0')
+        {
+            while (*p && *p != ' ')
+                ++p;
             continue;
+        }
 
         firstpattern = false;
 
         // Compare the first character of the string for optimization reasons
         // before doing more detailed checks.
-        bool patternIdentified = false;
         if (p[0] == '%')
         {
             // TODO: %var% should match only for
@@ -388,8 +391,7 @@ bool Token::Match(const Token *tok, const char pattern[], unsigned int varid)
             {
                 if (!tok->isName())
                     return false;
-
-                patternIdentified = true;
+                p += 5;
             }
 
             // Type..
@@ -404,62 +406,84 @@ bool Token::Match(const Token *tok, const char pattern[], unsigned int varid)
                 if (tok->str() == "delete")
                     return false;
 
-                patternIdentified = true;
+                p += 6;
             }
 
             // Accept any token
             else if (firstWordEquals(p, "%any%") == 0)
             {
-                patternIdentified = true;
+                p += 5;
             }
 
             else if (firstWordEquals(p, "%varid%") == 0)
             {
                 if (varid == 0)
                 {
-                    // TODO: Report this through ErrorLogger instead so these messages appear in the unit testing
-                    std::cerr << "\n###### If you see this, there is a bug ######" << std::endl
-                              << "Token::Match(\"" << pattern << "\", 0)" << std::endl;
+                    std::list<ErrorLogger::ErrorMessage::FileLocation> locationList;
+                    const ErrorLogger::ErrorMessage errmsg(locationList,
+                                                           Severity::error,
+                                                           "Internal error. Token::Match called with varid 0.",
+                                                           "cppcheckError");
+                    Check::reportError(errmsg);
                 }
 
                 if (tok->varId() != varid)
                     return false;
 
-                patternIdentified = true;
+                p += 7;
             }
 
             else if (firstWordEquals(p, "%num%") == 0)
             {
                 if (!tok->isNumber())
                     return false;
-
-                patternIdentified = true;
+                p += 5;
             }
 
             else if (firstWordEquals(p, "%bool%") == 0)
             {
                 if (!tok->isBoolean())
                     return false;
-
-                patternIdentified = true;
+                p += 6;
             }
 
             else if (firstWordEquals(p, "%str%") == 0)
             {
                 if (tok->_str[0] != '\"')
                     return false;
-
-                patternIdentified = true;
+                p += 5;
             }
-        }
 
-        if (patternIdentified)
-        {
-            // Pattern was identified already above.
+            else if (firstWordEquals(p, "%or%") == 0)
+            {
+                if (tok->str() != "|")
+                    return false;
+                p += 4;
+            }
+
+            else if (firstWordEquals(p, "%oror%") == 0)
+            {
+                if (tok->str() != "||")
+                    return false;
+                p += 6;
+            }
+
+            else if (firstWordEquals(p, tok->_str.c_str()))
+            {
+                p += tok->_str.length();
+            }
+
+            else
+            {
+                return false;
+            }
+
+            tok = tok->next();
+            continue;
         }
 
         // [.. => search for a one-character token..
-        else if (p[0] == '[' && chrInFirstWord(p, ']') && tok->_str.length() == 1)
+        else if (p[0] == '[' && tok->_str.length() == 1 && chrInFirstWord(p, ']'))
         {
             const char *temp = p + 1;
             bool chrFound = false;
@@ -499,6 +523,8 @@ bool Token::Match(const Token *tok, const char pattern[], unsigned int varid)
             if (res == 0)
             {
                 // Empty alternative matches, use the same token on next round
+                while (*p && *p != ' ')
+                    ++p;
                 continue;
             }
             else if (res == -1)
@@ -516,7 +542,12 @@ bool Token::Match(const Token *tok, const char pattern[], unsigned int varid)
         }
 
         else if (firstWordEquals(p, tok->_str.c_str()) != 0)
+        {
             return false;
+        }
+
+        while (*p && *p != ' ')
+            ++p;
 
         tok = tok->next();
     }
@@ -554,16 +585,7 @@ size_t Token::getStrLength(const Token *tok)
 bool Token::isStandardType() const
 {
     bool ret = false;
-    const char *type[] = {"bool", "char", "short", "int", "long", "float", "double", "size_t", "__int64", 0};
-    for (int i = 0; type[i]; i++)
-        ret |= (_str == type[i]);
-    return ret;
-}
-
-bool Token::isIntegerType() const
-{
-    bool ret = false;
-    const char *type[] = {"char", "short", "int", "long", "size_t", "__int64", 0};
+    const char *type[] = {"bool", "char", "short", "int", "long", "float", "double", "size_t", 0};
     for (int i = 0; type[i]; i++)
         ret |= (_str == type[i]);
     return ret;
@@ -584,6 +606,10 @@ void Token::move(Token *srcStart, Token *srcEnd, Token *newLocation)
     // Fix the tokens at newLocation
     newLocation->next()->previous(srcEnd);
     newLocation->next(srcStart);
+
+    // Update _progressValue
+    for (Token *tok = srcStart; tok && tok != srcEnd; tok = tok->next())
+        tok->_progressValue = newLocation->_progressValue;
 }
 
 //---------------------------------------------------------------------------
@@ -598,12 +624,23 @@ const Token *Token::findmatch(const Token *tok, const char pattern[], unsigned i
     return 0;
 }
 
+const Token *Token::findmatch(const Token *tok, const char pattern[], const Token *end, unsigned int varId)
+{
+    for (; tok && tok != end; tok = tok->next())
+    {
+        if (Token::Match(tok, pattern, varId))
+            return tok;
+    }
+    return 0;
+}
+
 void Token::insertToken(const std::string &tokenStr)
 {
     Token *newToken = new Token(tokensBack);
     newToken->str(tokenStr);
     newToken->_linenr = _linenr;
     newToken->_fileIndex = _fileIndex;
+    newToken->_progressValue = _progressValue;
     if (this->next())
     {
         newToken->next(this->next());
@@ -663,7 +700,7 @@ std::string Token::stringifyList(bool varid, const char *title, const std::vecto
 
     unsigned int lineNumber = 0;
     int fileInd = -1;
-    std::map<unsigned int, unsigned int> lineNumbers;
+    std::map<int, unsigned int> lineNumbers;
     for (const Token *tok = this; tok; tok = tok->next())
     {
         bool fileChange = false;
@@ -676,8 +713,8 @@ std::string Token::stringifyList(bool varid, const char *title, const std::vecto
 
             fileInd = static_cast<int>(tok->_fileIndex);
             ret << "\n\n##file ";
-            if (fileNames.size() > static_cast<unsigned int>(fileInd))
-                ret << fileNames.at(fileInd);
+            if (fileNames.size() > tok->_fileIndex)
+                ret << fileNames.at(tok->_fileIndex);
             else
                 ret << fileInd;
 
