@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,15 +21,15 @@
 #ifndef tokenizeH
 #define tokenizeH
 //---------------------------------------------------------------------------
-#include <list>
+
 #include <string>
 #include <map>
 #include <vector>
-#include "classinfo.h"
 
 class Token;
 class ErrorLogger;
 class Settings;
+class SymbolDatabase;
 
 /// @addtogroup Core
 /// @{
@@ -46,6 +46,18 @@ public:
     Tokenizer(const Settings * settings, ErrorLogger *errorLogger);
     virtual ~Tokenizer();
 
+    /** Is the code JAVA/C#. Used for bailouts */
+    bool isJavaOrCSharp() const
+    {
+        if (_files.size() != 1)
+            return false;
+        const std::string::size_type pos = _files[0].rfind(".");
+        if (pos != std::string::npos)
+            return (_files[0].substr(pos) == ".java" ||
+                    _files[0].substr(pos) == ".cs");
+        return false;
+    }
+
     /**
      * Tokenize code
      * @param code input stream for code, e.g.
@@ -65,9 +77,13 @@ public:
      *
      * @param FileName The filename
      * @param configuration E.g. "A" for code where "#ifdef A" is true
-     * @return false if Source code contains syntax errors
+     * @param preprocessorCondition Set this flag to true if the code is a preprocessor condition. It disables some simplifications
+     * @return false if source code contains syntax errors
      */
-    bool tokenize(std::istream &code, const char FileName[], const std::string &configuration = "");
+    bool tokenize(std::istream &code,
+                  const char FileName[],
+                  const std::string &configuration = "",
+                  const bool preprocessorCondition = false);
 
     /**
      * Create tokens from code.
@@ -90,9 +106,27 @@ public:
      */
     bool simplifyTokenList();
 
+    /**
+     * Delete all tokens in given token list
+     * @param tok token list to delete
+     */
     static void deleteTokens(Token *tok);
-    static const char *getParameterName(const Token *ftok, int par);
 
+    /**
+     * Get parameter name of function
+     * @param ftok The token for the function name in a function
+     *             implementation/declaration
+     * @param par   parameter number (1,2,3,..)
+     * @return if the parameter was found then the parameter name is
+     *         returned. Otherwise NULL is returned.
+     */
+    static const char *getParameterName(const Token *ftok, unsigned int par);
+
+    /**
+     * Get file:line for a given token
+     * @param tok given token
+     * @return location for given token
+     */
     std::string fileLine(const Token *tok) const;
 
     /**
@@ -102,35 +136,42 @@ public:
      */
     unsigned int sizeOfType(const Token *type) const;
 
+    /**
+     * Get filenames (the sourcefile + the files it include).
+     * The first filename is the filename for the sourcefile
+     * @return vector with filenames
+     */
     const std::vector<std::string> *getFiles() const;
 
+    /** recreate symbol database */
     void fillFunctionList();
+
+    /**
+     * Get function token by function name
+     * @todo better handling of overloaded functions
+     * @todo only scan parent scopes
+     * @param funcname function name
+     */
     const Token *getFunctionTokenByName(const char funcname[]) const;
+
+    /** get tokens */
     const Token *tokens() const;
 
+    /**
+     * get filename for given token
+     * @param tok The given token
+     * @return filename for the given token
+     */
     std::string file(const Token *tok) const;
 
     /**
-     * Find a class or struct member function
-     * @param tok where to begin the search
-     * @param classname name of class
-     * @param funcname name of function ("~ Fred" => destructor for fred, "%var%" => any function)
-     * @param indentlevel Just an integer that you initialize to 0 before the first call.
-     * @param isStruct is it a struct
-     * @return First matching token or NULL.
+     * get error messages that the tokenizer generate
      */
-    const Token *findClassFunction(const Token *tok, const std::string &classname, const std::string &funcname, int &indentlevel, bool isStruct = false) const;
+    virtual void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings);
 
-    /**
-     * get error messages
+    /** Simplify assignment in function call "f(x=g());" => "x=g();f(x);"
      */
-    virtual void getErrorMessages();
-
-    /**
-     * List of classes in currently checked source code and
-     * their member functions and member variables.
-     */
-    std::map<std::string, ClassInfo> _classInfoList;
+    void simplifyAssignmentInFunctionCall();
 
     /**
      * Simplify constant calculations such as "1+2" => "3"
@@ -145,6 +186,9 @@ public:
     /** Simplify labels */
     void labels();
 
+    /** Remove macros in global scope */
+    void removeMacrosInGlobalScope();
+
     /** Remove redundant assignment */
     void removeRedundantAssignment();
 
@@ -168,7 +212,7 @@ public:
     Token * initVar(Token * tok);
 
     /**
-     * Colapse compound standard types into a single token.
+     * Collapse compound standard types into a single token.
      * unsigned long long int => long _isUnsigned=true,_isLong=true
      */
     void simplifyStdType();
@@ -181,6 +225,12 @@ public:
      */
     bool simplifyQuestionMark();
 
+    /**
+     * Simplify compound assignments
+     * Example: ";a+=b;" => ";a=a+b;"
+     */
+    void simplifyCompoundAssignment();
+
     /**
      * simplify if-assignments
      * Example: "if(a=b);" => "a=b;if(a);"
@@ -188,7 +238,7 @@ public:
     void simplifyIfAssign();
 
     /**
-     * Simplify multiple assignmetns.
+     * Simplify multiple assignments.
      * Example: "a = b = c = 0;" => "a = 0; b = 0; c = 0;"
      */
     void simplifyVariableMultipleAssign();
@@ -267,8 +317,6 @@ public:
     /** Simplify "if else" */
     void elseif();
 
-    std::vector<const Token *> _functionList;
-
     void addtoken(const char str[], const unsigned int lineno, const unsigned int fileno, bool split = false);
     void addtoken(const Token *tok, const unsigned int lineno, const unsigned int fileno);
 
@@ -283,13 +331,18 @@ public:
      */
     bool simplifyConditions();
 
-    /** Remove reduntant code, e.g. if( false ) { int a; } should be
+    /** Remove redundant code, e.g. if( false ) { int a; } should be
      * removed, because it is never executed.
      * @return true if something is modified
      *         false if nothing is done.
      */
     bool removeReduntantConditions();
 
+    /**
+     * Reduces "; ;" to ";", except in "( ; ; )"
+     */
+    void removeRedundantSemicolons();
+
     /** Simplify function calls - constant return value
      * @return true if something is modified
      *         false if nothing is done.
@@ -306,7 +359,7 @@ public:
     void simplifyStructDecl();
 
     /**
-     * Remove redundant paranthesis:
+     * Remove redundant parenthesis:
      * - "((x))" => "(x)"
      * - "(function())" => "function()"
      * - "(delete x)" => "delete x"
@@ -411,13 +464,11 @@ public:
      */
     bool createLinks();
 
-    void syntaxError(const Token *tok, char c);
+    /** Syntax error */
+    void syntaxError(const Token *tok);
 
-    /**
-     * Update _classInfoList to contain class names and member
-     * functions and member variables.
-     */
-    void updateClassList();
+    /** Syntax error. Example: invalid number of ')' */
+    void syntaxError(const Token *tok, char c);
 
     /**
      * assert that tokens are ok - used during debugging for example
@@ -458,11 +509,42 @@ public:
      */
     void simplifyAsm();
 
+    /**
+     * Simplify bitfields - the field width is removed as we don't use it.
+     */
+    void simplifyBitfields();
+
     /**
      * Remove __builtin_expect(...), likely(...), and unlikely(...)
      */
     void simplifyBuiltinExpect();
 
+    /**
+     * Remove unnecessary member qualification
+     */
+    void removeUnnecessaryQualification();
+
+    /**
+     * Remove Microsoft MFC 'DECLARE_MESSAGE_MAP()'
+     */
+    void simplifyMicrosoftMFC();
+
+    /**
+     * Remove Borland code
+     */
+    void simplifyBorland();
+
+    /**
+     * Remove Qt signals and slots
+     */
+    void simplifyQtSignalsSlots();
+
+    /**
+     * Collapse operator name tokens into single token
+     * operator = => operator=
+     */
+    void simplifyOperatorName();
+
     /**
      * This will return a short name describing function parameters
      * e.g. parameters: (int a, char b) should get name "int,char,".
@@ -485,8 +567,29 @@ public:
 
     bool duplicateTypedef(Token **tokPtr, const Token *name);
     void duplicateTypedefError(const Token *tok1, const Token *tok2, const std::string & type);
+
+    /**
+     * Report error - duplicate declarations
+     */
     void duplicateDeclarationError(const Token *tok1, const Token *tok2, const std::string &type);
 
+    void unsupportedTypedef(const Token *tok) const;
+
+    /** Was there templates in the code? */
+    bool codeWithTemplates() const
+    {
+        return _codeWithTemplates;
+    }
+
+    void setSettings(const Settings *settings)
+    {
+        _settings = settings;
+    }
+
+    const SymbolDatabase *getSymbolDatabase() const;
+
+    Token *deleteInvalidTypedef(Token *typeDef);
+
 private:
     /** Disable copy constructor, no implementation */
     Tokenizer(const Tokenizer &);
@@ -494,15 +597,33 @@ private:
     /** Disable assignment operator, no implementation */
     Tokenizer &operator=(const Tokenizer &);
 
+    /** Token list */
     Token *_tokens, *_tokensBack;
+
+    /** sizeof information for known types */
     std::map<std::string, unsigned int> _typeSize;
+
+    /** filenames for the tokenized source code (source + included) */
     std::vector<std::string> _files;
-    const Settings * const _settings;
+
+    /** settings */
+    const Settings * _settings;
+
+    /** errorlogger */
     ErrorLogger * const _errorLogger;
 
     /** E.g. "A" for code where "#ifdef A" is true. This is used to
         print additional information in error situations. */
     std::string _configuration;
+
+    /**
+     * was there any templates? templates that are "unused" are
+     * removed from the token list
+     */
+    bool _codeWithTemplates;
+
+    /** Symbol database that all checks etc can use */
+    mutable SymbolDatabase *_symbolDatabase;
 };
 
 /// @}
