@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,10 +36,17 @@ private:
 
 
 
-    void check(const char code[], bool showAll = true)
+    void check(const char code[], bool inconclusive = true)
     {
+        // Clear the error buffer..
+        errout.str("");
+
+        Settings settings;
+        settings.inconclusive = inconclusive;
+        settings._checkCodingStyle = true;
+
         // Tokenize..
-        Tokenizer tokenizer;
+        Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
         tokenizer.tokenize(istr, "test.cpp");
 
@@ -49,13 +56,7 @@ private:
         // Fill function list
         tokenizer.fillFunctionList();
 
-        // Clear the error buffer..
-        errout.str("");
-
         // Check for buffer overruns..
-        Settings settings;
-        settings.inconclusive = showAll;
-        settings._checkCodingStyle = true;
         CheckBufferOverrun checkBufferOverrun(&tokenizer, &settings, this);
         checkBufferOverrun.bufferOverrun();
         checkBufferOverrun.negativeIndex();
@@ -103,12 +104,18 @@ private:
         TEST_CASE(array_index_27);
         TEST_CASE(array_index_28); // ticket #1418
         TEST_CASE(array_index_29); // ticket #1734
+        TEST_CASE(array_index_30); // ticket #2086 - out of bounds when type is unknown
+        TEST_CASE(array_index_31); // ticket #2120 - out of bounds in subfunction when type is unknown
+        TEST_CASE(array_index_32);
         TEST_CASE(array_index_multidim);
         TEST_CASE(array_index_switch_in_for);
         TEST_CASE(array_index_calculation);
         TEST_CASE(array_index_negative);
         TEST_CASE(array_index_for_decr);
         TEST_CASE(array_index_varnames);   // FP: struct member. #1576
+        TEST_CASE(array_index_for_break);  // FP: for,break
+        TEST_CASE(array_index_for);        // FN: for,if
+        TEST_CASE(array_index_for_neq);    // #2211: Using != in condition
 
         TEST_CASE(buffer_overrun_1);
         TEST_CASE(buffer_overrun_2);
@@ -125,6 +132,16 @@ private:
         TEST_CASE(buffer_overrun_13);
         TEST_CASE(buffer_overrun_14);
         TEST_CASE(buffer_overrun_15); // ticket #1787
+        TEST_CASE(buffer_overrun_16);
+        TEST_CASE(buffer_overrun_bailoutIfSwitch);  // ticket #2378 : bailoutIfSwitch
+
+        // It is undefined behaviour to point out of bounds of an array
+        // the address beyond the last element is in bounds
+        // char a[10];
+        // char *p1 = a + 10;  // OK
+        // char *p2 = a + 11   // UB
+        TEST_CASE(pointer_out_of_bounds_1);
+        TEST_CASE(pointer_out_of_bounds_2);
 
         TEST_CASE(sprintf1);
         TEST_CASE(sprintf2);
@@ -134,6 +151,8 @@ private:
         TEST_CASE(sprintf6);
         TEST_CASE(sprintf7);
         TEST_CASE(sprintf8);
+        TEST_CASE(sprintf9);
+        TEST_CASE(sprintf10);
 
         TEST_CASE(snprintf1);
         TEST_CASE(snprintf2);
@@ -141,6 +160,7 @@ private:
         TEST_CASE(snprintf4);
         TEST_CASE(snprintf5);
         TEST_CASE(snprintf6);
+        TEST_CASE(snprintf7);
 
         TEST_CASE(strncat1);
         TEST_CASE(strncat2);
@@ -148,8 +168,10 @@ private:
 
         TEST_CASE(strcat1);
         TEST_CASE(strcat2);
+        TEST_CASE(strcat3);
 
-        TEST_CASE(memfunc);		// memchr/memset/memcpy
+        TEST_CASE(memfunc1);		// memchr/memset/memcpy
+        TEST_CASE(memfunc2);
 
         TEST_CASE(varid1);
         TEST_CASE(varid2);
@@ -158,6 +180,8 @@ private:
 
         TEST_CASE(alloc1);    // Buffer allocated with new
         TEST_CASE(alloc2);    // Buffer allocated with malloc
+        TEST_CASE(alloc3);    // statically allocated buffer
+        TEST_CASE(malloc_memset);	// using memset on buffer allocated with malloc
 
         TEST_CASE(memset1);
         TEST_CASE(memset2);
@@ -167,14 +191,21 @@ private:
 
         TEST_CASE(terminateStrncpy1);
         TEST_CASE(terminateStrncpy2);
+        TEST_CASE(terminateStrncpy3);
         TEST_CASE(recursive_long_time);
 
         TEST_CASE(crash);	// Ticket #1587 - crash
 
         TEST_CASE(executionPaths1);
         TEST_CASE(executionPaths2);
+        TEST_CASE(executionPaths3);   // no FP for function parameter
+        TEST_CASE(executionPaths4);   // Ticket #2386 - Segmentation fault in the ExecutionPath handling
 
         TEST_CASE(cmdLineArgs1);
+
+        TEST_CASE(scope);   // handling different scopes
+
+        TEST_CASE(getErrorMessages);
     }
 
 
@@ -290,14 +321,16 @@ private:
 
     void arrayInfo()
     {
-        // Tokenize..
-        Tokenizer tokenizer;
-        std::istringstream istr("XY(1) const int a[2] = { 1, 2 };");
-        tokenizer.tokenize(istr, "test.cpp");
-
         // Clear the error buffer..
         errout.str("");
 
+        Settings settings;
+
+        // Tokenize..
+        Tokenizer tokenizer(&settings, this);
+        std::istringstream istr("XY(1) const int a[2] = { 1, 2 };");
+        tokenizer.tokenize(istr, "test.cpp");
+
         tokenizer.simplifySizeof();
 
         CheckBufferOverrun::ArrayInfo ai;
@@ -505,8 +538,7 @@ private:
               "    char str[5];\n"
               "    memclr( str );   // ERROR\n"
               "}\n");
-        TODO_ASSERT_EQUALS("[test.cpp:9] -> [test.cpp:3]: (error) Array index out of bounds\n", errout.str());
-        ASSERT_EQUALS("", errout.str());
+        ASSERT_EQUALS("[test.cpp:9] -> [test.cpp:3]: (error) Array 'str[5]' index 10 out of bounds\n", errout.str());
 
         check("static void memclr( int i, char *data )\n"
               "{\n"
@@ -518,8 +550,7 @@ private:
               "    char str[5];\n"
               "    memclr( 0, str );   // ERROR\n"
               "}\n");
-        TODO_ASSERT_EQUALS("[test.cpp:9] -> [test.cpp:3]: (error) Array index out of bounds\n", errout.str());
-        ASSERT_EQUALS("", errout.str());
+        ASSERT_EQUALS("[test.cpp:9] -> [test.cpp:3]: (error) Array 'str[5]' index 10 out of bounds\n", errout.str());
 
         check("static void memclr( int i, char *data )\n"
               "{\n"
@@ -531,8 +562,8 @@ private:
               "    char str[5];\n"
               "    memclr( 10, str );   // ERROR\n"
               "}\n");
-        TODO_ASSERT_EQUALS("[test.cpp:9] -> [test.cpp:3]: (possible error) Array index out of bounds\n", errout.str());
-        ASSERT_EQUALS("", errout.str());	// current result
+        TODO_ASSERT_EQUALS("[test.cpp:9] -> [test.cpp:3]: (possible error) Array index out of bounds\n",
+                           "", errout.str());
 
         // This is not an error
         check("static void memclr( char *data, int size )\n"
@@ -547,6 +578,20 @@ private:
               "    memclr( str, 5 );   // ERROR\n"
               "}\n");
         ASSERT_EQUALS("", errout.str());
+
+        // #2097..
+        check("void foo(int *p)\n"
+              "{\n"
+              "    --p;\n"
+              "    p[2] = 0;\n"
+              "}\n"
+              "\n"
+              "void bar()\n"
+              "{\n"
+              "    int p[3];\n"
+              "    foo(p+1);\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
     }
 
 
@@ -566,8 +611,8 @@ private:
               "{\n"
               "    memclr(abc->str);\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());
-        TODO_ASSERT_EQUALS("[test.cpp:13] -> [test.cpp:8]: (possible error) Array index out of bounds\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:13] -> [test.cpp:8]: (possible error) Array index out of bounds\n",
+                           "", errout.str());
     }
 
 
@@ -751,8 +796,7 @@ private:
               "        i+=1;\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());	// Catch changes
-        TODO_ASSERT_EQUALS("[test.cpp:6]: (error) Buffer overrun\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:6]: (error) Buffer overrun\n", "", errout.str());
     }
 
     void array_index_19()
@@ -957,8 +1001,7 @@ private:
               "    int *ip = &i[1];\n"
               "    ip[-10] = 1;\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());
-        TODO_ASSERT_EQUALS("[test.cpp:5]: (error) Array ip[-10] out of bounds\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:5]: (error) Array ip[-10] out of bounds\n", "", errout.str());
     }
 
     void array_index_29()
@@ -971,8 +1014,73 @@ private:
               "    int *ii = &i[-5];"
               "    ii[10] = 0;"
               "}\n");
+        TODO_ASSERT_EQUALS("[test.cpp:6]: (error) Array ii[10] out of bounds\n", "", errout.str());
+    }
+
+    void array_index_30()
+    {
+        // ticket #2086 - unknown type
+        check("void f() {\n"
+              "    UINT8 x[2];\n"
+              "    x[5] = 0;\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Array 'x[2]' index 5 out of bounds\n", errout.str());
+    }
+
+    void array_index_31()
+    {
+        // ticket #2120 - sub function, unknown type
+        check("struct s1 {\n"
+              "    unknown_type_t delay[3];\n"
+              "};\n"
+              "\n"
+              "void x(unknown_type_t *delay, const int *net) {\n"
+              "    delay[0] = 0;\n"
+              "}\n"
+              "\n"
+              "void y() {\n"
+              "    struct s1 obj;\n"
+              "    x(obj.delay, 123);\n"
+              "}\n");
         ASSERT_EQUALS("", errout.str());
-        TODO_ASSERT_EQUALS("[test.cpp:6]: (error) Array ii[10] out of bounds\n", errout.str());
+
+        check("struct s1 {\n"
+              "    unknown_type_t delay[3];\n"
+              "};\n"
+              "\n"
+              "void x(unknown_type_t *delay, const int *net) {\n"
+              "    delay[4] = 0;\n"
+              "}\n"
+              "\n"
+              "void y() {\n"
+              "    struct s1 obj;\n"
+              "    x(obj.delay, 123);\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:11] -> [test.cpp:6]: (error) Array 'obj . delay[3]' index 4 out of bounds\n", errout.str());
+
+        check("struct s1 {\n"
+              "    float a[0];\n"
+              "};\n"
+              "\n"
+              "void f() {\n"
+              "    struct s1 *obj;\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void array_index_32()
+    {
+        check("class X\n"
+              "{\n"
+              "    public:\n"
+              "    X()\n"
+              "    {\n"
+              "       m_x[0] = 0;\n"
+              "       m_x[1] = 0;\n"
+              "    }\n"
+              "    int m_x[1];\n"
+              "};\n");
+        TODO_ASSERT_EQUALS("[test.cpp:7]: (error) Array 'm_x[1]' index 1 out of bounds\n","", errout.str());
     }
 
     void array_index_multidim()
@@ -1038,7 +1146,7 @@ private:
               "  char a[10][10][10];\n"
               "  a[2*3][4*3][2] = 'a';\n"
               "}\n");
-        TODO_ASSERT_EQUALS("[test.cpp:4]: (error) Array 'a[10][10][10]' index a[6][12][2] out of bounds\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:4]: (error) Array 'a[10][10][10]' index a[6][12][2] out of bounds\n", "", errout.str());
 
         check("void f()\n"
               "{\n"
@@ -1106,8 +1214,7 @@ private:
               "  };\n"
               " }\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());
-        TODO_ASSERT_EQUALS("[test.cpp:12]: (error) Array index out of bounds\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:12]: (error) Array index out of bounds\n", "", errout.str());
     }
 
     void array_index_calculation()
@@ -1120,8 +1227,17 @@ private:
               "        data[(i-0)/2] = 0;\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());
-        TODO_ASSERT_EQUALS("[test.cpp:5]: (error) Array index out of bounds\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:5]: (error) Array 'data[8]' index 17 out of bounds\n", errout.str());
+
+        // #2199 - false negative: array out of bounds in loop when there is calculation
+        check("void f()\n"
+              "{\n"
+              "    char arr[5];\n"
+              "    for (int i = 0; i < 5; ++i) {\n"
+              "        arr[i + 7] = 0;\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:5]: (error) Array 'arr[5]' index 11 out of bounds\n", errout.str());
     }
 
     void array_index_negative()
@@ -1155,6 +1271,13 @@ private:
               "    if (p[-1]);\n"
               "}\n");
         ASSERT_EQUALS("", errout.str());
+
+        // ticket #1850
+        check("int f(const std::map<int, std::map<int,int> > &m)\n"
+              "{\n"
+              "    return m[0][-1];\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void array_index_for_decr()
@@ -1184,8 +1307,7 @@ private:
               "        val[i+1] = val[i];\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());	// catch changes
-        TODO_ASSERT_EQUALS("[test.cpp:5]: (error) Array 'val[5]' index -1 out of bounds\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:5]: (error) Array 'val[5]' index -1 out of bounds\n", "", errout.str());
     }
 
 
@@ -1206,6 +1328,58 @@ private:
     }
 
 
+    void array_index_for_break()
+    {
+        check("void f() {\n"
+              "    int a[2];\n"
+              "    for (int i = 0; i <= 2; ++i) {\n"
+              "        a[i] = 0;\n"
+              "        if (i==1) {\n"
+              "            break;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void array_index_for()
+    {
+        // Ticket #2370 - No false negative when there is no "break"
+        check("void f() {\n"
+              "    int a[10];\n"
+              "    for (int i = 0; i < 20; ++i) {\n"
+              "        if (i==1) {\n"
+              "        }\n"
+              "        a[i] = 0;\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:6]: (error) Buffer access out-of-bounds: a\n", errout.str());
+
+        // Ticket #2385 - No false positive
+        check("void f() {\n"
+              "    int a[10];\n"
+              "    for (int i = 0; i < 20; ++i) {\n"
+              "        if (i<10) {\n"
+              "        } else {\n"
+              "            a[i-10] = 0;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void array_index_for_neq()
+    {
+        // Ticket #2211 - for loop using != in the condition
+        check("void f() {\n"
+              "    int a[5];\n"
+              "    for (int i = 0; i != 10; ++i) {\n"
+              "        a[i] = 0;\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds: a\n", errout.str());
+    }
+
     void buffer_overrun_1()
     {
         check("void f()\n"
@@ -1279,8 +1453,7 @@ private:
               "char str[i];\n"
               "fread(str,sizeof(char),i+1,fd);\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());	// catch changes
-        TODO_ASSERT_EQUALS("[test.cpp:5]: (error) Buffer access out-of-bounds\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:5]: (error) Buffer access out-of-bounds: str\n", errout.str());
 
         check("void f(FILE* fd)\n"
               "{\n"
@@ -1303,8 +1476,7 @@ private:
               "char str[i];\n"
               "fwrite(str,sizeof(char),i+1,fd);\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());	// catch changes
-        TODO_ASSERT_EQUALS("[test.cpp:5]: (error) Buffer access out-of-bounds\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:5]: (error) Buffer access out-of-bounds: str\n", errout.str());
 
         check("void f(FILE* fd)\n"
               "{\n"
@@ -1313,6 +1485,33 @@ private:
               "}\n");
         ASSERT_EQUALS("", errout.str());
 
+        check("void f()\n"
+              "{\n"
+              "char str[3];\n"
+              "recv(s, str, 4, 0);\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds: str\n", errout.str());
+
+        check("void f()\n"
+              "{\n"
+              "char str[3];\n"
+              "recvfrom(s, str, 4, 0, 0x0, 0x0);\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds: str\n", errout.str());
+
+        check("void f()\n"
+              "{\n"
+              "char str[3];\n"
+              "send(s, str, 4, 0);\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds: str\n", errout.str());
+
+        check("void f()\n"
+              "{\n"
+              "char str[3];\n"
+              "sendto(s, str, 4, 0, 0x0, 0x0);\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds: str\n", errout.str());
     }
 
 
@@ -1374,8 +1573,7 @@ private:
               "  char s[3];\n"
               "  f1(s,3);\n"
               "}\n");
-        //ASSERT_EQUALS("[test.cpp:3]: (possible error) Buffer access out-of-bounds\n", errout.str());
-        ASSERT_EQUALS("", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:8] -> [test.cpp:3]: (error) Buffer access out-of-bounds\n", "", errout.str());
 
         check("void f1(char *s,int size)\n"
               "{\n"
@@ -1639,6 +1837,103 @@ private:
         ASSERT_EQUALS("[test.cpp:7]: (error) Buffer access out-of-bounds\n", errout.str());
     }
 
+    void buffer_overrun_16()
+    {
+        // unknown types
+        check("void f() {\n"
+              "    struct Foo foo[5];\n"
+              "    memset(foo, 0, sizeof(foo));\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"	// ticket #2093
+              "    gchar x[3];\n"
+              "    strcpy(x, \"12\");\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+
+        check("extern char a[10];\n"
+              "void f() {\n"
+              "    char b[25] = {0};\n"
+              "    std::memcpy(b, a, sizeof(a));\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void buffer_overrun_bailoutIfSwitch()
+    {
+        // No false positive
+        check("void f1(char *s) {\n"
+              "    if (x) s[100] = 0;\n"
+              "}\n"
+              "\n"
+              "void f2() {\n"
+              "    char a[10];\n"
+              "    f1(a);"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // No false positive
+        check("void f1(char *s) {\n"
+              "    if (x) return;\n"
+              "    s[100] = 0;\n"
+              "}\n"
+              "\n"
+              "void f2() {\n"
+              "    char a[10];\n"
+              "    f1(a);"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // No false negative
+        check("void f1(char *s) {\n"
+              "    if (x) { }\n"
+              "    s[100] = 0;\n"
+              "}\n"
+              "\n"
+              "void f2() {\n"
+              "    char a[10];\n"
+              "    f1(a);"
+              "}");
+        ASSERT_EQUALS("[test.cpp:8] -> [test.cpp:3]: (error) Array 'a[10]' index 100 out of bounds\n", errout.str());
+    }
+
+    void pointer_out_of_bounds_1()
+    {
+        check("void f() {\n"
+              "    char a[10];\n"
+              "    char *p = a + 100;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (portability) Undefined behaviour: pointer arithmetic result does not point into or just past the end of the array\n", errout.str());
+    }
+
+    void pointer_out_of_bounds_2()
+    {
+        check("void f() {\n"
+              "    char *p = malloc(10);\n"
+              "    p += 100;\n"
+              "    free(p);"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (portability) Undefined behaviour: pointer arithmetic result does not point into or just past the end of the buffer\n", errout.str());
+
+        check("void f() {\n"
+              "    char *p = malloc(10);\n"
+              "    p += 10;\n"
+              "    *p = 0;\n"
+              "    free(p);"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) p is out of bounds\n", errout.str());
+
+        check("void f() {\n"
+              "    char *p = malloc(10);\n"
+              "    p += 10;\n"
+              "    p = p - 1\n"
+              "    *p = 0;\n"
+              "    free(p);"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void sprintf1()
     {
         check("void f()\n"
@@ -1714,8 +2009,7 @@ private:
               "    char buf[3];\n"
               "    sprintf(buf, \"%s\", condition ? \"11\" : \"222\");\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());	// catch changes
-        TODO_ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds\n", "", errout.str());
     }
 
     void sprintf7()
@@ -1726,7 +2020,7 @@ private:
               "  struct Foo x;\n"
               "  sprintf(x.a, \"aa\");\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:5]: (error) Buffer access out-of-bounds\n", errout.str());
+        ASSERT_EQUALS("", errout.str());
     }
 
     void sprintf8()
@@ -1740,6 +2034,26 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void sprintf9()
+    {
+        check("void f()\n"
+              "{\n"
+              "  gchar str[3];\n"
+              "  sprintf(str, \"1\");\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void sprintf10()
+    {
+        check("void f()\n"
+              "{\n"
+              "  TString str = \"\";\n"
+              "  sprintf(str, \"1\");\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void snprintf1()
     {
         check("void f()\n"
@@ -1788,7 +2102,7 @@ private:
               "  struct Foo x;\n"
               "  snprintf(x.a, 2, \"aa\");\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:5]: (error) snprintf size is out of bounds\n", errout.str());
+        ASSERT_EQUALS("", errout.str());
     }
 
     void snprintf6()
@@ -1802,6 +2116,16 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void snprintf7()
+    {
+        check("void x() {\n"
+              "    const int nBezString = 1024;\n"
+              "    sal_Char pString[nBezString];\n"
+              "    snprintf(pString, nBezString, \"ab\");\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void strncat1()
     {
         check("void f(char *a, char *b)\n"
@@ -1810,7 +2134,7 @@ private:
               "    strncpy(str, a, 10);\n"
               "    strncat(str, b, 10);\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:5]: (style) Dangerous usage of strncat. Tip: the 3rd parameter means maximum number of characters to append\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:5]: (warning) Dangerous usage of strncat - 3rd parameter is the maximum number of characters to append.\n", errout.str());
     }
 
     void strncat2()
@@ -1820,7 +2144,7 @@ private:
               "    char str[5];\n"
               "    strncat(str, a, 5);\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) Dangerous usage of strncat. Tip: the 3rd parameter means maximum number of characters to append\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Dangerous usage of strncat - 3rd parameter is the maximum number of characters to append.\n", errout.str());
     }
 
     void strncat3()
@@ -1859,9 +2183,18 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void strcat3()
+    {
+        check("void f() {\n"
+              "  INT str[10];\n"
+              "  strcat(str, \"aa\");\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
 
     // memchr/memset/memcpy/etc
-    void memfunc()
+    void memfunc1()
     {
         check("struct S {\n"
               "    char a[5];\n"
@@ -1900,10 +2233,19 @@ private:
               "    char a[5], b[50];\n"
               "    memchr(a, b, 10);\n"
               "}\n");
-        TODO_ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds\n", errout.str());
-        ASSERT_EQUALS("", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:4]: (error) Buffer access out-of-bounds\n", "", errout.str());
     }
 
+    // ticket #2121 - buffer access out of bounds when using uint32_t
+    void memfunc2()
+    {
+        check("void f()\n"
+              "{\n"
+              "    unknown_type_t buf[4];\n"
+              "    memset(buf, 0, 100);\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
 
     void varid1()
     {
@@ -1954,6 +2296,14 @@ private:
               "}\n");
         ASSERT_EQUALS("[test.cpp:4]: (error) Array 's[10]' index 10 out of bounds\n", errout.str());
 
+        // ticket #1670 - false negative when using return
+        check("void f()\n"
+              "{\n"
+              "    char *s = new int[10];\n"
+              "    return s[10];\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Array 's[10]' index 10 out of bounds\n", errout.str());
+
         check("void foo()\n"
               "{\n"
               "char * buf = new char[8];\n"
@@ -2022,6 +2372,33 @@ private:
         ASSERT_EQUALS("[test.cpp:4]: (error) Array 'x[10]' index 10 out of bounds\n", errout.str());
     }
 
+    // statically allocated buffer
+    void alloc3()
+    {
+        check("void foo()\n"
+              "{\n"
+              "    const char *s = \"123\";\n"
+              "    s[10] = 0;\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Array 's[4]' index 10 out of bounds\n", errout.str());
+
+        check("void foo()\n"
+              "{\n"
+              "    char *s; s = \"\";\n"
+              "    s[10] = 0;\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Array 's[1]' index 10 out of bounds\n", errout.str());
+    }
+
+    void malloc_memset()
+    {
+        check("void f() {\n"
+              "    char *p = malloc(10);\n"
+              "    memset(p,0,100);\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Buffer access out-of-bounds\n", errout.str());
+    }
+
     void memset1()
     {
         check("void foo()\n"
@@ -2029,7 +2406,7 @@ private:
               "    char s[10];\n"
               "    memset(s, 5, '*');\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) The size argument is given as a char constant\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (warning) The size argument is given as a char constant\n", errout.str());
 
         check("void foo()\n"
               "{\n"
@@ -2097,7 +2474,7 @@ private:
         ASSERT_EQUALS(9, CheckBufferOverrun::countSprintfLength("%08ld", intAsParameter));
         ASSERT_EQUALS(6, CheckBufferOverrun::countSprintfLength("%.2d", intAsParameter));
         ASSERT_EQUALS(9, CheckBufferOverrun::countSprintfLength("%08.2d", intAsParameter));
-        TODO_ASSERT_EQUALS(5, CheckBufferOverrun::countSprintfLength("%x", intAsParameter));
+        TODO_ASSERT_EQUALS(5, 2, CheckBufferOverrun::countSprintfLength("%x", intAsParameter));
         ASSERT_EQUALS(5, CheckBufferOverrun::countSprintfLength("%4x", intAsParameter));
         ASSERT_EQUALS(6, CheckBufferOverrun::countSprintfLength("%5x", intAsParameter));
         ASSERT_EQUALS(5, CheckBufferOverrun::countSprintfLength("%.4x", intAsParameter));
@@ -2109,17 +2486,17 @@ private:
         Token floatTok(0);
         floatTok.str("1.12345f");
         floatAsParameter.push_back(&floatTok);
-        TODO_ASSERT_EQUALS(5, CheckBufferOverrun::countSprintfLength("%.2f", floatAsParameter));
+        TODO_ASSERT_EQUALS(5, 3, CheckBufferOverrun::countSprintfLength("%.2f", floatAsParameter));
         ASSERT_EQUALS(9, CheckBufferOverrun::countSprintfLength("%8.2f", floatAsParameter));
-        TODO_ASSERT_EQUALS(5, CheckBufferOverrun::countSprintfLength("%2.2f", floatAsParameter));
+        TODO_ASSERT_EQUALS(5, 3, CheckBufferOverrun::countSprintfLength("%2.2f", floatAsParameter));
 
         std::list<const Token*> floatAsParameter2;
         Token floatTok2(0);
         floatTok2.str("100.12345f");
         floatAsParameter2.push_back(&floatTok2);
-        TODO_ASSERT_EQUALS(7, CheckBufferOverrun::countSprintfLength("%2.2f", floatAsParameter2));
-        TODO_ASSERT_EQUALS(7, CheckBufferOverrun::countSprintfLength("%.2f", floatAsParameter));
-        TODO_ASSERT_EQUALS(7, CheckBufferOverrun::countSprintfLength("%4.2f", floatAsParameter));
+        TODO_ASSERT_EQUALS(7, 3, CheckBufferOverrun::countSprintfLength("%2.2f", floatAsParameter2));
+        TODO_ASSERT_EQUALS(7, 3, CheckBufferOverrun::countSprintfLength("%.2f", floatAsParameter));
+        TODO_ASSERT_EQUALS(7, 5, CheckBufferOverrun::countSprintfLength("%4.2f", floatAsParameter));
 
         std::list<const Token*> multipleParams;
         multipleParams.push_back(&strTok);
@@ -2205,7 +2582,7 @@ private:
               "    strncpy(baz, bar, sizeof(baz));\n"
               "    bar[99] = 0;\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) After a strncpy() the buffer should be zero-terminated\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (warning) After a strncpy() the buffer should be zero-terminated\n", errout.str());
 
         // Test with invalid code that there is no segfault
         check("char baz[100];\n"
@@ -2220,7 +2597,7 @@ private:
               "    foo(baz);\n"
               "    foo(baz);\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) After a strncpy() the buffer should be zero-terminated\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (warning) After a strncpy() the buffer should be zero-terminated\n", errout.str());
     }
 
     void terminateStrncpy2()
@@ -2232,7 +2609,23 @@ private:
               "    bar[99] = 0;\n"
               "    return baz;\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) After a strncpy() the buffer should be zero-terminated\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (warning) After a strncpy() the buffer should be zero-terminated\n", errout.str());
+    }
+
+    void terminateStrncpy3()
+    {
+        // Ticket #2170 - false positive
+        // The function bar is risky. But it might work that way intentionally.
+        check("char str[100];\n"
+              "\n"
+              "void foo(char *a) {\n"
+              "    strncpy(str, a, 100);\n"
+              "}\n"
+              "\n"
+              "void bar(char *p) {\n"
+              "    strncpy(p, str, 100);\n"
+              "}\n", false);
+        ASSERT_EQUALS("", errout.str());
     }
 
     void recursive_long_time()
@@ -2285,17 +2678,18 @@ private:
 
     void epcheck(const char code[])
     {
+        // Clear the error buffer..
+        errout.str("");
+
+        Settings settings;
+
         // Tokenize..
-        Tokenizer tokenizer;
+        Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
         tokenizer.tokenize(istr, "test.cpp");
         tokenizer.simplifyTokenList();
 
-        // Clear the error buffer..
-        errout.str("");
-
         // Check for buffer overruns..
-        Settings settings;
         CheckBufferOverrun checkBufferOverrun(&tokenizer, &settings, this);
         checkBufferOverrun.executionPaths();
     }
@@ -2336,6 +2730,31 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void executionPaths3()
+    {
+        epcheck("void f(char *VLtext)\n"
+                "{\n"
+                "    if ( x ) {\n"
+                "        return VLtext[0];\n"
+                "    } else {\n"
+                "        int wordlen = ab();\n"
+                "        VLtext[wordlen] = 0;\n"
+                "    }\n"
+                "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void executionPaths4()
+    {
+        // Ticket #2386 - Segmentation fault upon strange syntax
+        epcheck("void f() {\n"
+                "    switch ( x ) {\n"
+                "        case struct Tree : break;\n"
+                "    }\n"
+                "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void cmdLineArgs1()
     {
         check("int main(int argc, char* argv[])\n"
@@ -2445,6 +2864,28 @@ private:
 
         ASSERT_EQUALS("", errout.str());
     }
+
+    void scope()
+    {
+        check("class A {\n"
+              "private:\n"
+              "    struct X { char buf[10]; };\n"
+              "}\n"
+              "\n"
+              "void f()\n"
+              "{\n"
+              "    X x;\n"
+              "    x.buf[10] = 0;\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void getErrorMessages()
+    {
+        // Ticket #2292: segmentation fault when using --errorlist
+        CheckBufferOverrun c;
+        c.getErrorMessages(this, 0);
+    }
 };
 
 REGISTER_TEST(TestBufferOverrun)
