@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,19 +36,20 @@ private:
 
     void check(const char code[], bool showAll = false)
     {
+        // Clear the error buffer..
+        errout.str("");
+
+        Settings settings;
+        settings.inconclusive = showAll;
+        settings._checkCodingStyle = true;
+
         // Tokenize..
-        Tokenizer tokenizer;
+        Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
         tokenizer.tokenize(istr, "test.cpp");
         tokenizer.simplifyTokenList();
 
-        // Clear the error buffer..
-        errout.str("");
-
         // Check class constructors..
-        Settings settings;
-        settings.inconclusive = showAll;
-        settings._checkCodingStyle = true;
         CheckClass checkClass(&tokenizer, &settings, this);
         checkClass.constructors();
     }
@@ -65,6 +66,7 @@ private:
         TEST_CASE(initvar_operator_eq1);     // BUG 2190376
         TEST_CASE(initvar_operator_eq2);     // BUG 2190376
         TEST_CASE(initvar_operator_eq3);
+        TEST_CASE(initvar_operator_eq4);     // ticket #2204
         TEST_CASE(initvar_same_classname);      // BUG 2208157
         TEST_CASE(initvar_chained_assign);      // BUG 2270433
         TEST_CASE(initvar_2constructors);       // BUG 2270353
@@ -72,6 +74,10 @@ private:
         TEST_CASE(initvar_staticvar);
 
         TEST_CASE(initvar_private_constructor);     // BUG 2354171 - private constructor
+        TEST_CASE(initvar_copy_constructor); // ticket #1611
+        TEST_CASE(initvar_nested_constructor); // ticket #1375
+        TEST_CASE(initvar_nocopy1);            // ticket #2474
+        TEST_CASE(initvar_nocopy2);            // ticket #2484
 
         TEST_CASE(initvar_destructor);      // No variables need to be initialized in a destructor
 
@@ -93,7 +99,7 @@ private:
               "private:\n"
               "    int i;\n"
               "};\n");
-        ASSERT_EQUALS("[test.cpp:1]: (style) The class 'Fred' has no constructor. Member variables not initialized.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:1]: (style) The class 'Fred' does not have a constructor.\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
@@ -106,7 +112,7 @@ private:
               "private:\n"
               "    int i;\n"
               "};\n");
-        ASSERT_EQUALS("[test.cpp:1]: (style) The struct 'Fred' has no constructor. Member variables not initialized.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:1]: (style) The struct 'Fred' does not have a constructor.\n", errout.str());
     }
 
 
@@ -134,7 +140,7 @@ private:
               "    Fred() { }\n"
               "    int i;\n"
               "};\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) Member variable not initialized in the constructor 'Fred::i'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Member variable 'Fred::i' is not initialised in the constructor.\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
@@ -155,7 +161,7 @@ private:
               "    Fred() { }\n"
               "    int i;\n"
               "};\n");
-        ASSERT_EQUALS("[test.cpp:3]: (style) Member variable not initialized in the constructor 'Fred::i'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:3]: (warning) Member variable 'Fred::i' is not initialised in the constructor.\n", errout.str());
     }
 
 
@@ -189,7 +195,7 @@ private:
               "};\n"
               "Fred::Fred()\n"
               "{ }\n");
-        ASSERT_EQUALS("[test.cpp:7]: (style) Member variable not initialized in the constructor 'Fred::i'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::i' is not initialised in the constructor.\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
@@ -216,7 +222,7 @@ private:
               "};\n"
               "Fred::Fred()\n"
               "{ }\n");
-        ASSERT_EQUALS("[test.cpp:6]: (style) Member variable not initialized in the constructor 'Fred::i'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not initialised in the constructor.\n", errout.str());
     }
 
 
@@ -235,7 +241,7 @@ private:
               "{\n"
               "    i = _i;\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:8]: (style) Member variable not initialized in the constructor 'Fred::i'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:8]: (warning) Member variable 'Fred::i' is not initialised in the constructor.\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
@@ -249,7 +255,7 @@ private:
               "{\n"
               "    i = _i;\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:7]: (style) Member variable not initialized in the constructor 'Fred::i'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::i' is not initialised in the constructor.\n", errout.str());
     }
 
 
@@ -390,7 +396,7 @@ private:
               "    void operator=(const Fred &fred) { }\n"
               "    int i;\n"
               "};\n");
-        ASSERT_EQUALS("[test.cpp:5]: (style) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:5]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
@@ -398,7 +404,7 @@ private:
               "    void operator=(const Fred &fred) { }\n"
               "    int i;\n"
               "};\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='\n", errout.str());
     }
 
     void initvar_operator_eq3()
@@ -425,6 +431,69 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void initvar_operator_eq4()
+    {
+        check("class Fred\n"
+              "{\n"
+              "    int i;\n"
+              "public:\n"
+              "    Fred() : i(5) { }\n"
+              "    Fred & operator=(const Fred &fred)\n"
+              "    {\n"
+              "        if (&fred != this)\n"
+              "        {\n"
+              "        }\n"
+              "        return *this\n"
+              "    }\n"
+              "};\n");
+        ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='\n", errout.str());
+
+        check("class Fred\n"
+              "{\n"
+              "    int * i;\n"
+              "public:\n"
+              "    Fred() : i(NULL) { }\n"
+              "    Fred & operator=(const Fred &fred)\n"
+              "    {\n"
+              "        if (&fred != this)\n"
+              "        {\n"
+              "        }\n"
+              "        return *this\n"
+              "    }\n"
+              "};\n");
+        ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='\n", errout.str());
+
+        check("class Fred\n"
+              "{\n"
+              "    const int * i;\n"
+              "public:\n"
+              "    Fred() : i(NULL) { }\n"
+              "    Fred & operator=(const Fred &fred)\n"
+              "    {\n"
+              "        if (&fred != this)\n"
+              "        {\n"
+              "        }\n"
+              "        return *this\n"
+              "    }\n"
+              "};\n");
+        ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='\n", errout.str());
+
+        check("class Fred\n"
+              "{\n"
+              "    const int i;\n"
+              "public:\n"
+              "    Fred() : i(5) { }\n"
+              "    Fred & operator=(const Fred &fred)\n"
+              "    {\n"
+              "        if (&fred != this)\n"
+              "        {\n"
+              "        }\n"
+              "        return *this\n"
+              "    }\n"
+              "};\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void initvar_same_classname()
     {
         // Bug 2208157 - False positive: Uninitialized variable, same class name
@@ -488,6 +557,51 @@ private:
               "}\n");
 
         ASSERT_EQUALS("", errout.str());
+
+        check("class Foo {\n"
+              "    void func1()\n"
+              "    {\n"
+              "        struct Fred\n"
+              "        {\n"
+              "            int a;\n"
+              "            Fred() { a = 0; }\n"
+              "        };\n"
+              "    }\n"
+              "\n"
+              "    void func2()\n"
+              "    {\n"
+              "        struct Fred\n"
+              "        {\n"
+              "            int b;\n"
+              "            Fred() { b = 0; }\n"
+              "        };\n"
+              "    }\n"
+              "};\n");
+
+        ASSERT_EQUALS("", errout.str());
+
+        check("class Foo {\n"
+              "    void func1()\n"
+              "    {\n"
+              "        struct Fred\n"
+              "        {\n"
+              "            int a;\n"
+              "            Fred() { }\n"
+              "        };\n"
+              "    }\n"
+              "\n"
+              "    void func2()\n"
+              "    {\n"
+              "        struct Fred\n"
+              "        {\n"
+              "            int b;\n"
+              "            Fred() { }\n"
+              "        };\n"
+              "    }\n"
+              "};\n");
+
+        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::a' is not initialised in the constructor.\n"
+                      "[test.cpp:16]: (warning) Member variable 'Fred::b' is not initialised in the constructor.\n", errout.str());
     }
 
     void initvar_chained_assign()
@@ -620,7 +734,7 @@ private:
               "};\n"
               "Fred::Fred()\n"
               "{ }");
-        ASSERT_EQUALS("[test.cpp:7]: (style) Member variable not initialized in the constructor 'Fred::s'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::s' is not initialised in the constructor.\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
@@ -647,7 +761,7 @@ private:
               "};\n"
               "Fred::Fred()\n"
               "{ }");
-        ASSERT_EQUALS("[test.cpp:6]: (style) Member variable not initialized in the constructor 'Fred::s'\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::s' is not initialised in the constructor.\n", errout.str());
     }
 
 
@@ -677,6 +791,203 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void initvar_copy_constructor() // ticket #1611
+    {
+        check("class Fred\n"
+              "{\n"
+              "private:\n"
+              "    std::string var;\n"
+              "public:\n"
+              "    Fred() { };\n"
+              "    Fred(const Fred &) { };\n"
+              "};");
+        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::var' is not initialised in the constructor.\n", errout.str());
+
+        check("class Fred\n"
+              "{\n"
+              "private:\n"
+              "    std::string var;\n"
+              "public:\n"
+              "    Fred();\n"
+              "    Fred(const Fred &);\n"
+              "};\n"
+              "Fred::Fred() { };\n"
+              "Fred::Fred(const Fred &) { };\n");
+        ASSERT_EQUALS("[test.cpp:10]: (warning) Member variable 'Fred::var' is not initialised in the constructor.\n", errout.str());
+    }
+
+    void initvar_nested_constructor() // ticket #1375
+    {
+        check("class A {\n"
+              "public:\n"
+              "    A();\n"
+              "    struct B {\n"
+              "        B(int x);\n"
+              "        struct C {\n"
+              "            C(int y);\n"
+              "            struct D {\n"
+              "                int d;\n"
+              "                D(int z);\n"
+              "            };\n"
+              "            int c;\n"
+              "        };\n"
+              "        int b;\n"
+              "    };\n"
+              "private:\n"
+              "    int a;\n"
+              "    B b;\n"
+              "};\n"
+              "A::A(){}\n"
+              "A::B::B(int x){}\n"
+              "A::B::C::C(int y){}\n"
+              "A::B::C::D::D(int z){}\n");
+        ASSERT_EQUALS("[test.cpp:20]: (warning) Member variable 'A::a' is not initialised in the constructor.\n"
+                      "[test.cpp:20]: (warning) Member variable 'A::b' is not initialised in the constructor.\n"
+                      "[test.cpp:21]: (warning) Member variable 'B::b' is not initialised in the constructor.\n"
+                      "[test.cpp:22]: (warning) Member variable 'C::c' is not initialised in the constructor.\n"
+                      "[test.cpp:23]: (warning) Member variable 'D::d' is not initialised in the constructor.\n", errout.str());
+
+        check("class A {\n"
+              "public:\n"
+              "    A();\n"
+              "    struct B {\n"
+              "        B(int x);\n"
+              "        struct C {\n"
+              "            C(int y);\n"
+              "            struct D {\n"
+              "                D(const D &);\n"
+              "                int d;\n"
+              "            };\n"
+              "            int c;\n"
+              "        };\n"
+              "        int b;\n"
+              "    };\n"
+              "private:\n"
+              "    int a;\n"
+              "    B b;\n"
+              "};\n"
+              "A::A(){}\n"
+              "A::B::B(int x){}\n"
+              "A::B::C::C(int y){}\n"
+              "A::B::C::D::D(const A::B::C::D & d){}\n");
+        ASSERT_EQUALS("[test.cpp:20]: (warning) Member variable 'A::a' is not initialised in the constructor.\n"
+                      "[test.cpp:20]: (warning) Member variable 'A::b' is not initialised in the constructor.\n"
+                      "[test.cpp:21]: (warning) Member variable 'B::b' is not initialised in the constructor.\n"
+                      "[test.cpp:22]: (warning) Member variable 'C::c' is not initialised in the constructor.\n"
+                      "[test.cpp:23]: (warning) Member variable 'D::d' is not initialised in the constructor.\n", errout.str());
+
+        check("class A {\n"
+              "public:\n"
+              "    A();\n"
+              "    struct B {\n"
+              "        B(int x);\n"
+              "        struct C {\n"
+              "            C(int y);\n"
+              "            struct D {\n"
+              "                struct E { int e; };\n"
+              "                struct E d;\n"
+              "                D(const E &);\n"
+              "            };\n"
+              "            int c;\n"
+              "        };\n"
+              "        int b;\n"
+              "    };\n"
+              "private:\n"
+              "    int a;\n"
+              "    B b;\n"
+              "};\n"
+              "A::A(){}\n"
+              "A::B::B(int x){}\n"
+              "A::B::C::C(int y){}\n"
+              "A::B::C::D::D(const A::B::C::D::E & e){}\n");
+        ASSERT_EQUALS("[test.cpp:21]: (warning) Member variable 'A::a' is not initialised in the constructor.\n"
+                      "[test.cpp:21]: (warning) Member variable 'A::b' is not initialised in the constructor.\n"
+                      "[test.cpp:22]: (warning) Member variable 'B::b' is not initialised in the constructor.\n"
+                      "[test.cpp:23]: (warning) Member variable 'C::c' is not initialised in the constructor.\n"
+                      "[test.cpp:24]: (warning) Member variable 'D::d' is not initialised in the constructor.\n", errout.str());
+    }
+
+    void initvar_nocopy1() // ticket #2474
+    {
+        check("class B\n"
+              "{\n"
+              "    B (const B & Var);\n"
+              "    B & operator= (const B & Var);\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};\n");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class B\n"
+              "{\n"
+              "public:\n"
+              "    B (const B & Var);\n"
+              "    B & operator= (const B & Var);\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};\n");
+        ASSERT_EQUALS("[test.cpp:12]: (warning) Member variable 'A::m_SemVar' is not initialised in the constructor.\n"
+                      "[test.cpp:13]: (warning) Member variable 'A::m_SemVar' is not assigned a value in 'A::operator='\n", errout.str());
+
+        check("class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};\n");
+        ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'A::m_SemVar' is not initialised in the constructor.\n"
+                      "[test.cpp:7]: (warning) Member variable 'A::m_SemVar' is not assigned a value in 'A::operator='\n", errout.str());
+    }
+
+    void initvar_nocopy2() // ticket #2484
+    {
+        check("class B\n"
+              "{\n"
+              "    B (B & Var);\n"
+              "    B & operator= (const B & Var);\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};\n");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class B\n"
+              "{\n"
+              "public:\n"
+              "    B (B & Var);\n"
+              "    B & operator= (const B & Var);\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};\n");
+        ASSERT_EQUALS("[test.cpp:12]: (warning) Member variable 'A::m_SemVar' is not initialised in the constructor.\n"
+                      "[test.cpp:13]: (warning) Member variable 'A::m_SemVar' is not assigned a value in 'A::operator='\n", errout.str());
+    }
+
     void initvar_destructor()
     {
         check("class Fred\n"
@@ -706,7 +1017,7 @@ private:
               "\n"
               "void Fred::operator=(const Fred &f)\n"
               "{ }", true);
-        ASSERT_EQUALS("[test.cpp:13]: (style) Member variable 'Fred::ints' is not assigned a value in 'Fred::operator='\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:13]: (warning) Member variable 'Fred::ints' is not assigned a value in 'Fred::operator='\n", errout.str());
     }
 };
 
