@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,15 +48,20 @@ private:
 
         TEST_CASE(STLSize);
         TEST_CASE(STLSizeNoErr);
-        TEST_CASE(erase);
+        TEST_CASE(erase1);
         TEST_CASE(erase2);
         TEST_CASE(erase3);
         TEST_CASE(erase4);
+        TEST_CASE(erase5);
+        TEST_CASE(erase6);
         TEST_CASE(eraseBreak);
+        TEST_CASE(eraseContinue);
         TEST_CASE(eraseReturn1);
         TEST_CASE(eraseReturn2);
+        TEST_CASE(eraseReturn3);
         TEST_CASE(eraseGoto);
-        TEST_CASE(eraseAssign);
+        TEST_CASE(eraseAssign1);
+        TEST_CASE(eraseAssign2);
         TEST_CASE(eraseErase);
         TEST_CASE(eraseByValue);
 
@@ -72,6 +77,7 @@ private:
         TEST_CASE(pushback10);
 
         TEST_CASE(insert1);
+        TEST_CASE(insert2);
 
         TEST_CASE(invalidcode);
 
@@ -84,23 +90,39 @@ private:
         TEST_CASE(if_str_find);
 
         TEST_CASE(size1);
+
+        // Redundant conditions..
+        // if (ints.find(123) != ints.end()) ints.remove(123);
+        TEST_CASE(redundantCondition1);
+        TEST_CASE(redundantCondition2);
+
+        // missing inner comparison when incrementing iterator inside loop
+        TEST_CASE(missingInnerComparison1);
+        TEST_CASE(missingInnerComparison2);     // no FP when there is comparison
+        TEST_CASE(missingInnerComparison3);     // no FP when there is iterator shadowing
+        TEST_CASE(missingInnerComparison4);     // no FP when "break;" is used
+        TEST_CASE(missingInnerComparison5);     // Ticket #2154 - FP
+
+        // catch common problems when using the string::c_str() function
+        TEST_CASE(cstr);
     }
 
     void check(const std::string &code)
     {
+        // Clear the error buffer..
+        errout.str("");
+
+        Settings settings;
+        settings.inconclusive = true;
+        settings._checkCodingStyle = true;
+
         // Tokenize..
-        Tokenizer tokenizer;
+        Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code.c_str());
         tokenizer.tokenize(istr, "test.cpp");
         tokenizer.simplifyTokenList();
 
-        // Clear the error buffer..
-        errout.str("");
-
         // Check..
-        Settings settings;
-        settings.inconclusive = true;
-        settings._checkCodingStyle = true;
         CheckStl checkStl;
         checkStl.runSimplifiedChecks(&tokenizer, &settings, this);
     }
@@ -202,6 +224,22 @@ private:
               "}\n");
         ASSERT_EQUALS("", errout.str());
 
+        // Ticket #2481
+        check("void foo(std::vector<int> &r)\n"
+              "{\n"
+              "    std::vector<int>::iterator aI = r.begin();\n"
+              "    while(aI != r.end())\n"
+              "    {\n"
+              "        if (*aI == 0)\n"
+              "        {\n"
+              "            r.insert(aI, 42);\n"
+              "            break;\n"
+              "        }\n"
+              "        ++aI;\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+
         // Execution path checking..
         check("void foo(std::vector<int> &r, int c)\n"
               "{\n"
@@ -219,8 +257,7 @@ private:
               "        ++aI;\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("", errout.str());
-        TODO_ASSERT_EQUALS("[test.cpp:14] (error) After insert, the iterator 'aI' may be invalid", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:14] (error) After insert, the iterator 'aI' may be invalid", "", errout.str());
     }
 
     void iterator8()
@@ -340,7 +377,7 @@ private:
 
 
 
-    void erase()
+    void erase1()
     {
         check("void f()\n"
               "{\n"
@@ -349,7 +386,7 @@ private:
               "        foo.erase(it);\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:5]: (error) Dangerous iterator usage. After erase the iterator is invalid so dereferencing it or comparing it with another iterator is invalid.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:5]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
 
         check("for (it = foo.begin(); it != foo.end(); ++it)\n"
               "{\n"
@@ -359,8 +396,8 @@ private:
               "{\n"
               "    foo.erase(it);\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:3]: (error) Dangerous iterator usage. After erase the iterator is invalid so dereferencing it or comparing it with another iterator is invalid.\n"
-                      "[test.cpp:7]: (error) Dangerous iterator usage. After erase the iterator is invalid so dereferencing it or comparing it with another iterator is invalid.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:3]: (error) Dangerous iterator usage after erase()-method.\n"
+                      "[test.cpp:7]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
 
         check("void f(std::list<int> &ints)\n"
               "{\n"
@@ -370,13 +407,23 @@ private:
               "}\n");
         ASSERT_EQUALS("", errout.str());
 
+        // #2101
+        check("void f(vector< list<int> > &ints, unsigned int i)\n"
+              "{\n"
+              "    list<int>::iterator it;\n"
+              "    for(it = ints[i].begin(); it != ints[i].end(); it++) {\n"
+              "        if (*it % 2)\n"
+              "            it = ints[i].erase(it);\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void erase2()
     {
         check("static void f()\n"
               "{\n"
-              "    for (it = foo.begin(); it != foo.end(); it = next)\n"
+              "    for (iterator it = foo.begin(); it != foo.end(); it = next)\n"
               "    {\n"
               "        next = it;\n"
               "        next++;\n"
@@ -407,7 +454,7 @@ private:
               "        foo.erase(it);\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:6]: (error) Dangerous iterator usage. After erase the iterator is invalid so dereferencing it or comparing it with another iterator is invalid.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:6]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
 
         check("void f()\n"
               "{\n"
@@ -417,7 +464,7 @@ private:
               "        foo.erase(it);\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:6]: (error) Dangerous iterator usage. After erase the iterator is invalid so dereferencing it or comparing it with another iterator is invalid.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:6]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
 
         check("void f()\n"
               "{\n"
@@ -427,17 +474,86 @@ private:
               "        foo.erase(it);\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:6]: (error) Dangerous iterator usage. After erase the iterator is invalid so dereferencing it or comparing it with another iterator is invalid.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:6]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
+
+        check("void f()\n"
+              "{\n"
+              "    std::list<int>::iterator it = foo.begin();\n"
+              "    while (it != i2)\n"
+              "    {\n"
+              "        foo.erase(++it);\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:6]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
+    }
+
+    void erase5()
+    {
+        check("void f()\n"
+              "{\n"
+              "    std::list<int> foo;\n"
+              "    std::list<int>::iterator it;\n"
+              "    for (it = foo.begin(); it != foo.end(); ++it)\n"
+              "    {\n"
+              "        if (*it == 123)\n"
+              "            foo.erase(it);\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:8]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
+    }
+
+    void erase6()
+    {
+        check("void f() {\n"
+              "    std::vector<int> vec(3);\n"
+              "    std::vector<int>::iterator it;\n"
+              "    std::vector<int>::iterator itEnd = vec.end();\n"
+              "    for (it = vec.begin(); it != itEnd; it = vec.begin(), itEnd = vec.end())\n"
+              "    {\n"
+              "        vec.erase(it);\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void eraseBreak()
     {
         check("void f()\n"
               "{\n"
-              "    for (it = foo.begin(); it != foo.end(); ++it)\n"
+              "    for (iterator it = foo.begin(); it != foo.end(); ++it)\n"
               "    {\n"
               "        foo.erase(it);\n"
-              "        break;\n"
+              "        if (x)"
+              "            break;\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:5]: (error) Dangerous iterator usage after erase()-method.\n", errout.str());
+
+        check("void f()\n"
+              "{\n"
+              "    for (iterator it = foo.begin(); it != foo.end(); ++it)\n"
+              "    {\n"
+              "        if (x) {\n"
+              "            foo.erase(it);\n"
+              "            break;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void eraseContinue()
+    {
+        check("void f(std::vector<int> &ints)\n"
+              "{\n"
+              "    std::vector<int>::iterator it;\n"
+              "    std::vector<int>::iterator jt = ints.begin();\n"
+              "    for (it = ints.begin(); it != ints.end(); it = jt) {\n"
+              "        ++jt;\n"
+              "        if (*it == 1) {\n"
+              "            jt = ints.erase(it);\n"
+              "            continue;\n"
+              "        }\n"
               "    }\n"
               "}\n");
         ASSERT_EQUALS("", errout.str());
@@ -479,11 +595,47 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void eraseReturn3()
+    {
+        check("void f()\n"
+              "{\n"
+              "    std::vector<int> foo;\n"
+              "    std::vector<int>::iterator it;\n"
+              "    for (it = foo.begin(); it != foo.end(); ++it)\n"
+              "    {\n"
+              "        if (somecondition) {\n"
+              "            if (*it == 1)\n"
+              "                foo.erase(it);\n"
+              "            else\n"
+              "                *it = 0;\n"
+              "            return;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f()\n"
+              "{\n"
+              "    std::vector<int> foo;\n"
+              "    std::vector<int>::iterator it;\n"
+              "    for (it = foo.begin(); it != foo.end(); ++it)\n"
+              "    {\n"
+              "        if (a) {\n"
+              "            if (b)\n"
+              "                foo.erase(it);\n"
+              "            else\n"
+              "                *it = 0;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        TODO_ASSERT_EQUALS("[test.cpp:9]: (error) Dangerous iterator usage after erase()-method.\n", "", errout.str());
+    }
+
     void eraseGoto()
     {
         check("void f()\n"
               "{\n"
-              "    for (it = foo.begin(); it != foo.end(); ++it)\n"
+              "    for (iterator it = foo.begin(); it != foo.end(); ++it)\n"
               "    {\n"
               "        foo.erase(it);\n"
               "        goto abc;\n"
@@ -493,11 +645,11 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
-    void eraseAssign()
+    void eraseAssign1()
     {
         check("void f()\n"
               "{\n"
-              "    for (it = foo.begin(); it != foo.end(); ++it)\n"
+              "    for (iterator it = foo.begin(); it != foo.end(); ++it)\n"
               "    {\n"
               "        foo.erase(it);\n"
               "        it = foo.begin();\n"
@@ -506,6 +658,25 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void eraseAssign2()
+    {
+        check("void f(list<int> &ints)\n"
+              "{\n"
+              "    for (list<int>::iterator it = ints.begin(); it != ints.end();) {\n"
+              "        if (*it == 123) {\n"
+              "            list<int>::iterator copy = it;\n"
+              "            ++copy;\n"
+              "            ints.erase(it);\n"
+              "            it = copy;\n"
+              "        } else {\n"
+              "            it->second = 123;\n"
+              "            ++it;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void eraseErase()
     {
         check("void f(std::vector<ints> &ints)\n"
@@ -528,7 +699,15 @@ private:
               "        foo.erase(*it);\n"
               "    }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:6]: (error) Iterator 'it' becomes invalid when deleted by value from 'foo'\n", errout.str());
+        TODO_ASSERT_EQUALS("[test.cpp:6]: (error) Iterator 'it' becomes invalid when deleted by value from 'foo'\n", "", errout.str());
+
+        check("void f()\n"
+              "{\n"
+              "    std::set<int> foo;\n"
+              "    std::set<int> it = foo.begin();\n"
+              "    foo.erase(*it);\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
     }
 
 
@@ -739,7 +918,29 @@ private:
         ASSERT_EQUALS("[test.cpp:6]: (error) After insert, the iterator 'iter' may be invalid\n", errout.str());
     }
 
+    void insert2()
+    {
+        // Ticket: #2169
+        check("void f(std::vector<int> &vec) {\n"
+              "    for(std::vector<int>::iterator iter = vec.begin(); iter != vec.end(); ++iter)\n"
+              "    {\n"
+              "        vec.insert(iter, 0);\n"
+              "        break;\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
 
+        check("void f(std::vector<int> &vec) {\n"
+              "    for(std::vector<int>::iterator iter = vec.begin(); iter != vec.end(); ++iter)\n"
+              "    {\n"
+              "        if (*it == 0) {\n"
+              "            vec.insert(iter, 0);\n"
+              "            return;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
 
     void invalidcode()
     {
@@ -749,7 +950,8 @@ private:
                                 "    for ( \n"
                                 "}\n";
 
-        Tokenizer tokenizer(0, this);
+        Settings settings;
+        Tokenizer tokenizer(&settings, this);
         std::istringstream istr(src);
         ASSERT_EQUALS(false, tokenizer.tokenize(istr, "test.cpp"));
         ASSERT_EQUALS("[test.cpp:3]: (error) Invalid number of character (() when these macros are defined: ''.\n", errout.str());
@@ -775,7 +977,7 @@ private:
                   "        ;\n"
                   "}\n");
 
-            ASSERT_EQUALS("[test.cpp:4]: (error) " + stlCont[i]  + " range check should use != and not < since the order of the pointers isn't guaranteed\n", errout.str());
+            ASSERT_EQUALS("[test.cpp:4]: (error) Dangerous container iterator compare using < operator for " + stlCont[i] + "\n", errout.str());
         }
     }
 
@@ -820,7 +1022,7 @@ private:
               "{\n"
               "    if (s.find(12)) { }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:3]: (style) Suspicious condition. The result of find is an iterator, but it is not properly checked.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:3]: (warning) Suspicious condition. The result of find is an iterator, but it is not properly checked.\n", errout.str());
 
         // ok
         check("void f(std::set<int> s)\n"
@@ -839,7 +1041,7 @@ private:
               "{\n"
               "    if (std::find(a,b,c)) { }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:3]: (style) Suspicious condition. The result of find is an iterator, but it is not properly checked.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:3]: (warning) Suspicious condition. The result of find is an iterator, but it is not properly checked.\n", errout.str());
 
         // ok
         check("void f()\n"
@@ -856,7 +1058,7 @@ private:
               "{\n"
               "    if (s.find(\"abc\")) { }\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:3]: (style) Suspicious condition. string::find will return 0 if the string is found at position 0. If this is what you want to check then string::compare is a faster alternative because it doesn't scan through the string.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:3]: (warning) Suspicious checking of string::find() return value.\n", errout.str());
     }
 
 
@@ -867,28 +1069,28 @@ private:
               "    std::list<int> x;\n"
               "    if (x.size() == 0) {}\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) Use x.empty() instead of x.size() to guarantee fast code.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (performance) Possible inefficient checking for 'x' emptiness.\n", errout.str());
 
         check("void f()\n"
               "{\n"
               "    std::list<int> x;\n"
               "    if (x.size() != 0) {}\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) Use x.empty() instead of x.size() to guarantee fast code.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (performance) Possible inefficient checking for 'x' emptiness.\n", errout.str());
 
         check("void f()\n"
               "{\n"
               "    std::list<int> x;\n"
               "    if (x.size() > 0) {}\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) Use x.empty() instead of x.size() to guarantee fast code.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (performance) Possible inefficient checking for 'x' emptiness.\n", errout.str());
 
         check("void f()\n"
               "{\n"
               "    std::list<int> x;\n"
               "    if (x.size()) {}\n"
               "}\n");
-        ASSERT_EQUALS("[test.cpp:4]: (style) Use x.empty() instead of x.size() to guarantee fast code.\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (performance) Possible inefficient checking for 'x' emptiness.\n", errout.str());
 
         check("void f()\n"
               "{\n"
@@ -898,6 +1100,126 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void redundantCondition1()
+    {
+        check("void f()\n"
+              "{\n"
+              "    if (haystack.find(needle) != haystack.end())\n"
+              "        haystack.remove(needle);"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Redundant checking of STL container element.\n", errout.str());
+    }
+
+    void redundantCondition2()
+    {
+        check("void f()\n"
+              "{\n"
+              "    if (haystack.find(needle) != haystack.end())\n"
+              "    {\n"
+              "        haystack.remove(needle);\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Redundant checking of STL container element.\n", errout.str());
+    }
+
+    void missingInnerComparison1()
+    {
+        check("void f(std::set<int> &ints) {\n"
+              "    for (std::set<int>::iterator it = ints.begin(); it != ints.end(); ++it) {\n"
+              "        if (a) {\n"
+              "            it++;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Missing bounds check for extra iterator increment in loop.\n", errout.str());
+
+        check("void f(std::map<int,int> &ints) {\n"
+              "    for (std::map<int,int>::iterator it = ints.begin(); it != ints.end(); ++it) {\n"
+              "        ++it->second;\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void missingInnerComparison2()
+    {
+        check("void f(std::set<int> &ints) {\n"
+              "    for (std::set<int>::iterator it = ints.begin(); it != ints.end(); ++it) {\n"
+              "        if (a) {\n"
+              "            it++;\n"
+              "            if (it == ints.end())\n"
+              "                return;\n"
+              "        }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void missingInnerComparison3()
+    {
+        check("void f(std::set<int> &ints) {\n"
+              "    for (std::set<int>::iterator it = ints.begin(); it != ints.end(); ++it) {\n"
+              "        for (std::set<int>::iterator it = ints2.begin(); it != ints2.end(); ++it)\n"
+              "        { }\n"
+              "    }\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void missingInnerComparison4()
+    {
+        check("function f1(std::list<int> &l1) {\n"
+              "    for(std::list<int>::iterator i = l1.begin(); i != l1.end(); i++) {\n"
+              "        if (*i == 44) {\n"
+              "            l1.insert(++i, 55);\n"
+              "            break;\n"
+              "        }\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("function f1(std::list<int> &l1) {\n"
+              "    for(std::list<int>::iterator i = l1.begin(); i != l1.end(); i++) {\n"
+              "        if (*i == 44) {\n"
+              "            l1.insert(++i, 55);\n"
+              "            return;\n"
+              "        }\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void missingInnerComparison5()
+    {
+        check("void f() {\n"
+              "    for(it = map1.begin(); it != map1.end(); it++) {\n"
+              "        str[i++] = (*it).first;\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void cstr()
+    {
+        check("void f() {\n"
+              "    std::string errmsg;\n"
+              "    throw errmsg.c_str();\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Dangerous usage of c_str()\n", errout.str());
+
+        check("void f() {\n"
+              "    std::ostringstream errmsg;\n"
+              "    const char *c = errmsg.str().c_str();\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Dangerous usage of c_str()\n", errout.str());
+
+        check("std::string f();\n"
+              "\n"
+              "void foo() {\n"
+              "    const char *c = f().c_str();\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Dangerous usage of c_str()\n", errout.str());
+    }
 };
 
 REGISTER_TEST(TestStl)
