@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,9 +25,9 @@
 #include <vector>
 
 #if defined(_WIN32)
-#include "../lib/fileLister_win32.h"
+#include "../cli/fileLister_win32.h"
 #else // POSIX-style system
-#include "../lib/filelister_unix.h"
+#include "../cli/filelister_unix.h"
 #endif
 
 std::string objfile(std::string cppfile)
@@ -81,13 +81,13 @@ static void compilefiles(std::ostream &fout, const std::vector<std::string> &fil
         getDeps(files[i], depfiles);
         for (unsigned int dep = 0; dep < depfiles.size(); ++dep)
             fout << " " << depfiles[dep];
-        fout << "\n\t$(CXX) $(CXXFLAGS) " << args << " -c -o " << objfile(files[i]) << " " << files[i] << "\n\n";
+        fout << "\n\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) " << args << " -c -o " << objfile(files[i]) << " " << files[i] << "\n\n";
     }
 }
 
 static void getCppFiles(std::vector<std::string> &files, const std::string &path)
 {
-    getFileLister()->recursiveAddFiles(files, path, true);
+    getFileLister()->recursiveAddFiles(files, path);
     // only get *.cpp files..
     for (std::vector<std::string>::iterator it = files.begin(); it != files.end();)
     {
@@ -98,6 +98,16 @@ static void getCppFiles(std::vector<std::string> &files, const std::string &path
     }
 }
 
+
+static void makeConditionalVariable(std::ostream &os, const std::string &variable, const std::string &value)
+{
+    os << "ifndef " << variable << '\n'
+       << "    " << variable << '=' << value << '\n'
+       << "endif\n"
+       << "\n";
+}
+
+
 int main(int argc, char **argv)
 {
     const bool release(argc >= 2 && std::string(argv[1]) == "--release");
@@ -112,6 +122,15 @@ int main(int argc, char **argv)
     std::vector<std::string> testfiles;
     getCppFiles(testfiles, "test/");
 
+    if (libfiles.empty() && clifiles.empty() && testfiles.empty())
+    {
+        std::cerr << "No files found. Are you in the correct directory?" << std::endl;
+        return EXIT_FAILURE;
+    }
+
+    std::vector<std::string> externalfiles;
+    getCppFiles(externalfiles, "externals/");
+
 
     // QMAKE - lib/lib.pri
     {
@@ -119,21 +138,23 @@ int main(int argc, char **argv)
         if (fout1.is_open())
         {
             fout1 << "# no manual edits - this file is autogenerated by dmake\n\n";
-            fout1 << "HEADERS += $$PWD/check.h \\\n";
+            fout1 << "LIBS += -L../externals -lpcre\n";
+            fout1 << "INCLUDEPATH += ../externals\n";
+            fout1 << "HEADERS += $${BASEPATH}check.h \\\n";
             for (unsigned int i = 0; i < libfiles.size(); ++i)
             {
                 std::string fname(libfiles[i].substr(4));
                 if (fname.find(".cpp") == std::string::npos)
                     continue;   // shouldn't happen
                 fname.erase(fname.find(".cpp"));
-                fout1 << std::string(11, ' ') << "$$PWD/" << fname << ".h";
+                fout1 << std::string(11, ' ') << "$${BASEPATH}" << fname << ".h";
                 if (i < libfiles.size() - 1)
                     fout1 << " \\\n";
             }
             fout1 << "\n\nSOURCES += ";
             for (unsigned int i = 0; i < libfiles.size(); ++i)
             {
-                fout1 << "$$PWD/" << libfiles[i].substr(4);
+                fout1 << "$${BASEPATH}" << libfiles[i].substr(4);
                 if (i < libfiles.size() - 1)
                     fout1 << " \\\n" << std::string(11, ' ');
             }
@@ -142,42 +163,56 @@ int main(int argc, char **argv)
     }
 
 
-    std::ofstream fout("Makefile");
+    static const char makefile[] = "Makefile";
+    std::ofstream fout(makefile, std::ios_base::trunc);
+    if (!fout.is_open())
+    {
+        std::cerr << "An error occurred while trying to open "
+                  << makefile
+                  << ".\n";
+        return EXIT_FAILURE;
+    }
+
+    fout << "# This file is generated by tools/dmake, do not edit.\n\n";
 
     // Makefile settings..
     if (release)
     {
-        fout << "CXXFLAGS=-O2 -DNDEBUG -Wall\n";
+        makeConditionalVariable(fout, "CXXFLAGS", "-O2 -DNDEBUG -Wall");
     }
     else
     {
         // TODO: add more compiler warnings.
-        // -Wsign-conversion : generates too many compiler warnings currently
         // -Wlogical-op      : doesn't work on older GCC
+        // -Wconversion      : too many warnings
 
-        fout << "CXXFLAGS="
-             << "-Wall "
-             << "-Wextra "
-             << "-Wshadow "
-             << "-pedantic "
-             << "-Wno-long-long "
-             << "-Wfloat-equal "
-             << "-Wcast-qual "
-             << "-g -D_GLIBCXX_DEBUG\n";
+        // The _GLIBCXX_DEBUG doesn't work in cygwin
+        makeConditionalVariable(fout, "CXXFLAGS",
+                                "-Wall "
+                                "-Wextra "
+                                "-Wshadow "
+                                "-pedantic "
+                                "-Wno-long-long "
+                                "-Wfloat-equal "
+                                "-Wcast-qual "
+                                "-Wsign-conversion "
+//                                "-Wconversion "
+                                "-g");
     }
-    fout << "CXX=g++\n";
-    fout << "BIN=${DESTDIR}/usr/bin\n\n";
-    fout << "# For 'make man': sudo apt-get install xsltproc docbook-xsl docbook-xml\n";
+    makeConditionalVariable(fout, "CXX", "g++");
+    makeConditionalVariable(fout, "PREFIX", "/usr");
+    fout << "BIN=$(DESTDIR)$(PREFIX)/bin\n\n";
+    fout << "# For 'make man': sudo apt-get install xsltproc docbook-xsl docbook-xml on Linux\n";
     fout << "DB2MAN=/usr/share/sgml/docbook/stylesheet/xsl/nwalsh/manpages/docbook.xsl\n";
     fout << "XP=xsltproc -''-nonet -''-param man.charmap.use.subset \"0\"\n";
     fout << "MAN_SOURCE=man/cppcheck.1.xml\n\n";
 
     fout << "\n###### Object Files\n\n";
-    fout << "LIBOBJ =     " << objfile(libfiles[0]);
+    fout << "LIBOBJ =      " << objfile(libfiles[0]);
     for (unsigned int i = 1; i < libfiles.size(); ++i)
         fout << " \\" << std::endl << std::string(14, ' ') << objfile(libfiles[i]);
     fout << "\n\n";
-    fout << "CLIOBJ =     " << objfile(clifiles[0]);
+    fout << "CLIOBJ =      " << objfile(clifiles[0]);
     for (unsigned int i = 1; i < clifiles.size(); ++i)
         fout << " \\" << std::endl << std::string(14, ' ') << objfile(clifiles[i]);
     fout << "\n\n";
@@ -185,24 +220,35 @@ int main(int argc, char **argv)
     for (unsigned int i = 1; i < testfiles.size(); ++i)
         fout << " \\" << std::endl << std::string(14, ' ') << objfile(testfiles[i]);
     fout << "\n\n";
+    fout << "EXTOBJ =      " << objfile(externalfiles[0]);
+    for (unsigned int i = 1; i < externalfiles.size(); ++i)
+        fout << " \\" << std::endl << std::string(14, ' ') << objfile(externalfiles[i]);
+    fout << "\n\n";
 
 
     fout << "\n###### Targets\n\n";
-    fout << "cppcheck:\t$(LIBOBJ)\t$(CLIOBJ)\n";
-    fout << "\t$(CXX) $(CXXFLAGS) -o cppcheck $(CLIOBJ) $(LIBOBJ) $(LDFLAGS)\n\n";
-    fout << "all:\tcppcheck\ttestrunner\ttools\n\n";
-    fout << "testrunner:\t$(TESTOBJ)\t$(LIBOBJ)\tcli/threadexecutor.o\n";
-    fout << "\t$(CXX) $(CXXFLAGS) -o testrunner $(TESTOBJ) $(LIBOBJ) cli/threadexecutor.o $(LDFLAGS)\n\n";
+    fout << "cppcheck: $(LIBOBJ) $(CLIOBJ) $(EXTOBJ)\n";
+    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o cppcheck $(CLIOBJ) $(LIBOBJ) $(EXTOBJ) -lpcre $(LDFLAGS)\n\n";
+    fout << "all:\tcppcheck testrunner\n\n";
+    fout << "testrunner: $(TESTOBJ) $(LIBOBJ) $(EXTOBJ) cli/threadexecutor.o cli/cmdlineparser.o cli/cppcheckexecutor.o cli/filelister.o cli/filelister_unix.o cli/pathmatch.o\n";
+    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o testrunner $(TESTOBJ) $(LIBOBJ) $(EXTOBJ) -lpcre cli/threadexecutor.o cli/cmdlineparser.o cli/filelister.o cli/filelister_unix.o cli/pathmatch.o $(LDFLAGS)\n\n";
     fout << "test:\tall\n";
     fout << "\t./testrunner\n\n";
+    fout << "check:\tall\n";
+    fout << "\t./testrunner -g -q\n\n";
+    fout << "dmake:\ttools/dmake.cpp\n";
+    fout << "\t$(CXX) -o dmake tools/dmake.cpp cli/filelister*.cpp lib/path.cpp -Ilib\n\n";
     fout << "clean:\n";
 #ifdef _WIN32
     fout << "\tdel lib\*.o\n\tdel cli\*.o\n\tdel test\*.o\n\tdel *.exe\n";
 #else
-    fout << "\trm -f lib/*.o cli/*.o test/*.o testrunner cppcheck\n\n";
-    fout << "man:\t$(MAN_SOURCE)\n";
+    fout << "\trm -f lib/*.o cli/*.o test/*.o externals/tinyxml/*.o testrunner cppcheck cppcheck.1\n\n";
+    fout << "man:\tman/cppcheck.1\n\n";
+    fout << "man/cppcheck.1:\t$(MAN_SOURCE)\n\n";
     fout << "\t$(XP) $(DB2MAN) $(MAN_SOURCE)\n\n";
-    fout << "install:\tcppcheck\n";
+    fout << "tags:\n";
+    fout << "\tctags -R --exclude=doxyoutput .\n\n";
+    fout << "install: cppcheck\n";
     fout << "\tinstall -d ${BIN}\n";
     fout << "\tinstall cppcheck ${BIN}\n\n";
 #endif
@@ -210,10 +256,9 @@ int main(int argc, char **argv)
     fout << "\n###### Build\n\n";
 
     compilefiles(fout, libfiles, "-Ilib");
-    compilefiles(fout, clifiles, "-Ilib");
-    compilefiles(fout, testfiles, "-Ilib -Icli");
+    compilefiles(fout, clifiles, "-Ilib -Iexternals");
+    compilefiles(fout, testfiles, "-Ilib -Icli -Iexternals");
 
     return 0;
 }
 
-
